# -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Piston handlers for the  Apps Directory API."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'ServerStatusHandler',
    'ListMachinesHandler',
    'MachineHandler',
    'MetadataHandler',
    'PackagesHandler',
]

from django.http import (
    HttpResponse,
    HttpResponseBadRequest,
    HttpResponseNotFound,
)
from piston.handler import BaseHandler

from webcatalog.models import (
    Application,
    Machine,
)
from .forms import MachineCreateUpdateForm, MachineUpdatePackagesForm

# django-piston (outdated bit of software) expects those attributes to
# determine if the resonse content is iterator or not. Here we never
# use that possibliblit, so hardcoding the attribute.
HttpResponse._is_string = True
HttpResponseNotFound._is_string = True
HttpResponseBadRequest._is_string = True


class ServerStatusHandler(BaseHandler):
    allowed_methods = ('GET',)

    def read(self, request):
        return "ok"


class ListMachinesHandler(BaseHandler):
    allowed_methods = ('GET',)
    model = Machine
    fields = ('uuid', 'hostname', 'logo_checksum', 'packages_checksum')

    def read(self, request):
        result = Machine.objects.filter(owner=request.user)
        return result.defer('package_list')


class MachineHandler(BaseHandler):
    allowed_methods = ('GET', 'POST', 'DELETE')
    model = Machine
    fields = ('uuid', 'hostname', 'logo_checksum', 'packages_checksum')

    def read(self, request, uuid):
        try:
            return Machine.objects.get(owner=request.user, uuid=uuid)
        except Machine.DoesNotExist:
            return HttpResponseNotFound('Invalid machine UUID')

    def create(self, request, uuid):
        if not hasattr(request, 'data'):
            return HttpResponseBadRequest("Unable to deserialize request")
        form = MachineCreateUpdateForm(request.data)
        if form.is_valid():
            # Make this call work both for updating and creating machines
            instance, created = Machine.objects.get_or_create(
                owner=request.user, uuid=uuid)
            form = MachineCreateUpdateForm(request.data, instance=instance)
            return form.save()
        else:
            errors = dict((k, map(unicode, v))
                          for (k, v) in form.errors.items())
            result = {'status': 'error', 'errors': errors}
            return result

    def delete(self, request, uuid):
        instances = Machine.objects.filter(owner=request.user, uuid=uuid)
        if instances.count() > 0:
            instances.delete()
            return HttpResponse(status=204)
        else:
            return HttpResponseNotFound('Invalid machine UUID')


class PackagesHandler(BaseHandler):
    allowed_methods = ('GET', 'POST',)

    def read(self, request, uuid):
        try:
            instance = Machine.objects.get(owner=request.user, uuid=uuid)
            return instance.package_list
        except Machine.DoesNotExist:
            return HttpResponseNotFound('Invalid machine UUID')

    def create(self, request, uuid):
        try:
            instance = Machine.objects.get(owner=request.user, uuid=uuid)
        except Machine.DoesNotExist:
            return HttpResponseNotFound('Invalid machine UUID')
        if not hasattr(request, 'data'):
            return HttpResponseBadRequest("Unable to deserialize request")
        form = MachineUpdatePackagesForm(request.data, instance=instance)
        if form.is_valid():
            form.save()
            return 'Success'
        else:
            errors = dict((k, map(unicode, v))
                          for (k, v) in form.errors.items())
            result = {'status': 'error', 'errors': errors}
            return result


class MetadataHandler(BaseHandler):
    allowed_methods = ('GET',)
    model = Application

    fields = ('package_name', 'categories')

    def read(self, request, distroseries=None):
        apps = Application.objects.all()
        if distroseries:
            apps = apps.filter(distroseries__code_name=distroseries)
        else:
            apps = apps.filter(is_latest=True)
        return apps
