#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Department filters."""

from __future__ import absolute_import

import re

from django.db import models

__metaclass__ = type
__all__ = [
    'department_filters',
]


def category_filter(categories_set):
    """Returns a filter func that checks an app for certain categories."""
    def filter_func(app):
        return bool(app.categories_set.intersection(categories_set))
    return filter_func


def package_name_filter(name_regex):
    """Returns a filter func that checks if an app's name matches a regex"""
    def filter_func(app):
        return re.match(name_regex, app.package_name) is not None
    return filter_func


def section_filter(sections):
    """Returns a filter that checks if an app is in certain sections."""
    def filter_func(app):
        return app.section in sections
    return filter_func

# Taken from https://wiki.ubuntu.com/SoftwareCenter#line-793

department_filters = {
    'Accessories': [category_filter(set(['Utility', 'System']))],
    'Books & Magazines': [],
    'Education': [category_filter(set(['Education']))],
    'Fonts': [package_name_filter(r'[to]tf-.*')],
    'Games': [category_filter(set(['Game', 'Sports']))],
    'Board Games': [category_filter(set(['BoardGame']))],
    'Card Games': [category_filter(set(['CardGame']))],
    'Puzzles': [category_filter(set(['LogicGame']))],
    'Role-Playing': [category_filter(set(['RolePlaying']))],
    'Sports': [category_filter(set(['SportsGame']))],
    'Graphics': [category_filter(set(['Graphics']))],
    '3D': [category_filter(set(['3DGraphics']))],
    'Drawing': [category_filter(set(['VectorGraphics']))],
    'Painting': [category_filter(set(['RasterGraphics']))],
    'Photography': [category_filter(set(['Photography']))],
    'Publishing': [category_filter(set(['Publishing']))],
    'Scanning & OCR': [category_filter(set(['Scanning', 'OCR']))],
    'Viewers': [category_filter(set(['Viewer']))],
    'Internet': [category_filter(set(['Network']))],
    'Chat': [category_filter(set(['InstantMessaging', 'IRCClient']))],
    'File Sharing': [category_filter(set(['FileTransfer']))],
    'Mail': [category_filter(set(['Email']))],
    'Web Browsers': [category_filter(set(['WebBrowser']))],
    'Office': [category_filter(set(['Office']))],
    'Science & Engineering': [category_filter(set(['Science'])),
                              section_filter(['science'])],
    'Astronomy': [category_filter(set(['Astronomy']))],
    'Biology': [category_filter(set(['Biology']))],
    'Chemistry': [category_filter(set(['Chemistry']))],
    'Computing & Robotics': [category_filter(set(['ArtificialIntelligence',
                                                  'ComputerScience',
                                                  'Robotics']))],
    'Electronics': [category_filter(set(['Electronics']))],
    'Engineering': [category_filter(set(['Engineering']))],
    'Geography': [category_filter(set(['Geography']))],
    'Geology': [category_filter(set(['Geology', 'Geoscience']))],
    'Mathematics': [category_filter(set(['DataVisualization', 'Math',
                                         'NumericalAnalysis'])),
                    section_filter(['math', 'gnu-r'])],
    'Physics': [category_filter(set(['Electricity', 'Physics']))],
    'Sound & Video': [category_filter(set(['AudioVideo', 'Audio', 'Video']))],
    'Themes & Tweaks': [category_filter(set(['Settings']))],
    'Universal Access': [category_filter(set(['Accessibility']))],
    'Developer Tools': [category_filter(set(['Development'])),
                        section_filter(['devel'])],
    'Debugging': [category_filter(set(['Debugger']))],
    'Graphic Interface Design': [category_filter(set(['GUIDesigner']))],
    'Haskell': [section_filter(['haskell'])],
    'IDEs': [category_filter(set(['IDE']))],
    'Java': [section_filter(['java'])],
    'Libraries': [section_filter(['libdevel'])],
    'Lisp': [section_filter(['lisp'])],
    'Localization': [category_filter(set(['Translation']))],
    'Mono/CLI': [section_filter(['cli-mono'])],
    'OCaml': [section_filter(['ocaml'])],
    'Perl': [section_filter(['perl'])],
    'Profiling': [category_filter(set(['Profiling']))],
    'Python': [section_filter(['python'])],
    'Ruby': [section_filter(['ruby'])],
    'Version Control': [category_filter(set(['RevisionControl'])),
                        section_filter(['vcs'])],
    'Web Development': [category_filter(set(['WebDevelopment']))],
}
