#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Forms used by the Apps Directory."""

from __future__ import absolute_import

import apt
import json
from ConfigParser import ConfigParser
from StringIO import StringIO

from django import forms
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.validators import URLValidator
from django.template.loader import render_to_string

from webcatalog.hw import get_hw_short_description
from webcatalog.models import Application

__metaclass__ = type
__all__ = [
    'ArchiveApplicationForm',
    'desktop_field_mappings',
]

desktop_field_mappings = {
    'x-appinstall-package': 'package_name',
    'x-appinstall-popcon': 'popcon',
    'x-appinstall-channel': 'channel',
    'x-appinstall-screenshot-url': 'screenshot_url',
    'x-appinstall-architectures': 'architectures',
    'x-appinstall-keywords': 'keywords',
    'x-appinstall-description': 'description',
    'x-appinstall-section': 'section',
    'type': 'app_type',
    'icon': 'icon_name',
}


class ArchiveApplicationForm(forms.ModelForm):

    screenshot_url = forms.URLField(required=False)

    class Meta:
        model = Application
        exclude = ('imported_from_sca', 'archive_id', 'price', 'version')

    @classmethod
    def get_form_from_desktop_data(cls, str_data, distroseries):
        parser = ConfigParser()
        parser.readfp(StringIO(str_data))
        data = dict(parser.items('Desktop Entry'))
        for desktop_key, app_key in desktop_field_mappings.items():
            if desktop_key in data:
                data[app_key] = data[desktop_key]
                del(data[desktop_key])
        try:
            instance = Application.objects.get(
                package_name=data['package_name'], distroseries=distroseries)
        except Application.DoesNotExist:
            instance = None
        data['distroseries'] = distroseries.id
        return cls(data=data, instance=instance)

    def clean(self):
        cleaned_data = super(ArchiveApplicationForm, self).clean()

        if self.instance.id:
            if self.instance.imported_from_sca:
                raise forms.ValidationError(
                    u"Attempt to overwrite an sca-imported app.")

            # For all fields, ensure we don't overwrite valid data on an
            # associated instance with blank fields.
            for key, value in cleaned_data.items():
                if not value:
                    instance_value = getattr(self.instance, key, False)
                    if hasattr(instance_value, 'all'):
                        instance_value = instance_value.all()
                    cleaned_data[key] = instance_value or value

        return cleaned_data

    def save(self, commit=True):
        app = super(ArchiveApplicationForm, self).save(commit=commit)
        screenshot_url = self.cleaned_data['screenshot_url']
        if commit and screenshot_url:
            if screenshot_url not in app.screenshots:
                app.applicationmedia_set.create(
                    media_type='screenshot', url=screenshot_url)
        return app


class MultiURLField(forms.Field):
    def to_python(self, value):
        "Normalize data to a list of strings."
        # Return an empty list if no input was given.
        if not value:
            return []
        return value

    def validate(self, value):
        "Check if value consists only of valid urls."
        # Use the parent's handling of required fields, etc.
        super(MultiURLField, self).validate(value)

        for email in value:
            validator = URLValidator()
            validator(email)


class SCAApplicationForm(forms.ModelForm):
    screenshot_urls = MultiURLField(required=False)
    video_embedded_html_urls = MultiURLField(required=False)

    def clean_archive_id(self):
        archive_id = self.cleaned_data.get('archive_id')
        if not archive_id:
            return None
        return archive_id

    class Meta:
        model = Application
        exclude = ('section', 'popcon')

    @classmethod
    def from_api_data(cls, app_data, distroseries):
        app_data = app_data.copy()
        app_data['distroseries'] = distroseries.id
        app_data['imported_from_sca'] = True
        if 'description' in app_data:
            tagline, _, description = app_data['description'].partition('\n')
            app_data['comment'] = tagline
            app_data['description'] = description
        if 'debtags' in app_data and app_data['debtags']:
            app_data['debtags'] = json.dumps([get_hw_short_description(x)
                                              for x in app_data['debtags']])

        try:
            instance = Application.objects.get(
                package_name=app_data['package_name'],
                distroseries=distroseries)
        except Application.DoesNotExist:
            instance = None

        return cls(data=app_data, instance=instance)

    def clean_version(self):
        value = self.cleaned_data['version']
        return apt.apt_pkg.upstream_version(value)

    def save(self, commit=True):
        app = super(SCAApplicationForm, self).save(commit=commit)
        if commit:
            self.save_media_urls()
        return app

    def save_media_urls(self):
        for screenshot_url in self.cleaned_data['screenshot_urls']:
            self.instance.applicationmedia_set.get_or_create(
                url=screenshot_url, media_type='screenshot')

        for video_url in self.cleaned_data['video_embedded_html_urls']:
            self.instance.applicationmedia_set.get_or_create(
                url=video_url, media_type='video')


class EmailDownloadLinkForm(forms.Form):
    email = forms.EmailField()

    def send_email(self, app_name, link):
        subject = 'Link for {0}'.format(app_name)
        context = {'name': app_name, 'link': link}
        html = render_to_string('webcatalog/email_download_link.html',
                                context)
        text = render_to_string('webcatalog/email_download_link.txt',
                                context)
        sender = ("Ubuntu Application Directory <%s>" %
                  settings.NOREPLY_FROM_ADDRESS)
        recipient = [self.cleaned_data['email']]
        message = EmailMultiAlternatives(subject, text, sender, recipient)
        message.attach_alternative(html, 'text/html')
        message.send()
