#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Management command to update the 'is_latest' bit on *all* Applications."""

from __future__ import (
    absolute_import,
)

from django.core.management.base import BaseCommand

from webcatalog.models import Application

__metaclass__ = type
__all__ = [
    'Command',
]

BATCH_SIZE = 200


class Command(BaseCommand):
    help = "Updates the 'is_latest' bit on all Applications."

    def get_versions_for_app(self, packages):
        """Generate lists of pairs (app_id, prerelease).

        One list is generated for each package name, with one pair for each
        Application with that package_name.
        """
        versions = []
        current_package_name = None
        for app_id, app_package_name, prerelease in packages:
            if current_package_name != app_package_name:
                if versions:
                    yield versions
                    versions = []
                current_package_name = app_package_name
            versions.append((app_id, prerelease))
        if versions:
            yield versions

    def handle(self, *args, **options):
        """A naive implementation of this command could be:

        for p in Application.objects.values_list('package_name',
            flat=True).distinct(): Application.objects.check_latest(p)

        This command should do exactly that, only much faster.
        """
        # 1. Clear all is_latest bits
        Application.objects.all().update(is_latest=False)

        # 2. Update is_latest in batches of BATCH_SIZE.
        packages = Application.objects.values_list(
            'id', 'package_name', 'distroseries__prerelease')
        packages = packages.order_by('package_name', '-distroseries__version')
        current_package_name = None
        latest_app_ids = []
        for versions in self.get_versions_for_app(packages):
            latest_app_ids.append(Application.objects.get_latest_id(versions))
            if len(latest_app_ids) >= BATCH_SIZE:
                Application.objects.filter(id__in=latest_app_ids).update(
                    is_latest=True)
                latest_app_ids = []
        if latest_app_ids:
            Application.objects.filter(id__in=latest_app_ids).update(
                is_latest=True)
