#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Management command to clean up sessions and OAuth nonce tables."""

import time
from django.core.management.base import (
    CommandError,
    LabelCommand,
    make_option,
)
from django.db import connection

try:
    import psycopg2
    psycopg2
    psycopg2_available = True
except ImportError:
    psycopg2_available = False


class DjangoSessionCleaner(object):
    def __init__(self, cmd):
        self.cmd = cmd

    def declare_expired_keys_cursor(self, cur):
        self.cmd.output("Opening cursor\n", 1)
        # We order by expire_date to force the index to be used.
        cur.execute("CLOSE ALL")
        cur.execute("""
            DECLARE _django_session_clean NO SCROLL CURSOR WITH HOLD FOR
            SELECT session_key FROM django_session
            WHERE expire_date < CURRENT_TIMESTAMP
            ORDER BY expire_date
            """)

    def remove_batch(self, cur, batch_size):
        cur.execute("FETCH %s FROM _django_session_clean", [batch_size])
        session_keys = [session_key for session_key, in cur.fetchall()]
        if not session_keys:
            return 0
        cur.execute("""
            DELETE FROM django_session
            WHERE
                expire_date < CURRENT_TIMESTAMP
                AND session_key = ANY(%s)
            """, (session_keys,))
        # len() is better here than querying the cursor for rows actually
        # removed, as we don't want to terminate the script early in the
        # unlikely case all the sessions in out batch were reused between
        # now and when the expired keys cursor was opened.
        return len(session_keys)


class OAuthNonceCleaner(object):
    def __init__(self, cmd):
        self.cmd = cmd

    def declare_expired_keys_cursor(self, cur):
        self.cmd.output("Opening cursor\n", 1)
        # We order by created_at to force the index to be used.
        # Use a 5 hour clock skew as defined in openid.store.nonce
        cur.execute("CLOSE ALL")
        cur.execute("""
            DECLARE _oauth_nonce_clean NO SCROLL CURSOR WITH HOLD FOR
            SELECT id FROM webcatalog_nonce
            WHERE created_at < (CURRENT_TIMESTAMP - INTERVAL '5' HOUR)
            ORDER BY created_at
            """)

    def remove_batch(self, cur, batch_size):
        cur.execute("FETCH %s FROM _oauth_nonce_clean", [batch_size])
        nonce_ids = [nonce_id for nonce_id, in cur.fetchall()]
        if not nonce_ids:
            return 0
        cur.execute("""
            DELETE FROM webcatalog_nonce
            WHERE id = ANY(%s)
            """, (nonce_ids,))
        # len() is better here than querying the cursor for rows actually
        # removed, as we don't want to terminate the script early in the
        # unlikely case all the sessions in out batch were reused between
        # now and when the expired keys cursor was opened.
        return len(nonce_ids)


CLEANERS = {
    'django_session': DjangoSessionCleaner,
    'webcatalog_nonce': OAuthNonceCleaner,
}


class Command(LabelCommand):
    help = "Cleans the Django session and OAuth nonce tables."

    option_list = LabelCommand.option_list + (
        make_option(
            '--cursor-life', default=3600, dest="cursor_secs",
            action='store',
            help="Hold a cursor open a maximum of SECS seconds"),
        make_option(
            '--batch-time', default=5, dest="batch_secs",
            action="store",
            help="Try to keep batch removal transaction time to SECS seconds"),
    )

    def handle_label(self, table, **options):
        self.verbosity = int(options['verbosity'])
        if not psycopg2_available:
            raise CommandError('This command requires psycopg2')

        cursor_secs = int(options.get('cursor_secs', 3600))
        target_batch_time = int(options.get('batch_secs', 5))

        cleaner_cls = CLEANERS.get(table, None)
        if cleaner_cls is None:
            msg = ("Invalid cleaner.\nNo cleaner found for table: %s\n"
                   "Supported tables are: %s" % (table, CLEANERS.keys()))
            raise CommandError(msg)
        cleaner = cleaner_cls(self)
        cursor = connection.cursor()

        removed = -1
        total_removed = 0
        batch_size = 1
        cursor_opened_time = 0

        while True:
            # We don't want to hold the cursor open too long to allow
            # vacuum to reclaim rows. 1 hour default before it is reopened.
            if time.time() - cursor_opened_time > cursor_secs:
                cleaner.declare_expired_keys_cursor(cursor)
                cursor_opened_time = time.time()

            # Remove a batch of session rows.
            batch_start = time.time()
            removed = cleaner.remove_batch(cursor, batch_size)
            actual_batch_time = time.time() - batch_start

            total_removed += removed
            message = "Removed %d rows (%d total removed). Batch size %d\n"
            self.output(message % (removed, total_removed, batch_size), 1)

            # Done
            if removed == 0:
                self.output("All done.\n", 1)
                break

            # Increase or decrease the batch size by 10%, minimum 1.
            batch_size_wobble = int(batch_size * 0.1 + 1)
            if actual_batch_time > target_batch_time * 1.1:
                batch_size -= batch_size_wobble
            elif actual_batch_time < target_batch_time * 0.9:
                batch_size += batch_size_wobble
            if batch_size < 1:
                batch_size = 1

    def output(self, message, level=None, flush=False):
        if hasattr(self, 'stdout'):
            if level is None or self.verbosity >= level:
                self.stdout.write(message)
            if flush:
                self.stdout.flush()
