#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Management command to import app install data for a distroseries."""

from __future__ import absolute_import

import os
import re
import shutil
import tempfile
import urllib

from glob import iglob
from optparse import make_option
from random import choice
from tempfile import NamedTemporaryFile

import apt
from apt.cache import FetchFailedException
from apt_inst import DebFile
from django.conf import settings
from django.core.files.images import ImageFile
from django.core.management.base import LabelCommand

from webcatalog.forms import ArchiveApplicationForm
from webcatalog.models import (
    Application,
    DistroSeries,
)
from webcatalog.utilities import create_png_from_file

__metaclass__ = type
__all__ = []


def progress_char():
    return choice(u'♩♪').encode('utf-8')


class CacheMaker(object):
    sources = """
deb http://archive.ubuntu.com/ubuntu {distroseries} main universe
deb http://archive.ubuntu.com/ubuntu {distroseries}-updates main universe
deb http://archive.ubuntu.com/ubuntu {distroseries}-backports main universe
deb http://archive.canonical.com/ubuntu {distroseries} partner
deb http://extras.ubuntu.com/ubuntu {distroseries} main
"""

    def __init__(self, distroseries):
        self.distroseries = distroseries

    def create_apt_cache(self, series_cache_location):
        os.makedirs(os.path.join(series_cache_location, 'etc', 'apt'))
        sources_list = os.path.join(
            series_cache_location, 'etc', 'apt', 'sources.list')
        with open(sources_list, "w+") as sources_list_file:
            sources_list_file.write(self.sources.format(
                distroseries=self.distroseries))

    def get_apt_cache(self):
        # Create on-disk cache for distroseries if it
        # doesn't yet exist.
        series_cache_location = os.path.join(
            settings.DISK_APT_CACHE_LOCATION, self.distroseries)
        if not os.path.exists(series_cache_location):
            self.create_apt_cache(series_cache_location)

        # Force all packages into single architecture mode
        apt.apt_pkg.config.set("APT::Architecture", "i386")
        cache = apt.Cache(rootdir=series_cache_location)
        try:
            # XXX michaeln bug=879268 Lucid doesnt know about translations.
            # As long as our server is running on Lucid, we need to ensure
            # that apt is configured to download package translations,
            # even though on Lucid they'd normally not be necessary.
            apt.apt_pkg.config.set("APT::Acquire::Translation", "en")
            cache.update()
        except FetchFailedException:
            # Make sure __exit__ still gets called
            pass
        cache.open()
        return cache


class Command(LabelCommand):

    help = ("Update Application data from apt cache and "
            "app-install-data(-partner) packages.")
    option_list = LabelCommand.option_list + (
        make_option(
            '--local-app-install-data',
            action='store',
            dest='local_app_install_data',
            default='',
            help=('Use a local app-install-data deb package rather than '
                  'downloading from the archive.')),
        make_option(
            '--local-app-install-data-partner',
            action='store',
            dest='local_app_install_data_partner',
            default='',
            help=('Use a local app-install-data-partner deb package rather '
                  'than downloading from the archive.')),
    )
    verbosity = 1

    def set_cache(self, distroseries):
        """Set self.cache.

           Call set_cache at some point before accessing self.cache.
           Calling more than once does no harm.
        """
        if not hasattr(self, 'cache'):
            maker = CacheMaker(distroseries)
            self.cache = maker.get_apt_cache()

    def handle_label(self, distroseries_name, **options):
        self.verbosity = int(options['verbosity'])
        # First create the apps based on the current cache.
        distroseries, created = DistroSeries.objects.get_or_create(
            code_name=distroseries_name)
        if created:
            self.output(
                "Created a DistroSeries record called '{0}'.\n".format(
                    distroseries_name), 1)
        self.update_from_cache(distroseries_name)

        for package in ('app-install-data', 'app-install-data-partner', ):
            opt_name = 'local_{0}'.format(package.replace('-', '_'))
            deb_location = options.get(opt_name, '')
            self.update_from_app_data(package, distroseries, deb_location)

    def update_from_app_data(self, package_name, distroseries,
                             deb_location=''):
        data_dir = tempfile.mkdtemp()
        # Download app install data if necessary
        if deb_location == '':
            deb_location = self.get_package_data_for_series(
                package_name, distroseries.code_name, data_dir)

        # Extract and parse the deb archive.
        deb_file = DebFile(deb_location)
        self.output("Processing {0} to update database...\n".format(
            package_name), 1)
        deb_file.data.extractall(data_dir)
        matcher = data_dir + '/usr/share/app-install/desktop/*.desktop'
        icon_dir = data_dir + '/usr/share/app-install/icons/'

        for desktop_file in iglob(matcher):
            if self.verbosity > 1:
                self.output("Processing {0}\n".format(desktop_file), 1)
            elif self.verbosity > 0:
                self.output(progress_char(), 1, flush=True)
            self.process_desktop_file(desktop_file, icon_dir, distroseries)

        shutil.rmtree(data_dir)

    def get_package_data_for_series(self, package_name, distroseries,
                                    working_dir):
        install_data_url = self.get_package_uri_for_series(
            package_name, distroseries)
        rest, sep, deb_filename = install_data_url.rpartition('/')

        self.output(u"Downloading {0} from {1}...\n".format(package_name,
                    install_data_url), 1)
        deb_location = os.path.join(working_dir, deb_filename)
        urllib.urlretrieve(install_data_url, deb_location)
        self.output("Done.\n", 1)
        return deb_location

    def get_package_uri_for_series(self, package_name, distroseries):
        self.output("Determining URI for current version of "
                    "{0} in {1}.\n".format(package_name, distroseries), 1)
        self.set_cache(distroseries)
        return self.cache[package_name].candidate.uri

    def process_desktop_file(self, desktop_file_path, icon_dir, distroseries):
        with open(desktop_file_path, 'r') as desktop_file:
            data = desktop_file.read()
        form = ArchiveApplicationForm.get_form_from_desktop_data(
            data, distroseries)

        pkgname = form.data.get('package_name', '')
        if ":" in pkgname:
            self.output("skipped multiarch pkg '%s'" % pkgname)
            return

        if form.is_valid():
            app = form.save()
            app.update_departments()
            self.add_icon_to_app(app, icon_dir)
            if self.verbosity > 1:
                self.output(u"{0} created.\n".format(app.name).encode(
                    'utf-8'), 1)
        else:
            self.output(
                u"Skipping {0} as input failed validation: {1}.\n".format(
                    form.data.get('package_name', ''),
                    form.errors).encode('utf-8'), 1)

    def add_icon_to_app(self, app, icon_dir):
        # If the desktop file specifies an icon, we check for it in the
        # default location, otherwise we use the packagename and check
        # for that.
        icon_name = app.icon_name or app.package_name
        icon_path_generic = icon_dir + "{0}".format(icon_name)
        icon_path = ''

        # Some desktop files specify the icon extension as part of the
        # icon name.
        supported_extensions = ('xpm', 'png', 'svg', 'tiff')
        if icon_path_generic.endswith(supported_extensions):
            if os.path.exists(icon_path_generic):
                icon_path = icon_path_generic
        else:
            # Python's glob module doesn't seem to support extra matchers
            # such as '@(xpm|png|svg)'.
            for ext in supported_extensions:
                path_with_ext = icon_path_generic + '.' + ext
                if os.path.exists(path_with_ext):
                    icon_path = path_with_ext
                    break

        if icon_path:
            icon_path = self.convert_to_png(icon_path)
            with open(icon_path, "r") as icon_file:
                app.icon = ImageFile(icon_file)
                app.save()
        else:
            self.output(u"No icon found for {0} at {1}.({2}).\n".format(
                        app.name, icon_path_generic,
                        '|'.join(supported_extensions)).encode('utf-8'), 1)

    def convert_to_png(self, icon_path):
        parts = os.path.splitext(icon_path)
        if parts[1] in ['.xpm', '.svg', '.tiff']:
            target_path = parts[0] + '.png'
            if create_png_from_file(target_path, icon_path, 32, 32):
                icon_path = target_path
        return icon_path

    def fetch_icon_from_extras(self, app, candidate):
        """Fetch icon from the /meta/ directory on extras.ubuntu.com.

        This method will only do anything if the provided app is in extras.
        """
        icon_name = candidate.record.get('Icon')
        match = re.match('http://(.*)extras.ubuntu.com/', candidate.uri)
        if icon_name and match:
            uri = match.group() + 'meta/' + icon_name
            response = urllib.urlopen(uri)
            if response.getcode() == 200:
                parts = os.path.splitext(icon_name)
                with NamedTemporaryFile(suffix=parts[1]) as filep:
                    filep.write(response.read())
                    filep.flush()
                    icon_path = self.convert_to_png(filep.name)
                    with open(icon_path, "r") as icon_file:
                        app.icon = ImageFile(icon_file)
                        app.save()

    def update_from_cache(self, distroseries_name):
        distroseries, created = DistroSeries.objects.get_or_create(
            code_name=distroseries_name)
        if created:
            msg = "Created a DistroSeries record called '{0}'.\n"
            self.output(msg.format(distroseries_name), 1)

        prefetched_apps = dict(
            (app.package_name, app) for app in
            Application.objects.filter(distroseries=distroseries))

        self.set_cache(distroseries_name)
        # For each package in the apt-cache, we update (or possibly
        # create) a matching db entry.
        for package in self.cache:
            if self.verbosity > 1:
                self.output("Updating {0}\n".format(package.name), 1)
            elif self.verbosity > 0:
                self.output(progress_char(), 1, flush=True)
            candidate = package.candidate
            # USC uses the first line of the summary as the
            # application name and the rest as the comment..
            summary_lines = candidate.summary.split('\n')
            app_name = summary_lines[0]
            app_comment = '\n'.join(summary_lines[1:])
            if not app_comment:
                app_comment = package.name

            version = apt.apt_pkg.upstream_version(candidate.version)
            if package.name in prefetched_apps:
                app = prefetched_apps[package.name]
                if app.imported_from_sca:
                    continue
                updated = False
                for attribute, value in [
                    ('description', candidate.description),
                    ('section', candidate.section), ('version', version),
                    ('name', app_name), ('comment', app_comment),
                ]:
                    if getattr(app, attribute) != value:
                        setattr(app, attribute, value)
                        updated = True
                if updated:
                    app.save()
            else:
                app = Application.objects.create(
                    package_name=package.name,
                    distroseries=distroseries,
                    description=candidate.description,
                    section=candidate.section,
                    name=app_name,
                    version=version,
                    comment=app_comment,
                )

            screenshot_url = package.candidateRecord.get('Screenshot-Url')
            if screenshot_url is not None and (
                    screenshot_url not in app.screenshots):
                app.applicationmedia_set.create(
                    media_type='screenshot', url=screenshot_url)

            self.fetch_icon_from_extras(app, candidate)
        if self.verbosity == 1:
            self.output("\n", 1)
        return distroseries

    def output(self, message, level=None, flush=False):
        if hasattr(self, 'stdout'):
            if level is None or self.verbosity >= level:
                self.stdout.write(message)
            if flush:
                self.stdout.flush()
