#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Management command to import for purchase applications from Software
   Center.
"""

from __future__ import absolute_import

import json
import os
import urllib
from collections import defaultdict
from StringIO import StringIO
from tempfile import mkstemp

from django.conf import settings
from django.core.files.images import ImageFile
from django.core.management.base import BaseCommand

from webcatalog.forms import SCAApplicationForm
from webcatalog.models import (
    Application,
    DistroSeries,
)

__metaclass__ = type
__all__ = []


class Command(BaseCommand):
    help = "Import for-purchase applications from Software Center."
    verbosity = 0

    def handle(self, *args, **options):
        url = '%sapplications/any/ubuntu/any/any/' % settings.SCA_API_URL
        # call api and get apps for purchase
        response = urllib.urlopen(url)
        if response.code != 200:
            raise Exception("Couldn't connect to server at %s" % url)
        app_list = json.loads(response.read())
        imported_apps = Application.objects.filter(
            imported_from_sca=True).values_list(
                'package_name',
                'distroseries__code_name')
        existing_apps = defaultdict(list)
        for package_name, code_name in imported_apps:
            existing_apps[package_name].append(code_name)
        reverse_versions = dict((v, k) for (k, v) in
                                settings.UBUNTU_SERIES_FOR_VERSIONS.items())
        for app_data in app_list:
            # get icon data once per app (not per app per distroseries!)
            icon_data = self.get_icon_data(app_data)
            for series_name in app_data['series']:
                if series_name not in reverse_versions:
                    continue
                distroseries, created = DistroSeries.objects.get_or_create(
                    code_name=series_name)
                if created:
                    self.output(
                        "Created a DistroSeries record called '{0}'.\n".format(
                            series_name), 1)
                package_name = self.import_app_from_data(
                    app_data, icon_data, distroseries)
                if package_name:
                    existing_series = existing_apps.get(package_name)
                    if existing_series and series_name in existing_series:
                        existing_series.remove(series_name)
                        if not existing_series:
                            del(existing_apps[package_name])

            Application.objects.check_latest(app_data['package_name'])

        # Any sca-imported apps that existed before update, but were not
        # imported from sca are removed.
        for package_name, series in existing_apps.items():
            Application.objects.filter(
                package_name=package_name,
                distroseries__code_name__in=series).delete()

    def import_app_from_data(self, app_data, icon_data, distroseries):
        form = SCAApplicationForm.from_api_data(
            app_data, distroseries)
        if form.is_valid():
            app = form.save()
            department_names = app_data.get('department', [])
            app.update_departments(department_names)
            self.add_icon_to_app(app, data=icon_data)
            self.output(u"{0} created.\n".format(app.name).encode('utf-8'), 1)
        return app.package_name

    def get_icon_data(self, app_data):
        icon_data = app_data.get('icon_data', '')
        if icon_data:
            data = icon_data.decode('base64')
        else:
            url = app_data.get('icon_url', '')
            data = urllib.urlopen(url).read()
        return data

    def add_icon_to_app(self, app, data):
        pngfile = StringIO(data)
        try:
            fd, filename = mkstemp(prefix=app.package_name, suffix='.png')
            iconfile = os.fdopen(fd, 'w')
            iconfile.write(pngfile.read())
            iconfile.close()
            with open(filename) as f:
                app.icon = ImageFile(f)
                app.save()
        finally:
            if os.path.exists(filename):
                os.unlink(filename)

    def output(self, message, level=None, flush=False):
        if hasattr(self, 'stdout'):
            if level is None or self.verbosity >= level:
                self.stdout.write(message)
            if flush:
                self.stdout.flush()
