#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Management command to import review statistics for a distroseries."""

import json
import pytz

from datetime import (
    datetime,
    timedelta,
)
from optparse import make_option

from django.core.management.base import LabelCommand

from webcatalog.models import (
    Application,
    DistroSeries,
    ReviewStatsImport,
)
from webcatalog.utilities import (
    dampened_rating,
    WebServices,
)


class Command(LabelCommand):
    option_list = LabelCommand.option_list + (
        make_option(
            '--download-all',
            action='store_true',
            dest='download_all',
            default=False,
            help='Download all ratings stats instead of stats since last '
                 'import.'),
    )

    help = "Update application review statistics."
    verbosity = 1

    def handle_label(self, distroseries_name, **options):
        self.verbosity = int(options['verbosity'])
        self.download_all = options['download_all']
        # Check when we most recently imported review stats for this
        # distroseries.
        distroseries, created = DistroSeries.objects.get_or_create(
            code_name=distroseries_name)

        stats = self.download_review_stats(distroseries)

        Application.objects.filter(distroseries=distroseries).update(
            wilson_score=-1)

        self.update_apps_with_stats(distroseries, stats)

        self.update_last_import_timestamp(distroseries)

    def update_apps_with_stats(self, distroseries, stats):
        stats_dict = dict((stat.package_name, stat) for stat in stats)
        apps = Application.objects.filter(distroseries=distroseries)
        apps = apps.order_by('package_name')

        for app in apps:
            if not app.package_name in stats_dict:
                continue
            stat = stats_dict[app.package_name]
            app.ratings_average = stat.ratings_average
            app.ratings_total = stat.ratings_total
            histogram = stat.histogram or '[0, 0, 0, 0, 0]'
            app.ratings_histogram = histogram
            app.wilson_score = dampened_rating(json.loads(histogram))
            app.save()

    def update_last_import_timestamp(self, distroseries):
        review_stats_import, created = ReviewStatsImport.objects.get_or_create(
            distroseries=distroseries)
        review_stats_import.last_import = datetime.now(pytz.utc)
        review_stats_import.save()

    def download_review_stats(self, distroseries):
        # Download the appropriate stats based on above.
        num_days = None
        if not self.download_all:
            other_imports = ReviewStatsImport.objects.filter(
                distroseries=distroseries).order_by('-last_import')
            if other_imports:
                stats_import = other_imports[0]
                time_since_last_import = (
                    datetime.now(pytz.utc) - stats_import.last_import)
                if time_since_last_import < timedelta(days=1):
                    num_days = 1
                elif time_since_last_import < timedelta(days=3):
                    num_days = 3
                elif time_since_last_import < timedelta(days=7):
                    num_days = 7

        # Currently we have ratings in rnr for ubuntu and for-purchase
        # PPAs only, so we set origin to any. If we later support reviews
        # for general PPAs, we will need to update this.
        kwargs = dict(origin='any', distroseries=distroseries.code_name)
        if num_days:
            kwargs['days'] = num_days
        return WebServices().rnr_api.review_stats(**kwargs)
