#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Django object managers."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'ApplicationManager',
]

from django.conf import settings
from django.db import models
from django.http import Http404


class ApplicationManager(models.Manager):
    def find_best(self, **kwargs):
        options = self.filter(**kwargs).order_by('-distroseries__version')
        if options.exists():
            for option in options:
                if not option.distroseries.prerelease:
                    return option
            return options[0]

    def find_best_or_404(self, **kwargs):
        app = self.find_best(**kwargs)
        if app is None:
            raise Http404()
        return app

    def get_latest_id(self, versions):
        """Given a list of (app_id, prerelease) pairs, return the latest id"""
        if not versions:
            return
        for app_id, prerelease in versions:
            if not prerelease:
                return app_id
        return versions[0][0]

    def check_latest(self, package_name):
        """Check which app with the given package_name is the latest"""
        options = self.filter(package_name=package_name)
        versions = options.order_by('-distroseries__version').values_list(
            'id', 'distroseries__prerelease')
        latest_id = self.get_latest_id(versions)
        options.update(is_latest=False)
        self.filter(pk=latest_id).update(is_latest=True)


class DistroSeriesManager(models.Manager):

    def get_or_create(self, **kwargs):
        if not 'defaults' in kwargs:
            code_name = kwargs.get('code_name', '')
            versions = settings.UBUNTU_SERIES_FOR_VERSIONS
            reverse_versions = dict((v, k) for (k, v) in versions.items())
            kwargs['defaults'] = dict(version=reverse_versions[code_name])
        return super(DistroSeriesManager, self).get_or_create(**kwargs)
