#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Models for the package application."""

from __future__ import absolute_import

import logging
from datetime import datetime

from django.conf import settings
from django.contrib.auth.models import (
    Group,
    Permission,
    User,
)
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import models
from django.template.defaultfilters import slugify
from django.db.models.signals import post_syncdb

from webcatalog.department_filters import department_filters
from webcatalog.managers import ApplicationManager, DistroSeriesManager

__metaclass__ = type
__all__ = [
    'Application',
    'ApplicationMedia',
    'ApplicationWidget',
    'Department',
    'DistroSeries',
    'Exhibit',
    'Machine',
    'ReviewStatsImport',
]


class DistroSeries(models.Model):
    """An Ubuntu distribution series."""
    code_name = models.CharField(max_length=20, unique=True, db_index=True)
    version = models.CharField(max_length=10, blank=True)
    prerelease = models.BooleanField(default=False)

    objects = DistroSeriesManager()

    def __unicode__(self):
        return "Ubuntu %s (%s)" % (self.code_name.capitalize(), self.version)

    class Meta:
        app_label = 'webcatalog'


class Application(models.Model):
    # We'll most likely need one of these per lang/series (as each lang
    # will potentially have a different name) and names/descriptions can
    # change for a package for different series.
    # We may need to have one application item per language, depending
    # whether we have gettext data available that we could use at
    # runtime (much preferred)
    distroseries = models.ForeignKey(DistroSeries)

    # The following fields are extracted from app-install-data.
    package_name = models.CharField(max_length=100)
    name = models.CharField(max_length=255)
    version = models.CharField(max_length=64, blank=True)
    comment = models.CharField(max_length=255, blank=True)
    popcon = models.IntegerField(null=True, blank=True)
    channel = models.CharField(max_length=255, blank=True)
    mimetype = models.CharField(max_length=2048, blank=True)
    architectures = models.CharField(max_length=255, blank=True)
    keywords = models.CharField(max_length=255, blank=True)
    app_type = models.CharField(max_length=32, blank=True)
    section = models.CharField(max_length=32, blank=True)
    categories = models.CharField(max_length=255, blank=True)
    departments = models.ManyToManyField('Department', blank=True)
    icon_name = models.CharField(max_length=255, blank=True)
    icon = models.ImageField(
        upload_to='icons/%Y/%m', max_length=200, null=True, blank=True)
    imported_from_sca = models.BooleanField(default=False)
    archive_id = models.CharField(
        max_length=64, null=True, db_index=True, blank=True)
    price = models.DecimalField(
        max_digits=7, decimal_places=2, null=True,
        blank=True, help_text="For-purchase applications (in US Dollars).")

    ratings_total = models.IntegerField(null=True, blank=True)
    ratings_average = models.DecimalField(
        max_digits=3, decimal_places=2, null=True, blank=True)
    ratings_histogram = models.CharField(max_length=128, blank=True)
    is_latest = models.BooleanField()
    wilson_score = models.FloatField(null=True, blank=True, db_index=True)
    debtags = models.CharField(max_length=255, blank=True)
    license = models.CharField(
        max_length=64, blank=True,
        help_text=u"The name of the license used for the app.")

    # Other desktop fields used by s-c
    # x-gnome-fullname
    # X-AppInstall-Ignore
    # Icon

    # The following fields will need to be imported from the apt-cache
    # (using python-apt - as above we'll need access to info from different
    # series etc.)
    description = models.TextField(blank=True)

    objects = ApplicationManager()

    def __unicode__(self):
        return u"{0} ({1})".format(self.name, self.package_name)

    def _get_media(self, media_type):
        if getattr(self, '_media_cache', None) is None:
            self._media_cache = {}

        if not media_type in self._media_cache:
            medias = self.applicationmedia_set.filter(media_type=media_type)
            self._media_cache[media_type] = [media.url for media in medias]

        return self._media_cache[media_type]

    @property
    def screenshots(self):
        return self._get_media('screenshot')

    @property
    def video_iframe_urls(self):
        return self._get_media('video')

    @property
    def categories_set(self):
        """Return the set of categories for this app"""
        stripped = [x.strip() for x in self.categories.split(';')]
        return set(x for x in stripped if x)

    def available_distroseries(self):
        """Return the set of distroseries for which this app is available"""
        return (DistroSeries.objects
                .filter(application__package_name=self.package_name)
                .distinct().order_by('-code_name'))

    def update_departments(self, department_names=None):
        """Update the list of departments for this app"""
        department_names = department_names or []

        def get_dept(dept_name):
            dept, created = Department.objects.get_or_create(
                slug=slugify(dept_name), defaults=dict(name=dept_name))
            if created:
                logging.warn("Department %s automatically created!" %
                             dept_name)
            if not created and dept_name != dept.name:
                logging.warn("Department %s name updated to %s.",
                             dept.name, dept_name)
                dept.name = dept_name
                dept.save()
            return dept

        if self.imported_from_sca:
            if department_names:
                for dept_name in department_names:
                    dept = get_dept(dept_name)
                    self.departments.add(dept)
                return
            elif self.departments.all():
                return

        self.departments.clear()
        for dept_name, dept_filters in department_filters.items():
            for dept_filter in dept_filters:
                if dept_filter(self):
                    dept = get_dept(dept_name)
                    self.departments.add(dept)
                    break

    def crumbs(self):
        depts = self.departments.order_by('-parent')
        if depts.count():
            crumbs = depts[0].crumbs(distro=self.distroseries.code_name)
        else:
            crumbs = [{'name': 'All Apps', 'url': reverse('wc-index')}]
        url = reverse('wc-package-detail',
                      args=[self.distroseries.code_name, self.package_name])
        crumbs.append({'name': self.name, 'url': url})
        return crumbs

    def license_type(self):
        return self.license or "Open Source"

    @property
    def architectures_list(self):
        return self.architectures.split(',')

    @property
    def icon_url_or_default(self):
        """Return this app's icon's url, or an url to a generic icon."""
        if self.icon:
            return self.icon.url
        return settings.STATIC_URL + 'images/applications-other-64.png'

    class Meta:
        app_label = 'webcatalog'
        unique_together = ('distroseries', 'package_name')
        ordering = ('-wilson_score', 'name')


class ApplicationMedia(models.Model):
    MEDIA_CHOICES = (
        ('icon_16', 'Icon 16x16'),
        ('icon_32', 'Icon 32x32'),
        ('icon_64', 'Icon 64x64'),
        ('icon_128', 'Icon 128x128'),
        ('screenshot', 'Screenshot'),
        ('video', 'Video'),
    )

    application = models.ForeignKey(Application)
    media_type = models.CharField(max_length=16, choices=MEDIA_CHOICES)
    url = models.URLField()

    def __unicode__(self):
        return "{0} for {1}".format(self.media_type, self.application)

    class Meta:
        app_label = 'webcatalog'
        unique_together = (('application', 'url'),)
        ordering = ('url',)


class Department(models.Model):
    parent = models.ForeignKey('self', blank=True, null=True)
    name = models.CharField(max_length=64)
    slug = models.SlugField(unique=True)

    def __unicode__(self):
        return self.name

    def crumbs(self, distro=None):
        if self.parent:
            crumbs = self.parent.crumbs(distro=distro)
        else:
            crumbs = [{'name': 'All Apps', 'url': reverse('wc-index')}]
        kwargs = {'dept_slug_or_id': self.slug}
        if distro is not None:
            kwargs['distro'] = distro
        url = reverse('wc-department', kwargs=kwargs)
        crumbs.append({'name': self.name, 'url': url})
        return crumbs

    class Meta:
        app_label = 'webcatalog'


class ReviewStatsImport(models.Model):
    distroseries = models.ForeignKey(DistroSeries, unique=True)
    last_import = models.DateTimeField(default=datetime.utcnow)

    class Meta:
        app_label = 'webcatalog'


class Machine(models.Model):
    owner = models.ForeignKey(User, db_index=True)
    uuid = models.CharField(max_length=32, db_index=True)
    hostname = models.CharField(max_length=64)
    packages_checksum = models.CharField(max_length=56)
    package_list = models.TextField()
    logo_checksum = models.CharField(max_length=56, blank=True)

    class Meta:
        app_label = 'webcatalog'
        unique_together = ('owner', 'uuid')


class Exhibit(models.Model):
    sca_id = models.IntegerField()
    package_names = models.CharField(max_length=1024)
    banner_url = models.CharField(max_length=1024)
    click_url = models.URLField(default='')
    distroseries = models.ManyToManyField(DistroSeries)
    html = models.TextField()
    date_created = models.DateTimeField(auto_now_add=True)
    published = models.BooleanField()
    display = models.NullBooleanField(
        help_text="Yes: Always display. No: Never display. "
                  "Unknown: Display if published")
    weight = models.IntegerField(default=0)

    class Meta:
        app_label = 'webcatalog'

    def destination_url(self):
        if self.click_url:
            return self.click_url

        pkg_names = self.package_names.split(',')
        if len(pkg_names) == 1:
            pkg_name = pkg_names[0]
            if not pkg_name:
                return ''
            return reverse('wc-package-detail',
                           kwargs={'package_name': pkg_name})

        pkgs = '&'.join('pkg_name=%s' % pkg_name for pkg_name in pkg_names)
        return reverse('wc-package-list') + '?' + pkgs


class ApplicationWidget(models.Model):
    name = models.CharField(max_length=32,
                            help_text="Just a reference for the admin view.")
    applications = models.ManyToManyField(Application)
    template_snippet = models.TextField()

    def __unicode__(self):
        return self.name

    class Meta:
        app_label = 'webcatalog'


# GroupPermissions can't go in fixtures because Permissions are stored as
# model metadata, so our Permission's primary key can change without warning
def post_syncdb_handler(sender, **kwargs):
    ct = ContentType.objects.get_for_model(Application)
    group, _ = Group.objects.get_or_create(name='developers')
    perm, _ = Permission.objects.get_or_create(
        codename='schedule_tasks',
        name="Can schedule asynchronous tasks via the web",
        content_type=ct,
    )
    if not perm in group.permissions.all():
        group.permissions.add(perm)
        group.save()

post_syncdb.connect(post_syncdb_handler)
