#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Preflight check for webcatalog."""

from __future__ import absolute_import

from celery.app import default_app
from django import db
from django.conf import settings
from preflight import Preflight, register

import openid
import piston.utils

from webcatalog.utilities import WebServices

__metaclass__ = type
__all__ = [
    'WebcatalogPreflight'
]


class WebcatalogPreflight(Preflight):

    def authenticate(self, request):
        if not request.user.is_authenticated():
            return False
        allowed_teams = getattr(settings, 'PREFLIGHT_GROUPS', [])
        groups = request.user.groups.filter(name__in=allowed_teams)
        return groups.count() > 0

    def versions(self):
        return [
            {'name': 'openid', 'version': openid.__version__},
            {'name': 'piston', 'version': piston.utils.get_version()},
        ]

    def check_validate_config(self):
        """Make sure that our config files validate correctly. """
        parser = settings.__CONFIGGLUE_PARSER__
        return parser.is_valid()

    def check_rnr_available(self):
        """Make sure that rnr can be reached. """
        rnr = WebServices().rnr_api
        return rnr.server_status() == 'ok'

    def check_recommender_available(self):
        """Make sure that the recommender can be reached. """
        recommender = WebServices().recommender_api
        return recommender.server_status() == 'ok'

    def check_screenshots_available(self):
        """Make sure that screenshots are available. """
        screenshots = WebServices().get_screenshots_for_package('apt')
        return len(screenshots) > 0

    def check_identity_provider_available(self):
        """Make sure SSO can be reached."""
        sso = WebServices().identity_provider
        return not sso.validate_token(
            consumer_key='invalid', token="token")

    def check_database(self):
        """Make sure database connections are accepted."""
        cursor = db.connection.cursor()
        cursor.execute('SELECT 42')
        cursor.fetchone()

        return True

    def check_celery_workers(self):
        """Check that the celery worker is up and running."""
        return len(default_app.control.ping()) > 0

register(WebcatalogPreflight)
