#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Custom template tags for the Apps Directory."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'install_options',
]

import string
from urlparse import urlparse

from django import template
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from django.template import Template

from webcatalog.models import Application
from webcatalog.utilities import UserAgentString

register = template.Library()


@register.inclusion_tag('webcatalog/install_options_snippet.html',
                        takes_context=True)
def install_options(context, application):
    """Evaluate the installation options for the application in context."""
    useragent = UserAgentString(context['user_agent'])
    context['application'] = application
    context['message_text'] = ''
    context['display_install_button'] = False
    context['display_ubuntu_download'] = False

    if useragent.distroseries == application.distroseries.code_name:
        # If both the distroseries and architecture matches, install away,
        # otherwise if the architecture doesn't match inform the user.
        for arch in application.architectures_list:
            if useragent.matches_appinstall_arch(arch):
                context['display_install_button'] = True
                return context

        context['message_text'] = _(
            "{application_name} is not available for "
            "your computer.").format(application_name=application.name)
        return context

    if useragent.distroseries:
        # If we know a definitive distroseries but doesn't match the
        # application being viewed, we check for the app with the users
        # series and recommend it, or if it doesn't exist, say so.
        try:
            Application.objects.get(
                package_name=application.package_name,
                distroseries__code_name=useragent.distroseries)
            app_in_series_url = reverse(
                'wc-package-detail',
                args=[useragent.distroseries, application.package_name])
            context['message_text'] = _(
                '{application_name} is also available for '
                '<a href="{app_in_series_url}">your version of '
                'Ubuntu</a>.').format(
                    application_name=escape(application.name),
                    app_in_series_url=app_in_series_url)
            context['message_text'] = mark_safe(
                context['message_text'])
            return context
        except Application.DoesNotExist:
            context['message_text'] = _(
                '{application_name} is not available for your version '
                'of Ubuntu.').format(application_name=application.name)
            return context

    if useragent.is_linux:
        # XXX Michael Nelson 2011-06-28 bug=709125
        # Until we know what our staging/production domains will be and have
        # ubufox updated similarly to apt.canonical.com as per bug 709125,
        # we'll only have the correct user-agent for chromium, so we display
        # the install button for all linux browsers until that is resolved.
        # For most packages this will not affect ubuntu users at all (as
        # they are generally arch any and available across
        # distroseries). When the package doesn't exist for the Ubuntu
        # users distroseries, they'll simply be told that when software
        # center opens up. Non-ubuntu/debian users browsers won't
        # understand the apt url.
        context['display_install_button'] = True
        return context
    else:
        context['message_text'] = ''
        context['display_ubuntu_download'] = True
        return context


# These next two functions are taken from software-center's utils.py:
def normalize_package_description(desc):
    """ this takes a package description and normalizes it
        so that all uneeded \n are stripped away and all
        enumerations are at the start of the line and start with a "*"
        E.g.:
        Some potentially very long paragrah that is in a single line.
        A new paragrpah.
        A list:
        * item1
        * item2 that may again be very very long
    """
    BULLETS = ('- ', '* ', 'o ')
    prefix = string.whitespace + "".join([s.strip() for s in BULLETS])
    norm_description = ""
    in_blist = False
    # process it
    old_indent_level = 0
    for i, part in enumerate(desc.split("\n")):
        part = part.strip()
        # explicit newline
        if not part:
            norm_description += "\n"
            continue
        # get indent level
        for j, c in enumerate(part):
            if not c in prefix:
                indent_level = j
                break
        # check if in a enumeration
        if part[:2] in BULLETS:
            in_blist = True
            norm_description += "\n* " + part[2:]
        elif in_blist and old_indent_level == indent_level:
            norm_description += " " + part
        else:
            in_blist = False
            if not norm_description.endswith("\n"):
                norm_description += " "
            norm_description += part
        old_indent_level = indent_level
    return norm_description.strip()


@register.filter
def htmlize_package_description(desc):
    html = ""
    inside_li = False
    for part in normalize_package_description(desc).split("\n"):
        if part.startswith("* "):
            if not inside_li:
                html += "<ul>"
                inside_li = True
            html += '<li>%s</li>' % part[2:]
        else:
            if inside_li:
                html += "</ul>"
            html += '<p tabindex="0">%s</p>' % part
            inside_li = False
    if inside_li:
        html += "</ul>"
    return mark_safe(html)


@register.filter
def visible_page_range(page):
    first = max(page.number - 5, 1)
    last = min(first + 10, page.paginator.num_pages + 1)
    return range(first, last)


@register.inclusion_tag("webcatalog/rating_summary.html")
def rating_summary(num_stars, size='small', num_ratings=None):
    stars = []
    if not num_stars:
        return {}

    num_stars = float(num_stars)
    for count in range(1, 6):
        if num_stars >= count:
            stars.append(1)
        elif 0.5 >= count - num_stars > 0:
            stars.append(0.5)
        else:
            stars.append(0)

    return dict(stars=stars, total=num_ratings,
                STATIC_URL=settings.STATIC_URL, size=size)


@register.simple_tag(takes_context=True)
def render_app_widget(context, template_snippet):
    template = Template(template_snippet)
    return template.render(context)


@register.filter
def thumbnail_url(screenshot):
    parsed = urlparse(screenshot)
    host = parsed.netloc
    path = parsed.path
    thumbnail_hosts = settings.THUMBNAIL_HOSTS
    if host in thumbnail_hosts and path.startswith('/site_media/'):
        new_path = path.replace('/site_media/', '/dev/thumbnail/225x170/', 1)
        parsed = parsed._replace(path=new_path)
        screenshot = parsed.geturl()
    return screenshot
