#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""A TestCase class with a built in object factory."""

from __future__ import absolute_import

import os
import pytz

from datetime import (
    datetime,
    timedelta,
)
from itertools import count
from django.contrib.auth.models import (
    Group,
    Permission,
    User,
)
from django.contrib.sessions.models import Session
from django.test import (
    TestCase,
    TransactionTestCase,
)
from django_openid_auth.models import UserOpenID

from webcatalog.models import (
    Application,
    ApplicationMedia,
    ApplicationWidget,
    Consumer,
    Department,
    DistroSeries,
    Exhibit,
    Machine,
    Nonce,
    Token,
)
from webcatalog.utilities import full_claimed_id

__metaclass__ = type
__all__ = [
    'TestCaseWithFactory',
    'WebCatalogObjectFactory',
]


class WebCatalogObjectFactory(object):

    unique_id_counter = count()

    def get_unique_integer(self):
        """Return a unique integer for the test run."""
        return self.unique_id_counter.next()

    def get_unique_string(self, prefix=None):
        if prefix is None:
            prefix = "generic-string"
        return prefix + str(self.get_unique_integer())

    def make_user(self, username=None, email=None, password='test',
                  first_name=None, last_name=None, open_id=None,
                  is_admin=False, permissions=None, groups=None,
                  log_in=False):
        if username is None:
            username = self.get_unique_string(prefix='username')
        if email is None:
            email = self.get_unique_string(prefix='email') + '@example.com'
        if first_name is None:
            first_name = self.get_unique_string(prefix='firstname')
        if last_name is None:
            last_name = self.get_unique_string(prefix='lastname')

        user = User.objects.create_user(
            username=username, email=email, password=password)
        user.first_name = first_name
        user.last_name = last_name

        if is_admin:
            user.is_staff = True
            user.is_superuser = True
        user.save()

        # Create an openid record too.
        if open_id is None:
            open_id = full_claimed_id(self.get_unique_string(prefix='ident-'))
        elif open_id is False:
            return user
        useropenid = UserOpenID.objects.create(
            user=user, claimed_id=open_id, display_id=open_id)

        if permissions:
            for permission in permissions:
                user.user_permissions.add(
                    Permission.objects.get(codename=permission))

        if groups:
            for group_name in groups:
                group, created = Group.objects.get_or_create(name=group_name)
                user.groups.add(group)
        return user

    def make_application(self, package_name=None, name=None,
                         comment=None, description=None, icon_name='',
                         icon=None, distroseries=None, arch='i686',
                         ratings_average=None, ratings_total=None,
                         ratings_histogram='', screenshot_url='',
                         archive_id=None, version='', is_latest=False,
                         wilson_score=0.0, debtags=[], departments=None,
                         license='', price=None, imported_from_sca=False):
        if name is None:
            name = self.get_unique_string(prefix='Readable Name')
        if package_name is None:
            package_name = self.get_unique_string(prefix='pkg-name')
        if comment is None:
            comment = self.get_unique_string(prefix='comment')
        if description is None:
            description = self.get_unique_string(prefix='description')
        if distroseries is None:
            distroseries = self.make_distroseries()

        application = Application.objects.create(
            package_name=package_name, name=name, comment=comment,
            description=description, popcon=999, icon=icon,
            icon_name=icon_name, distroseries=distroseries, architectures=arch,
            ratings_average=ratings_average, ratings_total=ratings_total,
            ratings_histogram=ratings_histogram,
            archive_id=archive_id, version=version, is_latest=is_latest,
            wilson_score=wilson_score, debtags=debtags,
            license=license, price=price, imported_from_sca=imported_from_sca)

        if departments is not None:
            for d in departments:
                application.departments.add(d)

        if screenshot_url:
            ApplicationMedia.objects.create(
                application=application,
                url=screenshot_url,
                media_type='screenshot')
        return application

    def make_department(self, name, parent=None, slug=None):
        if slug is None:
            slug = self.get_unique_string(prefix='slug-')
        return Department.objects.create(name=name, parent=parent, slug=slug)

    def make_exhibit(self, package_names=None, published=True, display=None,
                     distroseries=None, weight=0, click_url=''):
        sca_id = self.get_unique_integer()
        if package_names is None:
            package_names = self.get_unique_string(prefix='package-')
        banner_url = self.get_unique_string(prefix='http://example/url')
        html = self.get_unique_string(prefix='html-')
        if distroseries is None:
            distroseries = self.make_distroseries()
        xibit = Exhibit.objects.create(
            sca_id=sca_id, package_names=package_names, banner_url=banner_url,
            html=html, published=published, display=display, weight=weight,
            click_url=click_url)
        xibit.distroseries.add(distroseries)
        return xibit

    def make_distroseries(self, code_name=None, version=None,
                          prerelease=False):
        if code_name is None:
            code_name = self.get_unique_string(prefix='series-')
        if version is None:
            version = "1.{0}".format(self.get_unique_integer())

        return DistroSeries.objects.create(
            code_name=code_name, version=version, prerelease=prerelease)

    def get_test_path(self, file_name):
        return os.path.join(
            os.path.dirname(__file__), 'test_data', file_name)

    def make_machine(self, owner=None, uuid=None, hostname=None,
                     package_list=None):
        if owner is None:
            owner = self.make_user()
        if hostname is None:
            hostname = self.get_unique_string(prefix='hostname-')
        if uuid is None:
            uuid = self.get_unique_string(prefix='uuid-')
        if package_list is None:
            package_list = self.get_unique_string(prefix='package-list-')
        packages_checksum = self.get_unique_string(prefix='package-checksum-')
        logo_checksum = self.get_unique_string(prefix='logo-checksum-')

        return Machine.objects.create(
            owner=owner, hostname=hostname,
            uuid=uuid, packages_checksum=packages_checksum,
            package_list=package_list, logo_checksum=logo_checksum)

    def make_oauth_token_and_consumer(self, user=None, save=True):
        """Create a new set of OAuth token and consumer creds."""
        if user is None:
            user = self.make_user()

        consumer_key = user.useropenid_set.get().claimed_id.split('/')[-1]
        consumer_secret = self.get_unique_string(prefix='consumer-secret-')
        consumer = Consumer(user=user, key=consumer_key,
                            secret=consumer_secret)
        if save:
            consumer.save()
        token_string = self.get_unique_string(prefix='token-')
        token_secret = self.get_unique_string(prefix='token-secret-')
        token_name = self.get_unique_string(prefix='token-name-')
        token = Token(consumer=consumer, token=token_string,
                      token_secret=token_secret, name=token_name)
        if save:
            token.save()
        return token, consumer

    def make_nonce(self, token=None, consumer=None, created_at=None,
                   nonce_string=None):
        if token is None or consumer is None:
            assert token is None and consumer is None
            token, consumer = self.make_oauth_token_and_consumer()
        if nonce_string is None:
            nonce_string = self.get_unique_string(prefix='nonce-')

        nonce = Nonce.objects.create(
            token=token, consumer=consumer, nonce=nonce_string)
        if created_at:
            nonce.created_at = created_at
            nonce.save()
        return nonce

    def make_session(self, expire_date=None):
        if expire_date is None:
            secs = self.get_unique_integer()
            expire_date = datetime.now(pytz.utc) + timedelta(seconds=secs)
        return Session.objects.create(
            session_key=self.get_unique_string(prefix='key-'),
            session_data=self.get_unique_string(prefix='session-data-'),
            expire_date=expire_date
        )

    def make_app_widget(self, applications=None, template_snippet=None):
        if applications is None:
            applications = [self.make_application()]
        if template_snippet is None:
            template_snippet = u"Promotion for {{ application.name }}"

        widget = ApplicationWidget.objects.create(
            template_snippet=template_snippet)
        widget.applications = applications


class TestCaseWithFactory(TestCase):

    def setUp(self):
        super(TestCaseWithFactory, self).setUp()
        self.factory = WebCatalogObjectFactory()

    def make_user(self, *args, **kwargs):
        """Invokes self.factory.make_user(), but can also log the user in"""
        logged_in = kwargs.pop('logged_in', False)
        password = kwargs.get('password', 'test')
        result = self.factory.make_user(*args, **kwargs)
        if logged_in:
            self.client.login(username=result.username, password=password)
        return result


class TxTestCaseWithFactory(TransactionTestCase):

    def setUp(self):
        super(TxTestCaseWithFactory, self).setUp()
        self.factory = WebCatalogObjectFactory()
