#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Apps Directory API tests."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'AppMetadataTestCase',
    'DeleteMachineTestCase',
    'GetMachineTestCase',
    'ListPackagesTestCase',
    'ListMachinesTestCase',
    'ServerStatusTestCase',
    'UpdateMachineTestCase',
    'UpdatePackageListTestCase',
]

import json
from django.utils import simplejson
from django.test import TestCase
from oauth.oauth import (
    OAuthRequest,
    OAuthConsumer,
    OAuthToken,
    OAuthSignatureMethod_PLAINTEXT,
)

from .factory import TestCaseWithFactory
from webcatalog.models import Machine


class ServerStatusTestCase(TestCase):
    def test_server_status(self):
        response = self.client.get('/cat/api/1.0/server-status/')
        self.assertEqual(response.content, '"ok"')


class AuthenticatedAPITestCase(TestCaseWithFactory):
    def auth_header_for_user(self, url, user=None, realm='OAuth'):
        token, consumer = self.factory.make_oauth_token_and_consumer(user=user)
        oaconsumer = OAuthConsumer(token.consumer.key, token.consumer.secret)
        oatoken = OAuthToken(token.token, token.token_secret)
        oarequest = OAuthRequest.from_consumer_and_token(
            oaconsumer, oatoken, http_url=url)
        oarequest.sign_request(
            OAuthSignatureMethod_PLAINTEXT(), oaconsumer, oatoken)
        header = oarequest.to_header(realm)
        return {'HTTP_AUTHORIZATION': header['Authorization']}


class ListMachinesTestCase(AuthenticatedAPITestCase):
    url = '/cat/api/1.0/list-machines/'

    def test_no_auth_returns_401(self):
        response = self.client.get(self.url)
        self.assertEqual(401, response.status_code)

    def test_read_no_machines(self):
        user = self.factory.make_user()
        response = self.client.get(
            self.url, **self.auth_header_for_user(self.url, user=user))
        self.assertEqual('[]', response.content)

    def test_read_multiple_machines(self):
        user = self.factory.make_user()
        machine1 = self.factory.make_machine(owner=user)
        machine2 = self.factory.make_machine(owner=user)
        response = self.client.get(
            self.url, **self.auth_header_for_user(self.url, user=user))
        data = simplejson.loads(response.content)
        self.assertEqual(2, len(data))
        expected = set([machine1.uuid, machine2.uuid])
        self.assertEqual(expected, set([x['uuid'] for x in data]))

    def test_only_returns_machines_for_the_authenticated_user(self):
        mymachine = self.factory.make_machine()
        othermachine = self.factory.make_machine()
        response = self.client.get(
            self.url,
            **self.auth_header_for_user(self.url, user=mymachine.owner))
        data = simplejson.loads(response.content)

        self.assertEqual(1, len(data))
        self.assertEqual(mymachine.uuid, data[0]['uuid'])


class UpdateMachineTestCase(AuthenticatedAPITestCase):
    url = '/cat/api/1.0/machine/%s/'

    def test_no_auth_returns_401(self):
        machine = self.factory.make_machine()
        response = self.client.get(self.url % machine.uuid)
        self.assertEqual(401, response.status_code)

    def test_update_success(self):
        machine = self.factory.make_machine()
        data = simplejson.dumps({'hostname': machine.hostname + '-updated'})
        url = self.url % machine.uuid

        response = self.client.post(
            url, data=data, content_type='application/json',
            **self.auth_header_for_user(url, user=machine.owner))

        data = simplejson.loads(response.content)
        self.assertEqual(machine.hostname + '-updated', data['hostname'])

    def test_creates_new_machine_if_not_owner(self):
        machine = self.factory.make_machine()
        otheruser = self.factory.make_user()
        data = simplejson.dumps({'hostname': machine.hostname + '-updated'})
        url = self.url % machine.uuid

        response = self.client.post(
            url, data=data, content_type='application/json',
            **self.auth_header_for_user(url, user=otheruser))

        data = simplejson.loads(response.content)
        self.assertEqual('', data['packages_checksum'])


class DeleteMachineTestCase(AuthenticatedAPITestCase):
    url = '/cat/api/1.0/machine/%s/'

    def test_no_auth_returns_401(self):
        machine = self.factory.make_machine()
        response = self.client.delete(self.url % machine.uuid)
        self.assertEqual(401, response.status_code)

    def test_delete_success(self):
        machine = self.factory.make_machine()
        url = self.url % machine.uuid

        response = self.client.delete(
            url, **self.auth_header_for_user(url, user=machine.owner))

        self.assertEqual(204, response.status_code)
        self.assertRaises(Machine.DoesNotExist, Machine.objects.get,
                          uuid=machine.uuid, owner=machine.owner)

    def test_delete_other_users_machine_fails(self):
        machine = self.factory.make_machine()
        otheruser = self.factory.make_user()
        url = self.url % machine.uuid

        response = self.client.delete(
            url, **self.auth_header_for_user(url, user=otheruser))

        self.assertEqual(404, response.status_code)


class GetMachineTestCase(AuthenticatedAPITestCase):
    url = '/cat/api/1.0/machine/%s/'

    def test_no_auth_returns_401(self):
        machine = self.factory.make_machine()
        response = self.client.get(self.url % machine.uuid)
        self.assertEqual(401, response.status_code)

    def test_get_success(self):
        machine = self.factory.make_machine()
        url = self.url % machine.uuid

        response = self.client.get(
            url, **self.auth_header_for_user(url, user=machine.owner))

        data = simplejson.loads(response.content)
        self.assertEqual(machine.hostname, data['hostname'])
        self.assertEqual(machine.logo_checksum, data['logo_checksum'])

    def test_get_other_users_machine_fails(self):
        machine = self.factory.make_machine()
        otheruser = self.factory.make_user()
        url = self.url % machine.uuid

        response = self.client.get(
            url, **self.auth_header_for_user(url, user=otheruser))

        self.assertEqual(404, response.status_code)


class ListPackagesTestCase(AuthenticatedAPITestCase):
    url = '/cat/api/1.0/packages/%s/'

    def test_no_auth_returns_401(self):
        machine = self.factory.make_machine()
        response = self.client.get(self.url % machine.uuid)
        self.assertEqual(401, response.status_code)

    def test_get_success(self):
        expected = 'some-random-package-list'
        machine = self.factory.make_machine(package_list=expected)
        url = self.url % machine.uuid

        response = self.client.get(
            url, **self.auth_header_for_user(url, user=machine.owner))

        data = simplejson.loads(response.content)
        self.assertEqual(expected, data)

    def test_get_other_users_package_list_fails(self):
        machine = self.factory.make_machine()
        otheruser = self.factory.make_user()
        url = self.url % machine.uuid

        response = self.client.get(
            url, **self.auth_header_for_user(url, user=otheruser))

        self.assertEqual(404, response.status_code)


class UpdatePackageListTestCase(AuthenticatedAPITestCase):
    url = '/cat/api/1.0/packages/%s/'

    def test_no_auth_returns_401(self):
        machine = self.factory.make_machine()
        response = self.client.post(self.url % machine.uuid)
        self.assertEqual(401, response.status_code)

    def test_post_success(self):
        expected = 'some-random-package-list'
        machine = self.factory.make_machine(package_list=expected)
        url = self.url % machine.uuid
        data = simplejson.dumps({'package_list': expected,
                                 'packages_checksum': 'foo'})

        response = self.client.post(
            url, data=data, content_type='application/json',
            **self.auth_header_for_user(url, user=machine.owner))

        self.assertContains(response, 'Success')
        updated = Machine.objects.get(uuid=machine.uuid, owner=machine.owner)
        self.assertEqual(expected, updated.package_list)

    def test_get_other_users_package_list_fails(self):
        machine = self.factory.make_machine()
        otheruser = self.factory.make_user()
        url = self.url % machine.uuid

        response = self.client.post(
            url, data='"foo"', content_type='application/json',
            **self.auth_header_for_user(url, user=otheruser))

        self.assertEqual(404, response.status_code)


class AppMetadataTestCase(TestCaseWithFactory):
    def test_is_public(self):
        response = self.client.get('/cat/api/1.0/apps-meta/')

        self.assertEqual(200, response.status_code)

    def test_contains_right_fields(self):
        self.factory.make_application(is_latest=True)

        response = self.client.get('/cat/api/1.0/apps-meta/')

        data = json.loads(response.content)
        self.assertEqual(1, len(data))
        expected = set(('package_name', 'categories'))
        self.assertEqual(expected, set(data[0].keys()))

    def test_gzipped_content(self):
        # Generate some content so that we gzip the content
        distroseries = self.factory.make_distroseries(code_name='rabbid')
        for i in range(4):
            self.factory.make_application(distroseries=distroseries)
        url = '/cat/api/1.0/apps-meta/%s/' % distroseries.code_name

        response = self.client.get(url, HTTP_ACCEPT_ENCODING='gzip')

        self.assertEqual('gzip', response['content-encoding'])
