#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test cases for the Apps Directory management commands."""

from __future__ import absolute_import

import apt
import json
import os
import pytz
import shutil
import tempfile

from datetime import (
    datetime,
    timedelta,
)
from decimal import Decimal

from django.conf import settings
from django.contrib.sessions.models import Session
from django.core.management import call_command
from django.db import connection
from django.utils.unittest import skipUnless
from django.test.testcases import TransactionTestCase

from mock import (
    call,
    patch,
    MagicMock,
    Mock,
)
from rnrclient import ReviewsStats

from webcatalog.models import (
    Application,
    DistroSeries,
    Exhibit,
    Nonce,
    ReviewStatsImport,
)
from webcatalog.management.commands import (
    import_app_install_data,
    import_ratings_stats,
)
from webcatalog.tests.factory import (
    TestCaseWithFactory,
    WebCatalogObjectFactory,
)
from webcatalog.tests.helpers import patch_settings

__metaclass__ = type
__all__ = [
    'CheckAllLatestTestCase',
    'CleanupTestCase',
    'CleanupDataTestCase',
    'ImportAppInstallTestCase',
    'ImportExhibitsTestCase',
    'ImportFromSCAAppsTestCase',
    'ImportRatingsTestCase',
    'ImportAllAppInstallDataTestCase',
    'ImportAllRatingsStatsTestCase',
]

TEST_VERSIONS = {
    '99.99': 'onion',
    '88.88': 'testwebcatseries',
    '11.04': 'natty',
    '10.10': 'maverick',
    '11.10': 'oneiric',
    '12.04': 'precise',
}


class ImportAppInstallTestCase(TestCaseWithFactory):

    FIREFOX_DESCRIPTION = (
        u'Firefox delivers safe, easy web browsing. A familiar user '
        u'interface, enhanced security features including protection '
        u'from online identity theft, and integrated search let you '
        u'get the most out of the web.'
    )

    # If you want to run these tests against the real apt cache (which
    # is slower, but the real deal) you can update the following
    # attribute. Note: 3 tests will fail which make assumptions about
    # the uri for apt-install-data or the value of descriptions for
    # firefox and scribus in the cache.
    use_mock_apt_cache = True

    def make_mock_apt_package(self, name, description, uri='', summary='',
                              screenshot_url=''):
        """Helper to DRY up creating a mock apt package."""
        mock_package = Mock(spec=apt.package.Package)
        mock_package.candidate.description = description
        mock_package.candidate.uri = uri
        mock_package.name = name
        mock_package.candidate.section = ''
        mock_package.candidate.summary = summary
        mock_package.candidate.version = '1.2.3-4ubuntu1~dev'
        apt_record = {}
        if screenshot_url:
            apt_record['Screenshot-Url'] = screenshot_url
        mock_package.candidateRecord = apt_record
        return mock_package

    def make_mock_apt_cache(self):
        """Helper creating a mock apt_cache object.

        And keeping the setUp readable.
        """
        mock_cache = MagicMock()
        mock_apt_firefox = self.make_mock_apt_package(
            'firefox', description=self.FIREFOX_DESCRIPTION)
        mock_apt_scribus = self.make_mock_apt_package(
            'scribus', description="Scribus description")
        mock_other_app = self.make_mock_apt_package(
            'otherapp', description="Otherapp description",
            summary="Otherapp the Internet\nA tagline for Otherapp",
            screenshot_url="http://example.com/screenshot_for_otherapp.png")
        mock_app_install_data = self.make_mock_apt_package(
            'app-install-data', description="App install desc.",
            uri='http://example.com/app-install-1.01.deb')
        cache_dict = {
            'firefox': mock_apt_firefox,
            'scribus': mock_apt_scribus,
            'otherapp': mock_other_app,
            'app-install-data': mock_app_install_data,
        }

        def getitem(self, key):
            return cache_dict.get(key, None)
        mock_cache.__getitem__ = getitem

        # XXX 2011-06-23 michaeln Can't assign a bound method to a magic
        # mock method. If the following issue is fixed, we'll be able to
        # do `mock_cache.__iter__ = cache_dict.itervalues` without a
        # MagicMock.
        # https://code.google.com/p/mock/issues/detail?id=98
        mock_cache.__iter__.return_value = cache_dict.itervalues()

        return mock_cache

    def setUp(self):
        """Create a mock apt-cache so we don't wait for the update."""
        super(ImportAppInstallTestCase, self).setUp()
        self.deb_location = self.factory.get_test_path(
            'app-install-data-test_all.deb')

        # We use a temporary on-disk cache location for our
        # tests.
        self.tmp_apt_cache = tempfile.mkdtemp()
        self.patch_cache_setting = patch.object(
            settings, 'DISK_APT_CACHE_LOCATION', self.tmp_apt_cache)
        self.patch_cache_setting.start()

        self.patch_ubuntu_version_setting = patch.object(
            settings, 'UBUNTU_SERIES_FOR_VERSIONS', TEST_VERSIONS)
        self.patch_ubuntu_version_setting.start()

        if self.use_mock_apt_cache:
            # The apt.Cache class is patched to return our own mock which
            # will be subscriptable with a pre-set dict with mock ock apt
            # applications for firefox and scribus.
            self.patch_cache = patch('apt.Cache')
            self.mock_cache_class = self.patch_cache.start()
            self.mock_cache = self.make_mock_apt_cache()
            self.mock_cache_class.return_value = self.mock_cache

    def tearDown(self):
        self.patch_cache_setting.stop()
        self.patch_ubuntu_version_setting.stop()
        shutil.rmtree(self.tmp_apt_cache)
        if self.use_mock_apt_cache:
            self.patch_cache.stop()
            super(ImportAppInstallTestCase, self).tearDown()

    def test_app_data_downloaded_by_default(self):
        get_data_fn = ('webcatalog.management.commands.'
                       'import_app_install_data.Command.'
                       'get_package_data_for_series')
        with patch(get_data_fn) as mock_get_data_fn:
            mock_get_data_fn.return_value = self.deb_location
            call_command('import_app_install_data', 'onion', verbosity=0)

        self.assertEqual(2, mock_get_data_fn.call_count)
        calls = ('app-install-data', 'app-install-data-partner')
        for call_args, pkgname in zip(mock_get_data_fn.call_args_list, calls):
            args = call_args[0]
            self.assertEqual(pkgname, args[0])
            self.assertEqual('onion', args[1])
            self.assertTrue(args[2].startswith('/tmp/tmp'))

    def test_get_package_data_for_series(self):
        # Requested and downloaded from the server.
        tmp_dir = tempfile.mkdtemp()
        get_uri_fn = (
            'webcatalog.management.commands.import_app_install_data.'
            'Command.get_package_uri_for_series')
        with patch(get_uri_fn) as mock_get_uri:
            mock_get_uri.return_value = 'http://example.com/my.deb'
            with patch('urllib.urlretrieve') as mock_urlretrieve:
                import_app_install_data.Command().get_package_data_for_series(
                    'app-install-data', 'natty', tmp_dir)
        shutil.rmtree(tmp_dir)

        mock_urlretrieve.assert_called_with(
            'http://example.com/my.deb', os.path.join(tmp_dir, 'my.deb'))

    def test_local_app_install_data(self):
        # We don't hit the network when provided a local app-install deb.
        with patch('urllib.urlretrieve') as mock_urlretrieve:
            call_command(
                'import_app_install_data', 'natty',
                local_app_install_data=self.deb_location,
                local_app_install_data_partner=self.deb_location,
                verbosity=0)

        self.assertEqual(0, mock_urlretrieve.call_count)

    def test_applications_created(self):
        self.assertEqual(0, Application.objects.count())

        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        self.assertEqual(4, Application.objects.count())
        scribus = Application.objects.get(package_name='scribus')
        self.assertEqual("Scribus (Stable)", scribus.name)
        self.assertEqual(
            'Graphic Page Layout and Publication (Stable)',
            scribus.comment)

    def test_applications_updated(self):
        # If applications already exist for the same series, it is
        # updated.
        app = self.factory.make_application(
            package_name='scribus', comment='old comment',
            distroseries=self.factory.make_distroseries(code_name='natty'))

        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        app_reloaded = Application.objects.get(id=app.id)
        self.assertEqual(
            'Graphic Page Layout and Publication (Stable)',
            app_reloaded.comment)

    def test_same_app_different_distroseries(self):
        # Importing the same app (package_name) as an existing app, but
        # for a different distroseries creates a new record.
        app = self.factory.make_application(
            package_name='scribus', comment='old comment',
            distroseries=self.factory.make_distroseries(code_name='natty'))

        call_command(
            'import_app_install_data', 'oneiric',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        app_reloaded = Application.objects.get(id=app.id)
        self.assertEqual('old comment', app_reloaded.comment)
        self.assertEqual(
            2, Application.objects.filter(package_name='scribus').count())

    def test_distroseries_added_to_app(self):
        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        self.assertEqual(4, Application.objects.count())
        scribus = Application.objects.get(package_name='scribus')
        self.assertEqual('natty', scribus.distroseries.code_name)

    def test_icon_added_to_model(self):
        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        self.assertEqual(4, Application.objects.count())
        scribus = Application.objects.get(package_name='scribus')
        firefox = Application.objects.get(package_name='firefox')
        self.assertTrue(firefox.icon.size > 1)
        self.assertTrue(scribus.icon.size > 1)

    def test_screenshot_added_to_model(self):
        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        otherapp = Application.objects.get(package_name='otherapp')
        self.assertEqual(1, otherapp.applicationmedia_set.count())
        app_media = otherapp.applicationmedia_set.all()[0]
        self.assertEqual('screenshot', app_media.media_type)
        self.assertEqual(
            "http://example.com/screenshot_for_otherapp.png",
            app_media.url)

    def get_package_uri_for_series(self):
        uri = import_app_install_data.Command().get_package_uri_for_series(
            'app-install-data', 'natty')

        self.assertEqual('http://example.com/app-install-1.01.deb', uri)

    def test_description_added_to_app(self):
        self.assertEqual(0, Application.objects.count())

        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        self.assertEqual(4, Application.objects.count())
        firefox = Application.objects.get(package_name='firefox')
        scribus = Application.objects.get(package_name='scribus')
        self.assertEqual(self.FIREFOX_DESCRIPTION, firefox.description)
        self.assertEqual("Scribus description", scribus.description)
        # The name and comment come directly from the desktop file when
        # it exists.
        self.assertEqual("Firefox Web Browser", firefox.name)
        self.assertEqual("Browse the World Wide Web", firefox.comment)

    def test_description_updated_in_app(self):
        app = self.factory.make_application(
            package_name='scribus',
            distroseries=self.factory.make_distroseries(code_name='natty'))

        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        app_reloaded = Application.objects.get(id=app.id)
        self.assertEqual(
            'Graphic Page Layout and Publication (Stable)',
            app_reloaded.comment)
        self.assertEqual("Scribus description", app_reloaded.description)

    def test_cache_maker_creates_sources_list(self):
        # If the on-disk cache doesn't yet exist, it is created.
        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        natty_cache = os.path.join(self.tmp_apt_cache, 'natty')
        sources_list_path = os.path.join(natty_cache, 'etc', 'apt',
                                         'sources.list')
        self.assertTrue(os.path.exists(os.path.join(sources_list_path)))
        with open(sources_list_path) as content:
            sources_list_content = content.read()
        sources = [
            '',  # empty line
            'http://archive.ubuntu.com/ubuntu natty main universe',
            'http://archive.ubuntu.com/ubuntu natty-updates main universe',
            'http://archive.ubuntu.com/ubuntu natty-backports main universe',
            'http://archive.canonical.com/ubuntu natty partner',
            'http://extras.ubuntu.com/ubuntu natty main',
        ]
        self.assertEqual('\ndeb '.join(sources) + '\n', sources_list_content)
        if self.use_mock_apt_cache:
            self.mock_cache_class.assert_called_with(rootdir=natty_cache)
            self.assertEqual(1, self.mock_cache.update.call_count)

    def test_cache_maker_doesnt_overwrite_existing(self):
        # If an on-disk cache for the distroseries already exists, a
        # new one is *not* created.
        series_cache = os.path.join(self.tmp_apt_cache, 'testwebcatseries')
        if os.path.exists(series_cache):
            shutil.rmtree(series_cache)
        os.makedirs(series_cache)

        create_apt_cache_fn = (
            'webcatalog.management.commands.import_app_install_data.'
            'CacheMaker.create_apt_cache')
        with patch(create_apt_cache_fn) as mock_create_apt_cache:
            call_command(
                'import_app_install_data', 'testwebcatseries',
                local_app_install_data=self.deb_location,
                local_app_install_data_partner=self.deb_location,
                verbosity=0)

        self.assertEqual(0, mock_create_apt_cache.call_count)
        if self.use_mock_apt_cache:
            self.mock_cache_class.assert_called_with(rootdir=series_cache)
            self.assertEqual(1, self.mock_cache.update.call_count)

    def test_update_from_cache_creates_distroseries_when_required(self):
        self.assertEqual(0, DistroSeries.objects.filter(
            code_name='natty').count())

        import_app_install_data.Command().update_from_cache('natty')

        self.assertEqual(1, DistroSeries.objects.filter(
            code_name='natty').count())

    def test_apt_cache_apps_used_also(self):
        self.assertEqual(0, Application.objects.count())

        import_app_install_data.Command().update_from_cache('natty')

        # There are 4 packages in our mock_cache which is patched in the
        # setUp.
        self.assertEqual(4, Application.objects.count())
        # As cache entries don't include the app name or comment, the
        # summary is split to create those fields.
        otherapp = Application.objects.get(package_name='otherapp')
        self.assertEqual('Otherapp the Internet', otherapp.name)
        self.assertEqual('A tagline for Otherapp', otherapp.comment)
        self.assertEqual('1.2.3', otherapp.version)

    def test_lang_specific_description_added_to_app(self):
        # Related to bug 879268 - our app server runs on lucid where
        # python-apt doesn't support language-specific descriptions
        # which have been used since oneiric.
        self.assertEqual(0, Application.objects.count())

        call_command(
            'import_app_install_data', 'precise',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        firefox = Application.objects.get(package_name='firefox')
        self.assertEqual(self.FIREFOX_DESCRIPTION, firefox.description)

    def _make_desktop_file(self, package_name,
                           screenshot_url='http://example.com',
                           desktop_file_version="1.0"):
        desktop_file = tempfile.mktemp()
        self.addCleanup(os.remove, desktop_file)

        with open(desktop_file, 'w') as f:
            f.write("""[Desktop Entry]
X-AppInstall-Package=%s
X-AppInstall-Popcon=149803
X-AppInstall-Section=main
Version=%s
Name=Foo Bar Baz
X-AppInstall-Screenshot-Url=%s
X-Ubuntu-Gettext-Domain=app-install-data
""" % (package_name, desktop_file_version, screenshot_url))
        return desktop_file

    def test_process_desktop_file_with_screenshot_url(self):
        """When processing a desktop file, handle if screenshot_url is set."""
        command = import_app_install_data.Command()
        desktop_file = self._make_desktop_file(
            'foo', screenshot_url='http://foo.com')

        distroseries = self.factory.make_distroseries()
        command.process_desktop_file(desktop_file, '', distroseries)
        # no breakage!

    def test_process_desktop_file_doesnt_override_version(self):
        distroseries = self.factory.make_distroseries(code_name='natty')
        app = self.factory.make_application(
            package_name='inkscape', version="1.6.3a123",
            distroseries=distroseries)
        desktop_file = self._make_desktop_file('inkscape',
                                               desktop_file_version="1.0")

        command = import_app_install_data.Command()
        command.process_desktop_file(desktop_file, '', distroseries)

        app = Application.objects.get(id=app.id)
        self.assertEqual("1.6.3a123", app.version)

    @patch('urllib.urlopen')
    def test_fetch_icon_from_extras(self, mock_urlopen):
        mock_urlopen.return_value.getcode.return_value = 200
        mock_urlopen.return_value.read.return_value = "I'm a png!"
        app = self.factory.make_application()
        candidate = Mock()
        candidate.uri = 'http://extras.ubuntu.com/pool/f/foobar.deb'
        candidate.record = {'Icon': 'foo.png'}
        command = import_app_install_data.Command()

        command.fetch_icon_from_extras(app, candidate)

        self.assertEqual("I'm a png!", app.icon.read())
        self.assertEqual(1, mock_urlopen.call_count)
        mock_urlopen.assert_called_with(
            'http://extras.ubuntu.com/meta/foo.png')

    @patch('urllib.urlopen')
    def test_fetches_nothing_if_no_icon(self, mock_urlopen):
        app = self.factory.make_application()
        candidate = Mock()
        candidate.uri = 'http://extras.ubuntu.com/pool/f/foobar.deb'
        candidate.record = {}
        command = import_app_install_data.Command()

        command.fetch_icon_from_extras(app, candidate)

        self.assertEqual(0, mock_urlopen.call_count)
        self.assertFalse(app.icon)

    @patch('urllib.urlopen')
    def test_fetches_nothing_if_not_in_extras(self, mock_urlopen):
        app = self.factory.make_application()
        candidate = Mock()
        candidate.uri = 'http://archive.ubuntu.com/pool/f/foobar.deb'
        candidate.record = {'Icon': 'foobar.png'}
        command = import_app_install_data.Command()

        command.fetch_icon_from_extras(app, candidate)

        self.assertEqual(0, mock_urlopen.call_count)
        self.assertFalse(app.icon)

    def test_no_overwrite_of_sca_imports(self):
        app = self.factory.make_application(
            name=u'Another name set by SCA', comment=u'A comment set by SCA',
            package_name='scribus',
            distroseries=self.factory.make_distroseries(code_name='natty'),
            imported_from_sca=True)

        call_command(
            'import_app_install_data', 'natty',
            local_app_install_data=self.deb_location,
            local_app_install_data_partner=self.deb_location,
            verbosity=0)

        app_reloaded = Application.objects.get(id=app.id)
        self.assertEqual(u'Another name set by SCA', app_reloaded.name)
        self.assertEqual(
            u'A comment set by SCA',
            app_reloaded.comment)


class ImportFromSCAAppsTestCase(TestCaseWithFactory):

    def setUp(self):
        super(ImportFromSCAAppsTestCase, self).setUp()
        self.natty = self.factory.make_distroseries(
            code_name='natty', version='11.04')
        self.maverick = self.factory.make_distroseries(
            code_name='maverick', version='10.10')

        self.patch_urlopen = patch('urllib.urlopen')
        self.mock_urlopen = self.patch_urlopen.start()

        def mock_urlopen_fn(url):
            """Return predetermined test_data content for given urls."""
            available_apps_url = (settings.SCA_API_URL +
                                  'applications/any/ubuntu/any/any/')
            icon_arb_url = ('http://sc.staging.ubuntu.com/'
                            'site_media/appmedia/2012/04/ktip.png')
            icon_hello_url = ('http://localhost:8000/'
                              'site_media/icons/2011/06/eg_64x64.png')
            filenames = {
                available_apps_url: 'sca_apps.txt',
                icon_arb_url: 'ubuntu-cof.png',
                icon_hello_url: 'ubuntu-cof.png',
            }
            curdir = os.path.dirname(__file__)
            sca_apps_file = os.path.join(curdir, 'test_data', filenames[url])
            with open(sca_apps_file) as content:
                self.response_content = content.read()
            mock_response = Mock()
            mock_response.code = 200
            mock_response.read.return_value = self.response_content
            return mock_response

        self.mock_urlopen.side_effect = mock_urlopen_fn
        self.addCleanup(self.patch_urlopen.stop)

    def test_app_imported_from_sca_is_found(self):
        apps_imported_from_sca = Application.objects.filter(
            imported_from_sca=True)
        self.assertEqual(0, len(apps_imported_from_sca))

        call_command('import_for_purchase_apps')

        app = Application.objects.get(name='MyApp',
                                      distroseries=self.natty)
        self.assertEqual(True, app.imported_from_sca)
        self.assertTrue(app.description.find('hello') > -1)
        self.assertEqual("Proprietary", app.license)

    def test_app_gets_distroseries(self):
        with patch_settings(UBUNTU_SERIES_FOR_VERSIONS=TEST_VERSIONS):
            call_command('import_for_purchase_apps')

        app = Application.objects.get(name='MyApp',
                                      distroseries=self.natty)
        self.assertEqual(2, len(app.available_distroseries()))

    def test_app_gets_only_distroseries_from_the_list_from_settings(self):
        with patch_settings(UBUNTU_SERIES_FOR_VERSIONS={'11.04': 'natty'}):
            call_command('import_for_purchase_apps')

            app = Application.objects.get(
                name='MyApp', distroseries=self.natty)

            self.assertEqual(1, len(app.available_distroseries()))

    def test_app_gets_price(self):
        call_command('import_for_purchase_apps')

        app = Application.objects.get(name='MyApp',
                                      distroseries=self.natty)
        self.assertEqual(Decimal('2.50'), app.price)

    def test_existing_app_gets_updated_by_package_name(self):
        self.factory.make_application(
            package_name='hello', distroseries=self.maverick)
        self.factory.make_application(
            package_name='hello', distroseries=self.natty)
        apps = Application.objects.filter(
            distroseries__in=(self.natty, self.maverick)).count()
        self.assertEqual(2, apps)

        with patch_settings(UBUNTU_SERIES_FOR_VERSIONS=TEST_VERSIONS):
            call_command('import_for_purchase_apps')

        actual_apps = Application.objects.filter(
            distroseries__in=(self.natty, self.maverick))
        self.assertEqual(2, actual_apps.count())
        for app in actual_apps:
            self.assertTrue(app.imported_from_sca)

    def test_app_gets_icon(self):
        call_command('import_for_purchase_apps')

        self.mock_urlopen.assert_called_with(
            'http://sc.staging.ubuntu.com/'
            'site_media/appmedia/2012/04/ktip.png')
        app = Application.objects.get(name='MyApp',
                                      distroseries=self.natty)
        # The urlopen double returns the test_data/ubuntu-cof.png which has
        # a size of 2438.
        self.assertEqual(2438, app.icon.size)

    def test_app_gets_version(self):
        call_command('import_for_purchase_apps')

        app = Application.objects.get(package_name='hello',
                                      distroseries=self.natty)
        self.assertEqual('1.2.3', app.version)

    def test_checks_latest(self):
        call_command('import_for_purchase_apps')

        app = Application.objects.get(package_name='hello',
                                      distroseries=self.natty)
        self.assertTrue(app.is_latest)

    def test_app_gets_screenshots(self):
        call_command('import_for_purchase_apps')

        app = Application.objects.get(package_name='hello',
                                      distroseries=self.natty)
        qs = app.applicationmedia_set.filter(media_type='screenshot')
        self.assertEqual(2, qs.count())

    def test_import_arb_app(self):
        call_command('import_for_purchase_apps')

        arb_apps = Application.objects.filter(package_name='eg_arb_app')
        self.assertEqual(2, arb_apps.count())
        self.assertEqual(None, arb_apps[0].archive_id)
        self.assertEqual(None, arb_apps[1].archive_id)

    def test_import_arb_app_handle_existing_none_archive_id(self):
        precise = self.factory.make_distroseries(code_name='precise')
        self.factory.make_application(archive_id=None, distroseries=precise)
        self.factory.make_application(archive_id=None, distroseries=precise)

        call_command('import_for_purchase_apps')

        arb_apps = Application.objects.filter(package_name='eg_arb_app')
        self.assertEqual(2, arb_apps.count())

    def test_import_sca_apps_updates_depts(self):
        call_command('import_for_purchase_apps')

        app = Application.objects.get(package_name='hello',
                                      distroseries=self.natty)
        self.assertEqual(['Books & Magazines'],
                         [dept.name for dept in app.departments.all()])

    def test_no_longer_existing_apps_are_removed(self):
        precise = self.factory.make_distroseries(code_name='precise')
        old_app = self.factory.make_application(
            package_name='hello', distroseries=precise,
            imported_from_sca=True)

        with patch_settings(UBUNTU_SERIES_FOR_VERIONS={
            '10.04': 'lucid',
            '11.04': 'natty',
        }):
            call_command('import_sca_apps')

        self.assertEqual(0, Application.objects.filter(pk=old_app.id).count())
        remaining_hello_apps = Application.objects.filter(
            imported_from_sca=True, package_name='hello')
        self.assertEqual(1, remaining_hello_apps.count())
        self.assertEqual('natty',
                         remaining_hello_apps[0].distroseries.code_name)


class ImportRatingsTestCase(TestCaseWithFactory):

    def setUp(self):
        super(ImportRatingsTestCase, self).setUp()
        review_stats_fn = 'rnrclient.RatingsAndReviewsAPI.review_stats'
        self.ratings_stats_patcher = patch(review_stats_fn)
        self.mock_review_stats = self.ratings_stats_patcher.start()
        self.mock_review_stats.return_value = []

        self.patch_ubuntu_version_setting = patch.object(
            settings, 'UBUNTU_SERIES_FOR_VERSIONS', TEST_VERSIONS)
        self.patch_ubuntu_version_setting.start()

    def tearDown(self):
        self.ratings_stats_patcher.stop()
        self.patch_ubuntu_version_setting.stop()
        super(ImportRatingsTestCase, self).tearDown()

    def test_creates_last_import_record(self):
        onion = self.factory.make_distroseries(code_name='onion')
        self.assertEqual(
            0, ReviewStatsImport.objects.filter(distroseries=onion).count())

        call_command('import_ratings_stats', 'onion')

        self.assertEqual(
            1, ReviewStatsImport.objects.filter(distroseries=onion).count())

    def test_updates_last_import_record(self):
        onion = self.factory.make_distroseries(code_name='onion')
        orig_timestamp = datetime(2011, 07, 18, 14, 43, tzinfo=pytz.utc)
        ReviewStatsImport.objects.create(
            distroseries=onion, last_import=orig_timestamp)

        call_command('import_ratings_stats', 'onion')

        import_records = ReviewStatsImport.objects.filter(distroseries=onion)
        self.assertEqual(1, import_records.count())
        self.assertTrue(orig_timestamp < import_records[0].last_import)

    def test_download_review_stats_no_previous(self):
        # If there have been no previous imports, the complete stats
        # will be retrieved.
        self.factory.make_distroseries(code_name='onion')

        call_command('import_ratings_stats', 'onion')

        self.mock_review_stats.assert_called_with(
            origin='any', distroseries='onion')

    def test_download_review_stats_with_recent(self):
        # If there has been a recent import, we'll only grab the
        # relevant stats.
        onion = self.factory.make_distroseries(code_name='onion')
        orig_timestamp = datetime.now(pytz.utc) - timedelta(2)
        ReviewStatsImport.objects.create(
            distroseries=onion, last_import=orig_timestamp)

        call_command('import_ratings_stats', 'onion')

        self.mock_review_stats.assert_called_with(
            origin='any', distroseries='onion', days=3)

    def test_force_download_all_stats(self):
        # If the download_all option is used, all stats are
        # downloaded, rather than the optimised last X days.
        onion = self.factory.make_distroseries(code_name='onion')
        orig_timestamp = datetime.now(pytz.utc) - timedelta(2)
        ReviewStatsImport.objects.create(
            distroseries=onion, last_import=orig_timestamp)

        call_command('import_ratings_stats', 'onion', download_all=True)

        self.mock_review_stats.assert_called_with(origin='any',
                                                  distroseries='onion')

    def test_update_app_stats(self):
        # Any stats provided in the api call will be used to update our
        # apps in the db.
        natty = self.factory.make_distroseries(code_name='natty')
        scribus = self.factory.make_application(package_name='scribus',
                                                distroseries=natty)
        otherapp = self.factory.make_application(package_name='otherapp',
                                                 distroseries=natty)

        d = dict(package_name='scribus', ratings_average='4.00',
                 ratings_total=4, histogram='[0, 1, 0, 1, 2]')
        scribus_stats = ReviewsStats.from_dict(d)
        self.mock_review_stats.return_value = [scribus_stats]
        call_command('import_ratings_stats', 'natty')

        scribus = Application.objects.get(id=scribus.id)
        self.assertEqual(Decimal('4.00'), scribus.ratings_average)
        self.assertEqual(4, scribus.ratings_total)
        self.assertEqual('[0, 1, 0, 1, 2]', scribus.ratings_histogram)
        self.assertAlmostEqual(3.36480032198111, scribus.wilson_score)
        otherapp = Application.objects.get(id=otherapp.id)
        self.assertEqual(None, otherapp.ratings_total)

    def test_update_works_with_many_stats(self):
        """Ensure update_apps_with_stats doesn't fail with many ratings."""
        natty = self.factory.make_distroseries(code_name='natty')
        d = dict(
            package_name=self.factory.get_unique_string(prefix='package-'),
            ratings_average='5.00', ratings_total=1,
            histogram='[0, 0, 0, 0, 1]',
        )
        stats = [ReviewsStats.from_dict(d.copy()) for x in range(3000)]
        command = import_ratings_stats.Command()

        # update_apps_with_stats returns None on success:
        self.assertIsNone(command.update_apps_with_stats(natty, stats))

    def test_update_works_with_null_histogram(self):
        """Ensure update_apps_with_stats works with null histogram data"""
        app = self.factory.make_application()
        stats = [ReviewsStats.from_dict(dict(package_name=app.package_name,
                                             ratings_average='5.00',
                                             ratings_total=1, histogram=None))]
        command = import_ratings_stats.Command()

        result = command.update_apps_with_stats(app.distroseries, stats)
        self.assertIsNone(result)

    def test_apps_with_no_reviews_end_up_with_minus_one(self):
        # Apps with no reviews should end up with wilson_score = -1
        natty = self.factory.make_distroseries(code_name='natty')
        app = self.factory.make_application(wilson_score=3.5,
                                            distroseries=natty)
        another_app = self.factory.make_application(wilson_score=3.5)

        call_command('import_ratings_stats', 'natty')

        retrieved = Application.objects.get(pk=app.id)
        self.assertEqual(-1, retrieved.wilson_score)
        retrieved = Application.objects.get(pk=another_app.id)
        self.assertEqual(3.5, retrieved.wilson_score)


class ImportExhibitsTestCase(TestCaseWithFactory):

    def setUp(self):
        super(ImportExhibitsTestCase, self).setUp()
        self.natty = self.factory.make_distroseries(
            code_name='natty', version='11.04')

    def mock_response(self, exhibits, changes=None):
        response = Mock()
        response.code = 200
        data = []
        for exhibit in exhibits:
            ds = exhibit.distroseries.get()
            data.append({
                'package_names': exhibit.package_names,
                'banner_url': exhibit.banner_url,
                'version': ds.version,
                'code_name': ds.code_name,
                'html': exhibit.html,
                'date_created': str(exhibit.date_created),
                'weight': str(exhibit.weight),
                'id': exhibit.sca_id,
            })
        if changes:
            for atts, change in zip(data, changes):
                atts.update(change)
        for xibit in data:
            xibit['distroseries'] = [{'code_name': xibit.pop('code_name'),
                                      'version': xibit.pop('version')}]
        return_value = json.dumps(data)
        response.read.return_value = return_value
        return response

    @patch('urllib.urlopen')
    def test_disables_current_exhibits(self, mock_urlopen):
        self.factory.make_exhibit(published=True, distroseries=self.natty)
        mock_urlopen.return_value = self.mock_response([])

        call_command('import_exhibits')

        retrieved = Exhibit.objects.get()
        self.assertFalse(retrieved.published)

    @patch('urllib.urlopen')
    def test_updates_and_enables_current_exhibits(self, mock_urlopen):
        xibit = self.factory.make_exhibit(
            published=False, distroseries=self.natty)
        new_package_names = self.factory.get_unique_string(prefix='new-')
        new_banner_url = self.factory.get_unique_string(prefix='new-')
        new_html = self.factory.get_unique_string(prefix='new-')
        mock_urlopen.return_value = self.mock_response(
            [xibit], changes=[{'package_names': new_package_names,
                               'banner_url': new_banner_url,
                               'html': new_html, 'weight': 3,
                               'click_url': 'http://example.com/foo'}])
        call_command('import_exhibits')

        retrieved = Exhibit.objects.get()
        self.assertTrue(retrieved.published)
        self.assertEqual(new_package_names, retrieved.package_names)
        self.assertEqual(new_banner_url, retrieved.banner_url)
        self.assertEqual(new_html, retrieved.html)
        self.assertEqual(3, retrieved.weight)
        self.assertEqual('http://example.com/foo', retrieved.click_url)

    @patch('urllib.urlopen')
    def test_creates_exhibit_if_scaid_does_not_match(self, mock_urlopen):
        xibit = self.factory.make_exhibit(
            published=False, distroseries=self.natty)
        new_id = xibit.sca_id + 20
        mock_urlopen.return_value = self.mock_response(
            [xibit], changes=[{'id': new_id}])

        call_command('import_exhibits')

        self.assertEqual(2, Exhibit.objects.all().count())
        self.assertFalse(Exhibit.objects.get(sca_id=xibit.sca_id).published)
        self.assertTrue(Exhibit.objects.get(sca_id=new_id).published)

    @patch('urllib.urlopen')
    def test_display_is_left_untouched(self, mock_urlopen):
        xibits = [self.factory.make_exhibit(display=d, distroseries=self.natty)
                  for d in [False, True, None]]

        mock_urlopen.return_value = self.mock_response(xibits)

        call_command('import_exhibits')

        retrieved = Exhibit.objects.all()
        self.assertEqual(3, len(retrieved))
        self.assertIs(False, retrieved.get(pk=xibits[0].pk).display)
        self.assertIs(True, retrieved.get(pk=xibits[1].pk).display)
        self.assertIs(None, retrieved.get(pk=xibits[2].pk).display)

    @patch('urllib.urlopen')
    def test_banner_url_is_correctly_munged(self, mock_urlopen):
        xibit = self.factory.make_exhibit(distroseries=self.natty)
        html = '<img src="/foo/bar/baz.png">'
        banner_url = 'http://example.com/foo/bar/baz.png'
        expected = '<img src="http://example.com/foo/bar/baz.png">'

        mock_urlopen.return_value = self.mock_response(
            [xibit], changes=[{'banner_url': banner_url, 'html': html}])

        call_command('import_exhibits')

        retrieved = Exhibit.objects.get()
        self.assertEqual(expected, retrieved.html)


class CheckAllLatestTestCase(TestCaseWithFactory):
    def test_updates_all(self):
        natty = self.factory.make_distroseries(code_name='natty',
                                               version='11.04')
        oneiric = self.factory.make_distroseries(code_name='oneiric',
                                                 version='11.10')
        self.factory.make_application(package_name='foo', distroseries=natty)
        self.factory.make_application(package_name='foo', distroseries=oneiric)
        self.factory.make_application(package_name='bar', distroseries=natty,
                                      is_latest=True)
        self.factory.make_application(package_name='baz', distroseries=oneiric)

        call_command('check_all_latest')

        retrieved = Application.objects.filter(package_name='foo').order_by(
            '-distroseries__code_name')
        self.assertEqual([True, False], [app.is_latest for app in retrieved])
        self.assertTrue(Application.objects.get(package_name='bar').is_latest)
        self.assertTrue(Application.objects.get(package_name='baz').is_latest)

    def test_updates_to_latest_released(self):
        for version in ['12.04', '12.10', '13.04', '13.10']:
            dseries = self.factory.make_distroseries(version=version)
            self.factory.make_application('whatnot', distroseries=dseries)
        dseries = self.factory.make_distroseries(version='14.04',
                                                 prerelease=True)
        self.factory.make_application('whatnot', distroseries=dseries)

        call_command('check_all_latest')

        retrieved = Application.objects.filter(package_name='whatnot')
        retrieved = retrieved.order_by('-distroseries__version')
        expected = [False, True, False, False, False]
        self.assertEqual(expected, [app.is_latest for app in retrieved])

    def test_updates_to_latest_to_prerelease_if_none_released(self):
        for version in ['12.04', '12.10', '13.04']:
            dseries = self.factory.make_distroseries(version=version,
                                                     prerelease=True)
            self.factory.make_application('gizmo', distroseries=dseries)

        call_command('check_all_latest')

        retrieved = Application.objects.filter(package_name='gizmo')
        retrieved = retrieved.order_by('-distroseries__version')
        expected = [True, False, False]
        self.assertEqual(expected, [app.is_latest for app in retrieved])


class CleanupTestCase(TestCaseWithFactory):

    @patch('webcatalog.management.commands.cleanup.connection')
    @patch('sys.stdout')
    def run_and_check_output_and_sql(self, func, output, sql, mock_stdout,
                                     mock_connection):
        func()
        actual_output = ''.join(str(call[0][0])
                                for call in mock_stdout.write.call_args_list)
        self.assertEqual(output, actual_output.split('\n'))
        call_list = mock_connection.cursor.return_value.execute.call_args_list
        for expected, call in zip(sql, call_list):
            actual = (' '.join(call[0][0].split()),) + call[0][1:]
            self.assertEqual(expected, actual)

    @skipUnless(connection.vendor == 'postgresql', "Requires postgresql")
    def test_clean_nonces_no_nonces(self):
        def cleanup():
            call_command('cleanup', 'webcatalog_nonce')
        output = ['Opening cursor',
                  'Removed 0 rows (0 total removed). Batch size 1',
                  'All done.', '']
        sql = [
            ('CLOSE ALL',),
            ('DECLARE _oauth_nonce_clean NO SCROLL CURSOR WITH HOLD FOR '
             'SELECT id FROM webcatalog_nonce '
             "WHERE created_at < (CURRENT_TIMESTAMP - INTERVAL '5' HOUR) "
             'ORDER BY created_at',),
            ('FETCH %s FROM _oauth_nonce_clean', [1]),
        ]
        self.run_and_check_output_and_sql(cleanup, output, sql)

    @skipUnless(connection.vendor == 'postgresql', "Requires postgresql")
    def test_clean_session_no_sessions(self):
        def cleanup():
            call_command('cleanup', 'django_session')
        output = ['Opening cursor',
                  'Removed 0 rows (0 total removed). Batch size 1',
                  'All done.', '']
        sql = [
            ('CLOSE ALL',),
            ('DECLARE _django_session_clean NO SCROLL CURSOR WITH HOLD FOR '
             'SELECT session_key FROM django_session WHERE expire_date < '
             'CURRENT_TIMESTAMP ORDER BY expire_date',),
            ('FETCH %s FROM _django_session_clean', [1]),
        ]
        self.run_and_check_output_and_sql(cleanup, output, sql)


class CleanupDataTestCase(TransactionTestCase):
    def setUp(self):
        self.factory = WebCatalogObjectFactory()
        self.patch_stdout = patch('sys.stdout')
        self.mock_stdout = self.patch_stdout.start()
        self.addCleanup(self.patch_stdout.stop)

    def check_expected_output(self):
        expected_output = '''Opening cursor
Removed 1 rows (1 total removed). Batch size 1
Removed 2 rows (3 total removed). Batch size 2
Removed 1 rows (4 total removed). Batch size 3
Removed 0 rows (4 total removed). Batch size 4
All done.
'''
        actual_output = ''.join(
            str(c[0][0]) for c in self.mock_stdout.write.call_args_list)
        self.assertEqual(expected_output, actual_output)

    @skipUnless(connection.vendor == 'postgresql', "Requires postgresql")
    def test_cleanup_nonces(self):
        now = datetime.now(pytz.utc)
        for i in range(8):
            created_at = now - timedelta(seconds=5000 * i)
            self.factory.make_nonce(created_at=created_at)
        self.assertEqual(8, Nonce.objects.count())

        call_command('cleanup', 'webcatalog_nonce')

        self.assertEqual(4, Nonce.objects.count())
        self.check_expected_output()

    @skipUnless(connection.vendor == 'postgresql', "Requires postgresql")
    def test_cleanup_sessions(self):
        self.create_django_sessions()
        self.assertEqual(8, Session.objects.count())

        call_command('cleanup', 'django_session')

        self.assertEqual(4, Session.objects.count())
        self.check_expected_output()

    def create_django_sessions(self):
        now = datetime.now(pytz.utc)
        for i in range(8):
            expire_date = now + timedelta(seconds=5000 * (4 - i))
            self.factory.make_session(expire_date=expire_date)

    @skipUnless(connection.vendor == 'postgresql', "Requires postgresql")
    def test_cleanup_session_when_it_takes_too_long(self):
        self.create_django_sessions()

        call_command('cleanup', 'django_session', batch_secs=0)

        self.assertEqual(4, Session.objects.count())


class ImportAllAppInstallDataTestCase(TestCaseWithFactory):
    @patch('sys.stdout')
    def test_handle(self, mock_stdout):
        patch_attr = ('webcatalog.management.commands.'
                      'import_all_app_install_data.call_command')
        with patch_settings(UBUNTU_SERIES_FOR_VERSIONS=TEST_VERSIONS):
            with patch(patch_attr) as mock_call_command:
                call_command('import_all_app_install_data')

            # In addition to calling import for each series, a final
            # call_command does check_all_latest automatically.
            self.assertEqual(mock_call_command.call_count,
                             len(TEST_VERSIONS) + 1)


class ImportAllRatingsStatsTestCase(TestCaseWithFactory):

    def setUp(self):
        patcher = patch('webcatalog.management.commands.'
                        'import_all_ratings_stats.call_command')
        self.mock_call_command = patcher.start()
        self.addCleanup(patcher.stop)

        patcher = patch('sys.stdout')
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_handle_download(self):
        with patch_settings(UBUNTU_SERIES_FOR_VERSIONS=TEST_VERSIONS):
            call_command('import_all_ratings_stats')

        expected_calls = [
            call('import_ratings_stats', series, verbosity=1,
                 download_all=False) for series in TEST_VERSIONS.values()]
        self.mock_call_command.assert_has_calls(expected_calls,
                                                any_order=True)

    def test_handle_download_all(self):
        with patch_settings(UBUNTU_SERIES_FOR_VERSIONS=TEST_VERSIONS):
            call_command('import_all_ratings_stats', download_all=True)

        expected_calls = [
            call('import_ratings_stats', series, verbosity=1,
                 download_all=True) for series in TEST_VERSIONS.values()]
        self.mock_call_command.assert_has_calls(expected_calls,
                                                any_order=True)
