#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test cases for the Apps Directory forms."""

from __future__ import absolute_import

from django.forms import ValidationError
from django.test import TestCase

from textwrap import dedent

from webcatalog.forms import (
    ArchiveApplicationForm,
    desktop_field_mappings,
    SCAApplicationForm,
    MultiURLField,
)
from webcatalog.models import Application, ApplicationMedia
from webcatalog.tests.factory import TestCaseWithFactory

__metaclass__ = type
__all__ = [
    'ArchiveApplicationFormTestCase',
    'SCAApplicationFormTestCase',
    'MultiURLFieldTestCase',
]


class ArchiveApplicationFormTestCase(TestCaseWithFactory):

    def get_desktop_data(self, overrides):
        data = {
            'Name': self.factory.get_unique_string(prefix='App Name'),
            'X-AppInstall-Package': self.factory.get_unique_string(
                prefix='pkg_name'),
            'X-AppInstall-Popcon': self.factory.get_unique_integer(),
            'X-AppInstall-Section': self.factory.get_unique_string(
                prefix='section'),
            'Type': 'Application',
            'Categories': 'cat1;cat2',
        }
        data.update(overrides)
        return data

    def get_desktop_entry(self, data):
        desktop_entry_template = dedent("""
            [Desktop Entry]
            GenericName=Page Layout (Stable)
            Terminal=false
            Icon=scribus
            X-StandardInstall=false
            StartupWMClass=scribus
            X-KDE-SubstituteUID=false
            X-KDE-Username=

            X-Ubuntu-Gettext-Domain=app-install-data""")
        for key, value in data.items():
            desktop_entry_template += "\n{key}={value}".format(
                key=key, value=value)

        desktop_data = desktop_entry_template.format(**data)

        return desktop_data

    def test_get_form_from_desktop_data(self):
        data = {
            'Name': 'My Package',
            'X-AppInstall-Package': 'mypkg',
        }
        desktop_entry = self.get_desktop_entry(self.get_desktop_data(data))

        form = ArchiveApplicationForm.get_form_from_desktop_data(
            desktop_entry, distroseries=self.factory.make_distroseries())

        self.assertTrue(form.is_valid())
        self.assertEqual('My Package', form.cleaned_data['name'])
        self.assertEqual('mypkg', form.cleaned_data['package_name'])

    def test_get_form_from_desktop_data_with_existing(self):
        # If the desktop data contains some blanks, then the initial
        # data from any existing instance should be used instead.
        app = self.factory.make_application(package_name='mypkg',
                                            description='MyPkg description')
        data = {
            'Name': 'My Package',
            'X-AppInstall-Package': 'mypkg',
        }
        desktop_entry = self.get_desktop_entry(self.get_desktop_data(data))

        form = ArchiveApplicationForm.get_form_from_desktop_data(
            desktop_entry, distroseries=app.distroseries)

        self.assertTrue(form.is_valid())
        self.assertEqual('MyPkg description', form.cleaned_data['description'])

    def test_extra_fields(self):
        # There are a bunch of extra fields which are also supported by
        # the model form.
        extra_desktop_info = {
            'X-AppInstall-Channel': 'channel-name',
            'X-AppInstall-Section': 'section-name',
            'X-AppInstall-Screenshot-Url': 'http://example.com/screenshot',
            'Type': 'Application',
            'X-AppInstall-Description': 'A description',
            'X-AppInstall-Architectures': 'i386;amd64',
            'X-AppInstall-Popcon': 9876,
            'X-AppInstall-Keywords': 'jazz,rock,country',
            'Categories': 'Graphics;Jazz;Publishing',
            'MimeType': 'application/vnd.scribus;text/javascript;'
        }
        info = self.get_desktop_data(extra_desktop_info)
        desktop_entry = self.get_desktop_entry(info)

        form = ArchiveApplicationForm.get_form_from_desktop_data(
            desktop_entry, distroseries=self.factory.make_distroseries())

        self.assertTrue(form.is_valid())
        for key, value in extra_desktop_info.items():
            form_key = desktop_field_mappings.get(key.lower(), key.lower())
            self.assertEqual(
                extra_desktop_info[key], form.cleaned_data[form_key])

    def test_screenshot_url_is_properly_saved_as_application_media(self):
        app = self.factory.make_application()
        extra_desktop_info = {
            'X-AppInstall-Screenshot-Url': 'http://example.com/screenshot',
            'X-AppInstall-Package': app.package_name,
        }
        info = self.get_desktop_data(extra_desktop_info)
        desktop_entry = self.get_desktop_entry(info)

        form = ArchiveApplicationForm.get_form_from_desktop_data(
            desktop_entry, distroseries=app.distroseries)
        form.save()

        app = Application.objects.get(pk=app.pk)

        self.assertTrue('http://example.com/screenshot' in app.screenshots)
        self.assertTrue(ApplicationMedia.objects.filter(
            application=app, media_type='screenshot',
            url='http://example.com/screenshot').exists())

    def test_screenshot_url_isnt_verified(self):
        # Verifying if this URLField actually exists is a bad idea:
        # - It expects us to allow arbitrary outgoing connections from the DC
        # - It adds a significant delay to tests unless carefully mocked
        # - It doesn't provide a real benefit as 99% of the time the field
        #   is populated by a script that doesn't have typo issues.
        distroseries = self.factory.make_distroseries()
        d = dict(screenshot_url='http://foo.com:42/broken',
                 section='required', name='required', package_name='required',
                 distroseries=distroseries.id)
        form = ArchiveApplicationForm(d)
        self.assertTrue(form.is_valid())

    def test_updating_an_sca_imported_app_fails_validation(self):
        app = self.factory.make_application(imported_from_sca=True)
        extra_desktop_info = {
            'X-AppInstall-Package': app.package_name,
        }
        info = self.get_desktop_data(extra_desktop_info)
        desktop_entry = self.get_desktop_entry(info)

        form = ArchiveApplicationForm.get_form_from_desktop_data(
            desktop_entry, distroseries=app.distroseries)

        self.assertFalse(form.is_valid())
        self.assertEqual(
            {'__all__': [u'Attempt to overwrite an sca-imported app.']},
            form.errors)


class SCAApplicationFormTestCase(TestCaseWithFactory):

    def test_from_api_data_shortens_debtag(self):
        app = self.factory.make_application()
        data = self.make_valid_data(debtags=['hardware::storage:cd-writer'])

        form = SCAApplicationForm.from_api_data(data, app.distroseries)

        self.assertEqual(form.data['debtags'], '["CD burner"]')

    def test_save_media_urls(self):
        app = self.factory.make_application(
            screenshot_url="http://example.com/screenshot1.png")
        self.assertEqual(1, app.applicationmedia_set.count())
        data = {
            'screenshot_urls': [
                'http://example.com/screenshot1.png',
                'http://example.com/screenshot2.png',
            ],
            'video_embedded_html_urls': [
                'http://example.com/video1.mp4',
            ],
            'archive_id': app.archive_id,
            'package_name': app.package_name,
            'name': app.name,
        }

        form = SCAApplicationForm.from_api_data(data,
                                                app.distroseries)
        self.assertTrue(form.is_valid())
        form.save_media_urls()

        self.assertEqual(3, app.applicationmedia_set.count())
        actual_urls = [media.url for media in
                       app.applicationmedia_set.all().order_by('url')]
        expected = [
            'http://example.com/screenshot1.png',
            'http://example.com/screenshot2.png',
            'http://example.com/video1.mp4',
        ]
        self.assertEqual(expected, actual_urls)

    def make_valid_data(self, **kwargs):
        media_url = u'http://sc.staging.ubuntu.com/site_media/appmedia/'
        data = {
            u'status': u'Published',
            u'signing_key_id': u'',
            u'description': u'This is a description.',
            u'package_name': self.factory.get_unique_string(prefix='pkg'),
            u'video_embedded_html_urls': [],
            u'series': {
                u'precise': [u'amd64', u'i386'],
                u'oneiric': [u'i386', u'amd64']
            },
            u'price': u'0.00',
            u'debtags': '["mouse"]',
            u'date_published': u'2012-04-30 19:34:08.011454',
            u'video_urls': [],
            u'screenshot_url': media_url + u'2012/04/IMG_20120401_144332.jpg',
            u'id': 595,
            u'categories': u'Utility',
            u'archive_id': self.factory.get_unique_string(prefix='archid/'),
            u'name': u'Test App',
            u'license': u'GNU GPL v3',
            u'support_url': u'http://example.com/foo',
            u'icon_url': media_url + u'2012/04/ktip.png',
            u'comment': u'This is a test app',
            u'version': u'1.2',
            u'screenshot_urls': [
                media_url + u'2012/04/IMG_20120401_144332.jpg',
            ],
            u'demo': None,
            u'department': [u'Accessories'],
            u'archive_root': u'http://extras.ubuntu.com/',
            u'tos_url': u'',
            u'channel': u'Independent',
        }
        data.update(**kwargs)
        return data

    def test_archive_id_is_set_to_none_when_supplied_as_empty_str(self):
        """Check that bug 1010655 is not happening again."""
        precise = self.factory.make_distroseries(code_name='precise')
        self.factory.make_application(archive_id='', distroseries=precise)

        data = self.make_valid_data(archive_id='')

        form = SCAApplicationForm.from_api_data(data, precise)

        self.assertTrue(form.is_valid())


class MultiURLFieldTestCase(TestCase):

    def test_normalize_empty_value(self):
        self.assertEqual(
            [], MultiURLField().to_python(''))
        self.assertEqual(
            [], MultiURLField().to_python(None))
        self.assertEqual(
            [], MultiURLField().to_python([]))

    def test_normalize_expects_list_strings(self):
        self.assertEqual(
            ['foo', 'bar'],
            MultiURLField().to_python(['foo', 'bar']))

    def test_validates_multiple_URLs(self):
        MultiURLField().validate(
            ['http://example.com', 'https://example.com/two'])

        self.assertRaises(
            ValidationError,
            MultiURLField().validate,
            ['httpd://example.com', 'https://example.com/two'])
