#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Apps Directory handlers tests."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'ListMachineHandlerTestCase',
    'MachineHandlerTestCase',
    'MetadataHandlerTestCase',
    'PackagesHandlerTestCase',
    'ServerStatusHandlerTestCase',
]

from django.http import HttpRequest
from django.test import TestCase
from django.contrib.auth.models import User

from webcatalog.models import Machine
from webcatalog.api.handlers import (
    ListMachinesHandler,
    MachineHandler,
    MetadataHandler,
    PackagesHandler,
    ServerStatusHandler,
)
from webcatalog.tests.factory import TestCaseWithFactory


class ServerStatusHandlerTestCase(TestCase):
    def test_read(self):
        ss_handler = ServerStatusHandler()
        self.assertEqual('ok', ss_handler.read(None))


class HandlerTestCase(TestCaseWithFactory):
    def setUp(self):
        super(HandlerTestCase, self).setUp()
        self.user = self.factory.make_user()
        self.request = HttpRequest()
        self.request.user = self.user


class ListMachineHandlerTestCase(HandlerTestCase):
    def test_no_machines_returns_empty_list(self):
        handler = ListMachinesHandler()
        machines = handler.read(self.request)
        self.assertEqual([], list(machines))

    def test_multiple_machines_returns_as_expected(self):
        machine1 = self.factory.make_machine(owner=self.user)
        machine2 = self.factory.make_machine(owner=self.user)
        handler = ListMachinesHandler()

        machines = handler.read(self.request)

        self.assertEqual(2, len(machines))
        expected = set([machine1.uuid, machine2.uuid])
        self.assertEqual(expected, set(x.uuid for x in machines))

    def test_machine_for_user_other_than_the_first(self):
        """Check that we're not returning only machiens for the first user"""
        self.factory.make_user()
        self.factory.make_user()
        # Not the first user in the system:
        user = User.objects.all()[1]
        machine = self.factory.make_machine(owner=user)
        handler = ListMachinesHandler()
        request = HttpRequest()
        request.user = user

        machines = handler.read(request)

        self.assertEqual(1, len(machines))
        self.assertEqual(machine.uuid, machines[0].uuid)


class MachineHandlerTestCase(HandlerTestCase):
    def test_read_invalid_uuid_returns_404(self):
        handler = MachineHandler()
        response = handler.read(self.request, self.factory.get_unique_string())
        self.assertEqual(404, response.status_code)

    def test_read(self):
        handler = MachineHandler()
        machine = self.factory.make_machine(self.user)
        returned = handler.read(self.request, machine.uuid)
        self.assertEqual(machine.uuid, returned.uuid)

    def test_create_no_data(self):
        """Test the case where no data was deserialized"""
        request = HttpRequest()
        handler = MachineHandler()
        response = handler.create(request, uuid='foo')
        self.assertContains(response, "Unable to deserialize request",
                            status_code=400)

    def test_create_missing_hostname(self):
        request = HttpRequest()
        request.data = {'logo_checksum': 'bar'}
        handler = MachineHandler()
        response = handler.create(request, 'uuid')
        expected = {'status': 'error',
                    'errors': {'hostname': [u'This field is required.']}}
        self.assertEqual(expected, response)

    def test_create_blank_logo_checksum(self):
        request = HttpRequest()
        data = {'hostname': 'foo', 'logo_checksum': ''}
        request.data = data
        request.user = self.user
        handler = MachineHandler()
        requested_uuid = 'uuid'

        machine = handler.create(request, requested_uuid)

        self.assertEqual(requested_uuid, machine.uuid)
        self.assertEqual('', machine.logo_checksum)

    def test_create_updates_existing(self):
        machine = self.factory.make_machine(owner=self.user)
        handler = MachineHandler()
        request = HttpRequest()
        request.user = self.user
        changed_hostname = machine.hostname + '-changed'
        request.data = {'hostname': changed_hostname}

        handler.create(request, machine.uuid)

        updated = Machine.objects.get(uuid=machine.uuid)
        self.assertEqual(changed_hostname, updated.hostname)
        self.assertEqual('', updated.logo_checksum)

    def test_delete_invalid_uuid_returns_404(self):
        handler = MachineHandler()
        response = handler.delete(self.request,
                                  self.factory.get_unique_string())
        self.assertEqual(404, response.status_code)

    def test_delete(self):
        machine = self.factory.make_machine(owner=self.user)
        handler = MachineHandler()

        response = handler.delete(self.request, machine.uuid)

        self.assertEqual(204, response.status_code)
        self.assertEqual('', response.content)
        self.assertRaises(Machine.DoesNotExist, Machine.objects.get,
                          uuid=machine.uuid)


class PackagesHandlerTestCase(HandlerTestCase):
    def test_read_invalid_uuid_returns_404(self):
        handler = PackagesHandler()
        response = handler.read(self.request, self.factory.get_unique_string())
        self.assertEqual(404, response.status_code)

    def test_read(self):
        expected = 'some-package-list'
        machine = self.factory.make_machine(owner=self.user,
                                            package_list=expected)
        handler = PackagesHandler()

        response = handler.read(self.request, machine.uuid)
        self.assertEqual(response, expected)

    def test_create_invalid_uuid_returns_404(self):
        handler = PackagesHandler()
        response = handler.create(self.request,
                                  self.factory.get_unique_string())
        self.assertEqual(404, response.status_code)

    def test_create_missing_data(self):
        machine = self.factory.make_machine(owner=self.user)
        request = HttpRequest()
        request.user = self.user
        data = {'packages_checksum': 'bar', 'package_list': 'some-data'}
        handler = PackagesHandler()
        for key in data:
            request.data = dict((k, data[k]) for k in data if k != key)
            response = handler.create(request, machine.uuid)
            self.assertEqual('error', response['status'])
            self.assertEqual([key], response['errors'].keys())

    def test_create_success(self):
        machine = self.factory.make_machine(owner=self.user)
        request = HttpRequest()
        request.user = self.user
        data = {'packages_checksum': 'bar', 'package_list': 'some-data'}
        request.data = data
        handler = PackagesHandler()

        response = handler.create(request, machine.uuid)

        updated = Machine.objects.get(uuid=machine.uuid)
        self.assertEqual('Success', response)
        self.assertEqual(machine.uuid, updated.uuid)
        self.assertEqual(data['packages_checksum'], updated.packages_checksum)
        self.assertEqual(data['package_list'], updated.package_list)

    def test_create_no_data(self):
        """Test the case where no data was deserialized"""
        machine = self.factory.make_machine(owner=self.user)
        request = HttpRequest()
        request.user = self.user
        handler = PackagesHandler()
        response = handler.create(request, uuid=machine.uuid)
        self.assertContains(response, "Unable to deserialize request",
                            status_code=400)


class MetadataHandlerTestCase(HandlerTestCase):
    def test_distroseries_restricts_to_distroseries(self):
        app = self.factory.make_application()
        app2 = self.factory.make_application(distroseries=app.distroseries)
        self.factory.make_application()  # For a different distroseries
        code_name = app.distroseries.code_name

        apps = MetadataHandler().read(self.request, distroseries=code_name)

        expected = set([app.name, app2.name])
        self.assertEqual(expected, set(x.name for x in apps))

    def test_no_distroseries_returns_all_latest(self):
        packages = set(['foo', 'bar', 'baz'])
        names = set()
        for package in packages:
            app = self.factory.make_application(package_name=package,
                                                is_latest=True)
            names.add(app.name)
            self.factory.make_application(package_name=package)

        apps = MetadataHandler().read(self.request)

        self.assertEqual(packages, set(x.package_name for x in apps))
        self.assertEqual(names, set(x.name for x in apps))
