#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test cases for object managers."""

from __future__ import absolute_import

from django.http import Http404

from webcatalog.tests.factory import TestCaseWithFactory
from webcatalog.models import Application

__metaclass__ = type
__all__ = [
    'ApplicationManagerTestCase',
]


class ApplicationManagerTestCase(TestCaseWithFactory):

    def make_apps(self, package_name, versions, prerelease=None):
        results = []
        for version in versions:
            is_prerelease = version == prerelease
            distroseries = self.factory.make_distroseries(
                version=version, prerelease=is_prerelease)
            app = self.factory.make_application(
                package_name, distroseries=distroseries)
            results.append(app)
        return results

    def test_find_best_returns_none(self):
        self.assertIsNone(Application.objects.find_best(package_name='foo'))

    def test_find_best_or_404_raises(self):
        self.assertRaises(Http404, Application.objects.find_best_or_404,
                          package_name='doodle')

    def test_find_best_returns_latest(self):
        expected, _ = self.make_apps('gparted', ['14.10', '14.04'])

        retrieved = Application.objects.find_best(package_name='gparted')

        self.assertEqual(expected.id, retrieved.id)

    def test_find_best_returns_latest_released(self):
        _, expected = self.make_apps('vlc', ['14.10', '14.04'],
                                     prerelease='14.10')

        retrieved = Application.objects.find_best(package_name='vlc')

        self.assertEqual(expected.id, retrieved.id)

    def test_find_best_returns_prerelease_if_no_release_available(self):
        [expected] = self.make_apps('amnesia', ['14.10'],
                                    prerelease='14.10')

        retrieved = Application.objects.find_best(package_name='amnesia')

        self.assertEqual(expected.id, retrieved.id)

    def test_check_latest(self):
        self.make_apps('foobar', ['10.04', '10.10', '11.04', '11.10'],
                       prerelease='11.10')

        Application.objects.check_latest('foobar')

        retrieved = Application.objects.filter(package_name='foobar').order_by(
            '-distroseries__version')
        self.assertEqual([False, True, False, False],
                         [app.is_latest for app in retrieved])
