#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test cases for models."""

from __future__ import absolute_import

from django.core.urlresolvers import reverse
from django.core.files.images import ImageFile

from webcatalog.models import Application
from webcatalog.tests.factory import TestCaseWithFactory
from webcatalog.tests.helpers import patch_settings

__metaclass__ = type
__all__ = [
    'ApplicationTestCase',
    'DepartmentTestCase',
    'ExhibitTestCase',
]


class ApplicationTestCase(TestCaseWithFactory):
    def test_categories_set(self):
        app = self.factory.make_application()
        app.categories = 'foo;;;; bar  '
        self.assertEqual(set(['foo', 'bar']), app.categories_set)

    def test_empty_category_set(self):
        app = self.factory.make_application()
        app.categories = ''
        self.assertEqual(set(), app.categories_set)

    def test_update_empty_departments(self):
        app = self.factory.make_application()

        app.update_departments()

        self.assertEqual(0, app.departments.count())

    def test_update_departments(self):
        app = self.factory.make_application()
        app.categories = 'Game;'

        app.update_departments()

        self.assertEqual(1, app.departments.count())
        self.assertEqual('Games', app.departments.get().name)

    def test_update_departments_for_sca_doesnt_clear(self):
        dept_names = ['dept1', 'dept2']
        departments = [
            self.factory.make_department(name) for name in dept_names]
        app = self.factory.make_application(imported_from_sca=True,
                                            departments=departments)

        app.update_departments()

        self.assertEqual(dept_names,
                         [dept.name for dept in app.departments.all()])

    def test_update_departments_for_sca_from_categories(self):
        # If an sca-imported app doesn't have any departments, it
        # should be updated as per the categories.
        app = self.factory.make_application(imported_from_sca=True)
        app.categories = 'Game;'

        app.update_departments()

        self.assertEqual(1, app.departments.count())
        self.assertEqual('Games', app.departments.get().name)

    def test_update_departments_for_sca_from_departments(self):
        app = self.factory.make_application(imported_from_sca=True)

        app.update_departments(['Books & Magazines'])

        self.assertEqual(1, app.departments.count())
        self.assertEqual('Books & Magazines', app.departments.get().name)

    def test_update_departments_new_name(self):
        dept = self.factory.make_department('Role Spielen',
                                            slug='role-spielen')
        app = self.factory.make_application(
            departments=[dept], imported_from_sca=True)

        app.update_departments(['Role-Spielen'])

        self.assertEqual(app.departments.count(), 1)
        dept = app.departments.get()
        self.assertEqual('role-spielen', dept.slug)
        self.assertEqual('Role-Spielen', dept.name)

    def test_crumbs_no_department(self):
        app = self.factory.make_application()
        app_url = reverse('wc-package-detail',
                          args=[app.distroseries.code_name, app.package_name])
        self.assertEqual([], list(app.departments.all()))
        expected = [
            {'name': 'All Apps', 'url': reverse('wc-index')},
            {'name': app.name, 'url': app_url},
        ]
        self.assertEqual(expected, app.crumbs())

    def test_crumbs_department_without_parent(self):
        app = self.factory.make_application()
        app.categories = 'Game;'
        app.update_departments()
        dept = app.departments.get()
        app_url = reverse('wc-package-detail',
                          args=[app.distroseries.code_name, app.package_name])
        dept_url = reverse('wc-department',
                           kwargs={'distro': app.distroseries.code_name,
                                   'dept_slug_or_id': dept.slug})
        expected = [
            {'name': 'All Apps', 'url': reverse('wc-index')},
            {'name': dept.name, 'url': dept_url},
            {'name': app.name, 'url': app_url},
        ]
        self.assertEqual(expected, app.crumbs())

    def test_crumbs_department_with_parent(self):
        app = self.factory.make_application()
        app.categories = 'RolePlaying;'
        app.update_departments()
        dept = app.departments.get()
        app_url = reverse('wc-package-detail',
                          args=[app.distroseries.code_name, app.package_name])
        dept_url = reverse('wc-department',
                           kwargs={'distro': app.distroseries.code_name,
                                   'dept_slug_or_id': dept.slug})
        parent_url = reverse('wc-department',
                             kwargs={'distro': app.distroseries.code_name,
                                     'dept_slug_or_id': dept.parent.slug})
        expected = [
            {'name': 'All Apps', 'url': reverse('wc-index')},
            {'name': dept.parent.name, 'url': parent_url},
            {'name': dept.name, 'url': dept_url},
            {'name': app.name, 'url': app_url},
        ]
        self.assertEqual(expected, app.crumbs())

    def test_icon_url_or_default_no_icon(self):
        app = self.factory.make_application()

        with patch_settings(STATIC_URL='/assets/'):
            icon_url = app.icon_url_or_default

        self.assertEqual('/assets/images/applications-other-64.png', icon_url)

    def test_icon_url_or_default_with_icon(self):
        app = self.factory.make_application(icon=ImageFile(open(
            self.factory.get_test_path('ubuntu-cof.png'))))

        with patch_settings(MEDIA_URL='/site_media/'):
            icon_url = app.icon_url_or_default

        self.assertTrue(icon_url.startswith('/site_media/'))
        self.assertTrue('ubuntu-cof' in icon_url)

    def test_screenshots(self):
        app = self.factory.make_application()
        app.applicationmedia_set.create(
            media_type='screenshot',
            url='http://example.com/screenshot1.png')
        app.applicationmedia_set.create(
            media_type='screenshot',
            url='http://example.com/screenshot2.png')
        app.applicationmedia_set.create(
            media_type='video',
            url='http://example.com/video.m4v')

        urls = app.screenshots
        expected = [
            'http://example.com/screenshot1.png',
            'http://example.com/screenshot2.png',
        ]
        self.assertEqual(expected, urls)

    def test_screenshots_cached(self):
        # The db isn't hit if _screenshots is already
        # cached.
        app = self.factory.make_application()
        app.applicationmedia_set.create(
            media_type='screenshot',
            url='http://example.com/screenshot1.png')
        app.screenshots
        app.applicationmedia_set.create(
            media_type='screenshot',
            url='http://example.com/screenshot2.png')

        self.assertEqual(['http://example.com/screenshot1.png'],
                         app.screenshots)

    def test_video_iframes_urls(self):
        app = self.factory.make_application()
        app.applicationmedia_set.create(
            media_type='video',
            url='http://example.com/video1.m4v')
        app.applicationmedia_set.create(
            media_type='video',
            url='http://example.com/video2.m4v')
        app.applicationmedia_set.create(
            media_type='screenshot',
            url='http://example.com/screenshot.png')

        urls = app.video_iframe_urls
        expected = [
            'http://example.com/video1.m4v',
            'http://example.com/video2.m4v',
        ]
        self.assertEqual(expected, urls)

    def test_video_iframes_urls_cached(self):
        app = self.factory.make_application()
        app.applicationmedia_set.create(
            media_type='video',
            url='http://example.com/video1.m4v')
        app.video_iframe_urls
        app.applicationmedia_set.create(
            media_type='video',
            url='http://example.com/video2.m4v')

        urls = app.video_iframe_urls

        self.assertEqual(['http://example.com/video1.m4v'], urls)

    def test_ordering_with_wilson_score(self):
        barbacoa = self.factory.make_application(name='barbacoa',
                                                 wilson_score=0.2)
        zaraza = self.factory.make_application(name='zaraza',
                                               wilson_score=0.8)
        abedul = self.factory.make_application(name='abedul',
                                               wilson_score=0.5)

        self.assertEqual([zaraza, abedul, barbacoa],
                         list(Application.objects.all()))

    def test_ordering_without_wilson_score(self):
        zaraza = self.factory.make_application(name='zaraza',
                                               wilson_score=None)
        abedul = self.factory.make_application(name='abedul',
                                               wilson_score=None)

        self.assertEqual([abedul, zaraza], list(Application.objects.all()))


class DepartmentTestCase(TestCaseWithFactory):
    def test_crumbs_no_parent(self):
        dept = self.factory.make_department('Foo')
        expected = [
            {'name': 'All Apps', 'url': reverse('wc-index')},
            {'name': dept.name,
             'url': reverse('wc-department',
                            kwargs={'dept_slug_or_id': dept.slug})}
        ]
        self.assertEqual(expected, dept.crumbs())

    def test_crumbs_with_parent(self):
        parent = self.factory.make_department('Foo')
        dept = self.factory.make_department('Bar', parent=parent)
        expected = [
            {'name': 'All Apps', 'url': reverse('wc-index')},
            {'name': dept.parent.name,
             'url': reverse('wc-department',
                            kwargs={'dept_slug_or_id': dept.parent.slug})},
            {'name': dept.name,
             'url': reverse('wc-department',
                            kwargs={'dept_slug_or_id': dept.slug})}
        ]
        self.assertEqual(expected, dept.crumbs())

    def test_crumbs_with_distroseries(self):
        parent = self.factory.make_department('Foo')
        dept = self.factory.make_department('Bar', parent=parent)
        parent_url = reverse('wc-department',
                             kwargs={'distro': 'frobbly',
                                     'dept_slug_or_id': parent.slug})
        dept_url = reverse('wc-department',
                           kwargs={'distro': 'frobbly',
                                   'dept_slug_or_id': dept.slug})
        expected = [
            {'name': 'All Apps', 'url': reverse('wc-index')},
            {'name': dept.parent.name, 'url': parent_url},
            {'name': dept.name, 'url': dept_url}
        ]
        self.assertEqual(expected, dept.crumbs(distro='frobbly'))


class ExhibitTestCase(TestCaseWithFactory):
    def test_destination_url(self):
        exhibit = self.factory.make_exhibit(package_names='foobar')
        expected = reverse('wc-package-detail', args=['foobar'])
        self.assertEqual(expected, exhibit.destination_url())

    def test_destination_url_multiple_packages(self):
        exhibit = self.factory.make_exhibit(package_names='foobar,baz')
        expected = reverse('wc-package-list') + '?pkg_name=foobar&pkg_name=baz'
        self.assertEqual(expected, exhibit.destination_url())

    def test_destination_url_blank_packages(self):
        exhibit = self.factory.make_exhibit(package_names='')
        self.assertEqual('', exhibit.destination_url())

    def test_destination_url_click_url(self):
        exhibit = self.factory.make_exhibit(package_names='foobar,baz',
                                            click_url='http://example.com/f')

        self.assertEqual('http://example.com/f', exhibit.destination_url())
