#   -*- coding: utf-8 -*-
#   This file is part of the Ubuntu Web Catalog
#   Copyright (C) 2011-2012 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the oops_wsgi setup."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'OopsWsgiTestCase',
]

import bson
import os
import re
from shutil import rmtree
from tempfile import mkdtemp

from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.conf import settings
from django.test import TestCase
from mock import (
    Mock,
    patch,
)
from webcatalog.wsgi import make_app
from webcatalog.tests.helpers import patch_settings


class OopsWsgiTestCase(TestCase):

    def setUp(self):
        super(OopsWsgiTestCase, self).setUp()
        self.oops_dir = mkdtemp()
        self.addCleanup(rmtree, self.oops_dir)
        self.environ = {
            'PATH_INFO': '/',
            'QUERY_STRING': '',
            'REMOTE_ADDR': '127.0.0.1',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'wsgi.version': (1, 0),
            'wsgi.url_scheme': 'http',
            'wsgi.multiprocess': True,
            'wsgi.multithread': False,
            'wsgi.run_once': False,
            'wsgi.input': '',
        }
        cache.clear()

    def request_wsgi_response(self, environ):
        start_response = Mock()
        oops_config = {
            'publishers': [{
                'type': 'datedir',
                'error_dir': self.oops_dir,
                'inherit_id': True,
                'new_only': False,
            }],
        }
        with patch_settings(OOPSES=oops_config):
            app = make_app()
            response = app(environ, start_response).next()
        return response

    def read_oops_from_disk(self):
        # Helper to read in the oops written to disk.
        # There is only one date directory in the oops directory.
        dated_dirs = os.listdir(self.oops_dir)
        self.assertEqual(1, len(dated_dirs))

        # There is only one oops.
        todays_oops_path = dated_dirs[0]
        oops_list = os.listdir(self.oops_dir + '/' + todays_oops_path)
        self.assertEqual(1, len(oops_list))

        oops_path = self.oops_dir + '/' + todays_oops_path + '/' + oops_list[0]
        with open(oops_path, "r") as oops_file:
            oops_file_contents = oops_file.read()
        return bson.loads(oops_file_contents)

    def request_with_exception(self):
        self.environ['PATH_INFO'] = reverse('wc-index')
        view_fn_path = 'webcatalog.views.render_to_response'
        with patch(view_fn_path) as mock_render:
            mock_render.side_effect = ZeroDivisionError
            response = self.request_wsgi_response(self.environ)
        return response

    def test_oops_on_disk_on_error(self):
        response = self.request_with_exception()

        self.assertIn('Server Error', response)
        oops_id_re = re.compile('<p id="oops-id">(OOPS-[0-9a-f]{32})</p>')
        match = oops_id_re.search(response)
        self.assertTrue(match)
        oops_id = match.groups()[0]
        oops_data = self.read_oops_from_disk()
        self.assertEqual('ZeroDivisionError', oops_data['type'])
        self.assertEqual(oops_id, oops_data['id'])
