#   -*- coding: utf-8 -*-
#   This file is part of the Ubuntu Web Catalog
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth.models import (
    Group,
    User,
)
from mock import patch

from webcatalog.tests.factory import TestCaseWithFactory
from webcatalog.tests.helpers import patch_settings

__metaclass__ = type


class TestPreflight(TestCaseWithFactory):
    @patch('celery.app.default_app.control.ping')
    @patch('webcatalog.utilities.WebServices.identity_provider')
    @patch('webcatalog.utilities.WebServices.get_screenshots_for_package')
    @patch('webcatalog.utilities.WebServices.recommender_api')
    @patch('webcatalog.utilities.WebServices.rnr_api')
    def test_preflight_requires_login(self, mock_rnr, mock_recommender,
                                      mock_screenshots, mock_sso, mock_ping):
        self.make_user(groups=['preflight'], logged_in=True)

        with patch_settings(PREFLIGHT_GROUPS=['preflight']):
            response = self.client.get('/preflight/')

        self.assertEqual(200, response.status_code)

    def test_no_login_means_preflight_not_found(self):
        with patch_settings(PREFLIGHT_GROUPS=['preflight']):
            response = self.client.get('/preflight/')

        self.assertEqual(404, response.status_code)

    def test_login_but_wrong_group_means_preflight_not_found(self):
        self.make_user(logged_in=True)

        response = self.client.get('/preflight/')

        self.assertEqual(404, response.status_code)

    @patch('celery.app.default_app.control.ping')
    @patch('webcatalog.utilities.WebServices.identity_provider')
    @patch('webcatalog.utilities.WebServices.get_screenshots_for_package')
    @patch('webcatalog.utilities.WebServices.recommender_api')
    @patch('webcatalog.utilities.WebServices.rnr_api')
    def test_success(self, mock_rnr, mock_recommender, mock_screenshots,
                     mock_sso, mock_ping):
        mock_rnr.server_status.return_value = 'ok'
        mock_recommender.server_status.return_value = 'ok'
        mock_screenshots.return_value = ['some_screenshot.jpg']
        mock_sso.validate_token.return_value = False
        mock_ping.return_value = ['worker-1']
        self.make_user(groups=['preflight'], logged_in=True)

        with patch_settings(PREFLIGHT_GROUPS=['preflight']):
            response = self.client.get('/preflight/')

        self.assertEqual(200, response.status_code)
        libs = set([x['name'] for x in response.context[0]['versions']])
        apps = response.context['applications']
        checks = set((x.name, x.passed) for x in apps[0].checks)
        expected_libs = set([
            'Python',
            'openid',
            'piston',
            'Django',
            'Platform',
            'preflight',
        ])
        expected_checks = set([
            ('validate_config', False),
            ('database', True),
            ('rnr_available', True),
            ('identity_provider_available', True),
            ('recommender_available', True),
            ('rnr_available', True),
            ('screenshots_available', True),
            ('celery_workers', True),
        ])
        self.assertEqual(expected_libs, libs)
        self.assertEqual(expected_checks, checks)
