#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Apps Directory template tags."""

from __future__ import absolute_import

import unittest

from django.core.urlresolvers import reverse
from django.template import Context
from mock import Mock

from webcatalog.templatetags.webcatalog import (
    htmlize_package_description,
    install_options,
    rating_summary,
    visible_page_range,
    thumbnail_url,
)
from webcatalog.tests.factory import TestCaseWithFactory
from webcatalog.tests.helpers import patch_settings

__metaclass__ = type
__all__ = [
    'InstallOptionsTestCase',
    'HtmlizePackageDescriptionTestCase',
    'RatingSummaryTestCase',
    'VisiblePageRangeTestCase',
]


class InstallOptionsTestCase(TestCaseWithFactory):

    def setUp(self):
        super(InstallOptionsTestCase, self).setUp()
        self.natty = self.factory.make_distroseries(code_name='natty',
                                                    version='11.04')
        self.lucid = self.factory.make_distroseries(code_name='lucid',
                                                    version='10.04')

    def make_useragent(self, linux=True, arch='x86_64', ubuntu_version=None):
        os = 'X11; Linux' if linux else 'Windows NT 5.1'
        version_string = ''
        if ubuntu_version:
            version_string = "Ubuntu/" + ubuntu_version
        return "blah {os} {arch}; {version_string} blah".format(
            os=os, arch=arch, version_string=version_string)

    def assert_context_values(self, context, application, expected):
        actual = install_options(context, application)
        self.assertEqual(expected['application'], actual['application'])
        self.assertEqual(expected['message_text'], actual['message_text'])
        self.assertEqual(expected['display_install_button'],
                         actual['display_install_button'])
        self.assertEqual(expected['display_ubuntu_download'],
                         actual['display_ubuntu_download'])

    def test_matching_series_arch(self):
        # If the users series and arch match - install.
        application = self.factory.make_application(
            package_name='pkgfoo', distroseries=self.natty,
            arch='amd64')
        user_agent = self.make_useragent(ubuntu_version='11.04', arch='x86_64')
        context = Context(dict(user_agent=user_agent))

        expected = dict(application=application, display_install_button=True,
                        message_text='', display_ubuntu_download=False)
        self.assert_context_values(context, application, expected)

    def test_matching_series_arch_not_supported(self):
        # If the users series matches but arch not supported, we let
        # them know.
        application = self.factory.make_application(
            name="Package Foo", package_name='pkgfoo',
            distroseries=self.natty, arch='i686')
        user_agent = self.make_useragent(ubuntu_version='11.04',
                                         arch='solaris')
        context = Context(dict(user_agent=user_agent))

        expected = dict(application=application, display_install_button=False,
                        message_text='Package Foo is not available for your '
                        'computer.', display_ubuntu_download=False)
        self.assert_context_values(context, application, expected)

    def test_different_series_with_available(self):
        # If the users series is different from the app, but the app is
        # available for their series, then we display a link to the app
        # for their series.
        natty_application = self.factory.make_application(
            name="Package Foo", package_name='pkgfoo', distroseries=self.natty)
        self.factory.make_application(
            name="Package Foo", package_name='pkgfoo', distroseries=self.lucid)
        context = Context(
            dict(user_agent=self.make_useragent(ubuntu_version='10.04')))

        lucid_app_url = reverse('wc-package-detail', args=['lucid', 'pkgfoo'])
        msg = ('Package Foo is also available for '
               '<a href="{lucid_app_url}">your version of Ubuntu'
               '</a>.'.format(lucid_app_url=lucid_app_url))
        expected = dict(application=natty_application,
                        display_install_button=False,
                        message_text=msg, display_ubuntu_download=False)
        self.assert_context_values(context, natty_application, expected)

    def test_different_series_without_available(self):
        # If the users series is different from the app, and the app is
        # available for their series, then we say so.
        natty_application = self.factory.make_application(
            name="Package Foo", package_name='pkgfoo',
            distroseries=self.natty)
        context = Context(
            dict(user_agent=self.make_useragent(ubuntu_version='12.04')))

        expected = dict(application=natty_application,
                        display_install_button=False,
                        message_text='Package Foo is not available for your '
                        'version of Ubuntu.', display_ubuntu_download=False)
        self.assert_context_values(context, natty_application, expected)

    def test_linux_only_displays_install_button(self):
        # XXX Michael Nelson 2011-06-28 bug=709125
        # If the useragent tells us only Linux, we display the apt link
        # because until we have ubufox updated for whatever our
        # webcatalog domain will be (see update for apt.ubuntu.com on
        # bug 709125) only chromium in natty provides the version in the
        # user agent.
        application = self.factory.make_application(package_name='pkgfoo')
        context = Context(dict(
            user_agent=self.make_useragent(ubuntu_version='')))

        expected = dict(application=application, display_install_button=True,
                        message_text='', display_ubuntu_download=False)
        self.assert_context_values(context, application, expected)

    def test_non_linux_displays_download_ubuntu(self):
        # A non-linux client is provided a link to download ubuntu.
        application = self.factory.make_application(
            package_name='pkgfoo', name="Package Foo")
        user_agent = self.make_useragent(linux=False)
        context = Context(dict(user_agent=user_agent))

        expected = dict(application=application, display_install_button=False,
                        message_text='', display_ubuntu_download=True)
        self.assert_context_values(context, application, expected)

    def test_template_tag_uses_config_for_ubuntu_versions(self):
        application = self.factory.make_application(
            name="Fame", distroseries=self.natty, arch='amd64')
        user_agent = self.make_useragent(ubuntu_version='20.04', arch='x86_64')
        context = Context(dict(user_agent=user_agent))

        with patch_settings(UBUNTU_SERIES_FOR_VERSIONS={'20.04': 'fancy'}):
            install_options(context, application)

        self.assertFalse(context['display_install_button'])
        self.assertEqual(
            "Fame is not available for your version of Ubuntu.",
            context['message_text'])


#file-roller
d1 = """
File-roller is an archive manager for the GNOME environment. It allows you to:

* Create and modify archives.
* View the content of an archive.
* View a file contained in an archive.
* Extract files from the archive.
File-roller supports the following formats:
* Tar (.tar) archives, including those compressed with
  gzip (.tar.gz, .tgz), bzip (.tar.bz, .tbz), bzip2 (.tar.bz2, .tbz2),
  compress (.tar.Z, .taz), lzip (.tar.lz, .tlz), lzop (.tar.lzo, .tzo),
  lzma (.tar.lzma) and xz (.tar.xz)
* Zip archives (.zip)
* Jar archives (.jar, .ear, .war)
* 7z archives (.7z)
* iso9660 CD images (.iso)
* Lha archives (.lzh)
* Single files compressed with gzip (.gz), bzip (.bz), bzip2 (.bz2),
  compress (.Z), lzip (.lz), lzop (.lzo), lzma (.lzma) and xz (.xz)
File-roller doesn't perform archive operations by itself, but relies on \
standard tools for this.
"""

#drgeo
d2 = """
This is the Gtk interactive geometry software. It allows one
to create geometric figure plus the interactive manipulation of such
figure in respect with their geometric constraints. It is usable in
teaching situation with students from primary or secondary level.

Dr. Geo comes with a complete set of tools arranged
in different categories:

 * points
 * lines
 * geometric transformations
 * numeric function
 * macro-construction
 * DGS object - Dr. Geo Guile Script
 * DSF - Dr Geo Scheme Figure, it is interactive figure defined in
   a file and evaluated with the embedded Scheme interpretor, awesome!
 * Export facilities in the LaTeX and EPS formats

Several figures and macro-constructions examples are available
in the /usr/share/drgeo/examples folder.

More information about Dr. Geo can be found at
its web site http://www.gnu.org/software/dr_geo/dr_geo.html

Installing the drgeo-doc package is also encouraged to get
more of Dr. Geo.
"""

# totem
d3 = """
Totem is a simple yet featureful media player for GNOME which can read
a large number of file formats. It features :
.
   * Shoutcast, m3u, asx, SMIL and ra playlists support
   * DVD (with menus), VCD and Digital CD (with CDDB) playback
   * TV-Out configuration with optional resolution switching
   * 4.0, 5.0, 5.1 and stereo audio output
   * Full-screen mode (move your mouse and you get nice controls) with
     Xinerama, dual-head and RandR support
   * Aspect ratio toggling, scaling based on the video's original size
   * Full keyboard control
   * Simple playlist with repeat mode and saving feature
   * GNOME, Nautilus and GIO integration
   * Screenshot of the current movie
   * Brightness and Contrast control
   * Visualisation plugin when playing audio-only files
   * Video thumbnailer for nautilus
   * Nautilus properties page
   * Works on remote displays
   * DVD, VCD and OGG/OGM subtitles with automatic language selection
   * Extensible with plugins
"""


class HtmlizePackageDescriptionTestCase(unittest.TestCase):
    def test_htmlize(self):
        for descr in [d1, d2, d3]:
            html_descr = htmlize_package_description(descr)
            star_count = descr.count("*")
            li_count = html_descr.count("<li>")
            self.assertEqual(star_count, li_count)

    def test_other_bullets(self):
        d1 = """This is another test.
        - Some dash.
        - Some other dash.
        - And finally, something completely different.
        """
        d2 = """This is another test.
        o Some circle.
        o Some other circle,
          in the same line.
        o And finally, something completely different.
        And a suffix, just in case.
        """
        d3 = """This is a mixed test.
        - Some dash.
        * Some other dash.
        o And finally, something completely different.
        """
        for descr in d1, d2, d3:
            html_descr = htmlize_package_description(descr)
            li_count = html_descr.count("<li>")
            self.assertEqual(3, li_count)


class VisiblePageRangeTestCase(unittest.TestCase):
    def test_visible_page_range(self):
        cases = [
            (2, 1, [1, 2]),
            (20, 1, range(1, 11)),
            (3, 2, [1, 2, 3]),
            (20, 2, range(1, 11)),
            (4, 4, [1, 2, 3, 4]),
            (20, 8, range(3, 13)),
            (8, 8, range(3, 9)),
            (20, 18, range(13, 21)),
            (400, 400, range(395, 401)),
        ]
        for num_pages, current_page, expected in cases:
            page = Mock()
            page.number = current_page
            page.paginator.num_pages = num_pages
            result = visible_page_range(page)
            self.assertTrue(len(result) <= 10)
            self.assertEqual(expected, result)


class RatingSummaryTestCase(TestCaseWithFactory):
    def test_rating_summary(self):
        result = rating_summary('3.5', num_ratings=234)

        self.assertEqual([1, 1, 1, 0.5, 0], result['stars'])
        self.assertEqual(234, result['total'])


class ThumbnailURLTestCase(TestCaseWithFactory):
    def test_thumbnail_url(self):
        staging_sca = 'http://sc.staging.ubuntu.com/'
        staging_dp = 'http://developer.staging.ubuntu.com/'
        prod_sca = 'http://software-center.ubuntu.com/'
        prod_dp = 'http://myapps.developer.ubuntu.com/'
        cases = [
            (staging_sca + 'site_media/example.jpg',
             staging_sca + 'thumbnail/225x170/example.jpg'),
            (staging_dp + 'something_different/foo.jpg',
             staging_dp + 'something_different/foo.jpg'),
            (prod_dp + 'site_media/app/example.jpg',
             prod_dp + 'thumbnail/225x170/app/example.jpg'),
            (prod_sca + 'foobar.jpg',
             prod_sca + 'foobar.jpg'),
            ('http://example.com/someother.jpg',
             'http://example.com/someother.jpg'),
        ]
        for src, expected in cases:

            dst = thumbnail_url(src)

            self.assertEqual(expected, dst)
