#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Apps Directory utilities."""
__metaclass__ = type
__all__ = [
    'CreatePNGFromFileTestCase',
    'IdentityProviderTestCase',
    'ScreenshotGetterTestCase',
    'WebServicesRecommenderTestCase',
]

import json
import os

from webcatalog.tests.factory import TestCaseWithFactory
from webcatalog.utilities import (
    create_png_from_file,
    WebServices,
)

from django.conf import settings
from django.core.cache import cache
from django.test import TestCase
from httplib2 import ServerNotFoundError
from mock import patch
from piston_mini_client.failhandlers import APIError


class CreatePNGFromFileTestCase(TestCaseWithFactory):
    def test_file_conversion(self):
        filenames = [
            self.factory.get_test_path('firefox.xpm'),  # Regular xpm
            self.factory.get_test_path('mini-gsmc.xpm'),  # named colors
            self.factory.get_test_path('access.svg'),  # SVG file
        ]
        pngpath = 'test_file_DELETEME.png'
        self.addCleanup(lambda: os.path.exists(pngpath) and os.unlink(pngpath))
        for f in filenames:
            self.assertTrue(create_png_from_file(pngpath, f, 32, 32))
            self.assertTrue(os.path.exists(pngpath))

    def test_invalid_files(self):
        filenames = [
            self.factory.get_test_path('junk.xpm'),
            self.factory.get_test_path('empty.xpm'),
            self.factory.get_test_path('junk.svg'),
        ]
        pngpath = 'test_file_DELETEME.png'
        self.addCleanup(lambda: os.path.exists(pngpath) and os.unlink(pngpath))
        for f in filenames:
            self.assertFalse(create_png_from_file(pngpath, f, 32, 32))
            self.assertFalse(os.path.exists(pngpath))


class IdentityProviderTestCase(TestCase):
    service_root = 'https://login.ubuntu.com/api/1.0'
    username = 'MyUsername'
    password = 'password'

    def mock_sso(self, *args, **kwargs):
        url = args[0]
        self.assertTrue(url.startswith(self.service_root))
        if 'ws.op=validate_token' in url:
            body = ('{"consumer_secret": "foobarconsumersecret", '
                    '"token": "foo", "consumer_key": "foobar", '
                    '"name": "test-token", '
                    '"token_secret": "foobartokensecret"}')
        elif 'ws.op=me' in url:
            body = ('{"username": "username", "preferred_email": null, '
                    '"displayname": "Foo", "unverified_emails": ["f@o.com"], '
                    '"verified_emails": [], "openid_identifier": "foo_id"}')
        else:
            self.fail("Unexpected request: %s" % url)
        response = {'status': '200'}
        return response, body

    def setUp(self):
        super(IdentityProviderTestCase, self).setUp()
        self.orig_mode = settings.SSO_AUTH_MODE_NO_UBUNTU_SSO_PLAINTEXT_ONLY
        self.orig_service_root = settings.SSO_API_SERVICE_ROOT
        settings.SSO_API_SERVICE_ROOT = self.service_root

        # Force our webservice client to recreate the the ServiceRoot
        WebServices._identity_provider = None

    def tearDown(self):
        super(IdentityProviderTestCase, self).tearDown()
        settings.SSO_AUTH_MODE_NO_UBUNTU_SSO_PLAINTEXT_ONLY = self.orig_mode
        settings.SSO_API_SERVICE_ROOT = self.orig_service_root

    @patch('httplib2.Http.request')
    def test_identity_provider_unavailable(self, mock_request):
        mock_request.side_effect = ServerNotFoundError
        settings.SSO_AUTH_MODE_NO_UBUNTU_SSO_PLAINTEXT_ONLY = False

        self.assertRaises(
            ServerNotFoundError,
            WebServices().get_data_for_account, 'eg_token', 'eg_ident')

    @patch('httplib2.Http.request')
    def test_get_data_for_account(self, mock_request):
        # We tell the mockserver about the token we'll be checking.
        mock_request.side_effect = self.mock_sso
        settings.SSO_AUTH_MODE_NO_UBUNTU_SSO_PLAINTEXT_ONLY = False
        web_services = WebServices()

        data = web_services.get_data_for_account('eg_token', 'eg_ident')

        expected = [
            u'username', u'preferred_email', u'displayname', u'name',
            u'unverified_emails', u'verified_emails', u'consumer_secret',
            u'token', u'openid_identifier', u'consumer_key', u'token_secret',
        ]
        self.assertEqual(set(expected), set(data))


class WebServicesRecommenderTestCase(TestCaseWithFactory):

    def setUp(self):
        super(WebServicesRecommenderTestCase, self).setUp()
        recommend_app_fn = (
            'sreclient.SoftwareCenterRecommenderAPI.recommend_app')
        self.recommend_app_patcher = patch(recommend_app_fn)
        self.mock_recommend_app = self.recommend_app_patcher.start()
        self.addCleanup(self.recommend_app_patcher.stop)
        self.eg_recommends = {
            u'rid': u'defe066c7ad7c43f71bac58c3f23cc62',
            u'data': [
                {u'rating': 4.0, u'package_name': u'nautilus-gksu'},
                {u'rating': 4.0, u'package_name': u'tribaltrouble2'},
                {u'rating': 4.0, u'package_name': u'acm'},
                {u'rating': 4.0, u'package_name': u'zgv'},
                {u'rating': 3.0, u'package_name': u'nautilus-wallpaper'},
            ],
            u'app': u'firefox',
        }
        self.mock_recommend_app.return_value = self.eg_recommends
        cache.clear()

    def test_get_recommends_for_package_uncached(self):
        result = WebServices().get_recommends_for_package('firefox')

        self.assertEqual(result, self.eg_recommends)
        self.assertEqual(1, self.mock_recommend_app.call_count)

    def test_get_recommends_caches_result(self):
        self.assertIs(None, cache.get('get_recommends_for_package-firefox'))

        result = WebServices().get_recommends_for_package('firefox')

        self.assertEqual(
            self.eg_recommends,
            cache.get('get_recommends_for_package-firefox'))

    def test_get_recommends_for_package_cached(self):
        cache.set('get_recommends_for_package-firefox', {'foo': 'bar'})

        result = WebServices().get_recommends_for_package('firefox')

        self.assertEqual({'foo': 'bar'}, result)
        self.assertEqual(0, self.mock_recommend_app.call_count)

    def test_get_recommends_for_package_socket_error(self):
        self.mock_recommend_app.side_effect = APIError('500', 'error')

        result = WebServices().get_recommends_for_package('firefox')

        self.assertEqual({'data': [], 'package': 'firefox'}, result)


class ScreenshotGetterTestCase(TestCase):
    @patch('webcatalog.utilities.urllib.urlopen')
    def test_valid_response_returns_list_of_urls(self, mock_urlopen):
        shots = [
            {'version': '1.0', 'large_image_url': 'http://example.com/foo'},
            {'version': '1.1', 'large_image_url': 'http://example.com/bar'},
        ]
        response = json.dumps({'package': 'digikam', 'screenshots': shots})
        mock_urlopen.return_value.getcode.return_value = 200
        mock_urlopen.return_value.read.return_value = response
        expected = ['http://example.com/foo', 'http://example.com/bar']
        self.assertEqual(
            expected, WebServices().get_screenshots_for_package('digikam'))

    @patch('webcatalog.utilities.urllib.urlopen')
    def test_not_found_response_returns_empty_list(self, mock_urlopen):
        mock_urlopen.return_value.getcode.return_value = 404
        self.assertEqual([], WebServices().get_screenshots_for_package('foo'))

    @patch('webcatalog.utilities.urllib.urlopen')
    def test_invalid_response_returns_empty_list(self, mock_urlopen):
        mock_urlopen.return_value.getcode.return_value = 200
        mock_urlopen.return_value.read.return_value = 'invalid json'
        self.assertEqual([], WebServices().get_screenshots_for_package('foo'))

    @patch('webcatalog.utilities.urllib.urlopen')
    def test_unexpected_response_returns_empty_list(self, mock_urlopen):
        response = json.dumps({'package': 'The Spanish Inquisition'})
        mock_urlopen.return_value.getcode.return_value = 200
        mock_urlopen.return_value.read.return_value = response
        self.assertEqual([], WebServices().get_screenshots_for_package('foo'))
