#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test cases for the Apps Directory views."""

from __future__ import absolute_import

import json
import re

from decimal import Decimal

from django.conf import settings
from django.core import mail
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.test import TestCase

from mock import patch
from piston_mini_client.failhandlers import APIError
from rnrclient import ReviewDetails

from webcatalog.models import (
    Department,
    DistroSeries,
)
from webcatalog.tests.factory import TestCaseWithFactory
from webcatalog.tests.helpers import patch_settings

__metaclass__ = type
__all__ = [
    'ApplicationDetailNoSeriesTestCase',
    'ApplicationDetailTestCase',
    'ApplicationListTestCase',
    'ApplicationRecommendsTestCase',
    'ApplicationReviewsTestCase',
    'ApplicationScreenshotsTestCase',
    'IndexTestCase',
    'TasksViewTestCase',
    'TaskStatusViewTestCase',
    'TermsOfServiceTestCase',
    'OverviewTestCase',
    'SearchTestCase',
    'ComboViewTestCase',
]


WINDOWS_USERAGENT = ('Mozilla/5.0 (Windows NT 5.1; rv:2.0) '
                     'Gecko/20100101 Firefox/4.0')

UBUNTU_USERAGENT = ('Mozilla/5.0 (X11; Linux x86_64; rv:2.0.1) '
                    'Gecko/20100101 Firefox/4.0.1')


class ApplicationDetailTestCase(TestCaseWithFactory):
    def setUp(self):
        super(ApplicationDetailTestCase, self).setUp()
        cache.clear()

    def get_app_details_url(self, app):
        return reverse('wc-package-detail',
                       args=[app.distroseries.code_name, app.package_name])

    def get_package_details_response(self, package_name,
                                     useragent=UBUNTU_USERAGENT, distro=None):
        kwargs = dict(package_name=package_name)
        if distro:
            kwargs['distro'] = distro
        url = reverse('wc-package-detail', kwargs=kwargs)

        if useragent:
            return self.client.get(url, HTTP_USER_AGENT=useragent)
        else:
            return self.client.get(url)

    def get_app_and_response(self, code_name='natty', version='11.04',
                             arch='x86_64', name=None, comment=None,
                             description=None, detail_distro=None,
                             detail_package=None, imported_from_sca=False,
                             screenshot_url='', useragent=UBUNTU_USERAGENT):
        series, created = DistroSeries.objects.get_or_create(
            code_name=code_name, version=version)
        app = self.factory.make_application(package_name='pkgfoo',
                                            distroseries=series, arch=arch,
                                            name=name, comment=comment,
                                            description=description,
                                            screenshot_url=screenshot_url)
        if imported_from_sca:
            app.imported_from_sca = True
            app.save()
        if not detail_distro:
            detail_distro = app.distroseries.code_name
        if not detail_package:
            detail_package = app.package_name

        response = self.get_package_details_response(
            detail_package, distro=detail_distro, useragent=useragent)
        return response, app

    def test_renders_correct_template(self):
        response, app = self.get_app_and_response()

        self.assertTemplateUsed(response, 'webcatalog/application_detail.html')

    def test_includes_application_details(self):
        # The details of an application are included in the rendered html.
        response, app = self.get_app_and_response(
            name='My app foo', comment='The best app eva.',
            description='A long description.')

        self.assertContains(response, "My app foo")
        self.assertContains(response, "The best app eva.")
        self.assertContains(response, "A long description.")

    def test_screenshot_thumbnail_url(self):
        # The thumbnail link to screenshots is included.
        response, app = self.get_app_and_response()

        self.assertContains(
            response, '<img src="http://screenshots.ubuntu.com/'
                      'thumbnail-with-version/pkgfoo/ignored"')

    def test_includes_custom_screenshot_url_if_provided(self):
        expected = 'http://foo.com/%s.png' + self.factory.get_unique_string()
        response, app = self.get_app_and_response(screenshot_url=expected)

        self.assertContains(response, expected)

    def test_link_to_apt_package(self):
        response, app = self.get_app_and_response()

        self.assertContains(response, 'href="apt://pkgfoo"')

    def test_link_to_package_reviews(self):
        response, app = self.get_app_and_response()

        url = reverse('wc-package-reviews',
                      args=[app.distroseries.code_name, app.package_name])
        self.assertContains(response, '<a href="{0}"'.format(url))

    def test_link_to_reviews_snippet(self):
        response, app = self.get_app_and_response()

        url = reverse('wc-package-reviews-ajax',
                      args=[app.distroseries.code_name, app.package_name])
        self.assertContains(response, 'var reviews_uri = "{0}"'.format(url))

    def test_link_to_recommended_apps_not_present(self):
        with patch_settings(NUM_RECOMMENDED_APPS=0):
            response, app = self.get_app_and_response()

        self.assertNotContains(response, '<div class="recommendations">')

    def test_button_for_non_puchase_app(self):
        self.factory.make_application(package_name='pkgfoo',
                                      price=None)

        response = self.get_package_details_response('pkgfoo')

        self.assertContains(response, '/assets/images/scbutton-free-200px.png')
        self.assertNotContains(response,
                               '/assets/images/scbutton-non-free-200px.png')

    def test_button_for_for_puchase_app(self):
        self.factory.make_application(package_name='pkgfoo',
                                      price=Decimal('12.99'))

        response = self.get_package_details_response('pkgfoo')

        self.assertNotContains(response,
                               '/assets/images/scbutton-free-200px.png')
        self.assertContains(response,
                            '/assets/images/scbutton-non-free-200px.png')

    def test_open_graph_tags_are_present(self):
        response, app = self.get_app_and_response()
        regex = r'<meta property="([^"]+)" content="([^"]+)" />'
        og_tags = dict(m for m in re.findall(regex, response.content)
                       if 'og:' in m[0])

        self.assertEqual(og_tags, {
            'og:site_name': "Ubuntu Apps Directory",
            'og:title': app.name,
            'og:image': '/assets/images/applications-other-64.png'
        })

    def test_twitter_button_is_present(self):
        response, app = self.get_app_and_response()

        self.assertContains(response, "Tweet</a>")
        self.assertContains(response, "https://twitter.com/share")

    def test_hib_widget_is_present_for_hib_apps(self):
        app = self.factory.make_application(package_name='pkgfoo',
                                            name=u'A great foo')
        self.factory.make_app_widget(
            applications=[app],
            template_snippet=u"<p>Promotion for {{ application.name }}</p>")

        response = self.get_package_details_response('pkgfoo')

        self.assertContains(response, '<p>Promotion for A great foo</p>')

    def test_hib_widget_not_present_for_non_hib_apps(self):
        self.factory.make_application(package_name='pkgfoo',
                                      name=u'A great foo')
        other_app = self.factory.make_application(package_name='pkgbar',
                                                  name=u'Another app')
        self.factory.make_app_widget(
            applications=[other_app],
            template_snippet=u"Promotion for {{ application.name }}")

        response = self.get_package_details_response('pkgfoo')

        self.assertNotContains(response, 'Promotion for')

    def test_multiple_promotions_for_app(self):
        app = self.factory.make_application(package_name='pkgfoo',
                                            name=u'A great foo')
        self.factory.make_app_widget(
            applications=[app],
            template_snippet=u"Promotion X for {{ application.name }}")
        self.factory.make_app_widget(
            applications=[app],
            template_snippet=u"Promotion Y for {{ application.name }}")

        response = self.get_package_details_response('pkgfoo')

        self.assertContains(response, 'Promotion X for A great foo')
        self.assertContains(response, 'Promotion Y for A great foo')

    def test_twitter_link_does_not_contain_none(self):
        app = self.factory.make_application()

        response = self.client.get(reverse('wc-package-detail',
                                           args=[app.package_name]))

        self.assertContains(
            response,
            'data-url="http://testserver/cat/applications/{0}/"'.format(
                app.package_name))

    def test_google_plus_button_is_present(self):
        response, app = self.get_app_and_response()

        self.assertContains(response, "<g:plusone")

    def test_facebook_like_button_is_present(self):
        response, app = self.get_app_and_response()

        self.assertContains(response, '<div class="fb-like"')

    def test_link_to_different_series(self):
        # If the users series is different from the app, but the app is
        # available for their series, then we display a link to the app
        # for their series.
        lucid = self.factory.make_distroseries(code_name='lucid',
                                               version='10.04')
        lucid_app = self.factory.make_application(package_name='pkgfoo',
                                                  distroseries=lucid)

        response, app = self.get_app_and_response(
            name="<a>Escape me", useragent="blah Ubuntu/10.04 blah")

        lucid_app_url = reverse('wc-package-detail', args=['lucid', 'pkgfoo'])
        self.assertContains(
            response,
            '&lt;a&gt;Escape me is also available for '
            '<a href="{lucid_app_url}">your version of Ubuntu</a>.'.format(
                lucid_app_url=lucid_app_url))

    def test_includes_right_navigation(self):
        lucid = self.factory.make_distroseries(code_name='lucid',
                                               version='10.04')
        maverick = self.factory.make_distroseries(code_name='maverick',
                                                  version='10.10')
        self.factory.make_application(package_name='pkgfoo',
                                      distroseries=lucid)
        self.factory.make_application(package_name='pkgfoo',
                                      distroseries=maverick)

        response, app = self.get_app_and_response(name="Foobar")

        for ds in ['lucid', 'maverick']:
            url = reverse('wc-package-detail', args=[ds, 'pkgfoo'])
            self.assertContains(response, '<a href="{0}">Ubuntu'.format(url))

    def test_uses_codename_only_for_prerelease(self):
        codenames_versions = (
            ('oneiric', '11.10', False),
            ('precise', '12.04', False),
            ('quantal', '12.10', True),
        )
        for code_name, version, prerelease in codenames_versions:
            self.factory.make_application(
                package_name='pkgfoo',
                distroseries=self.factory.make_distroseries(
                    code_name=code_name, version=version,
                    prerelease=prerelease))

        response = self.client.get(
            reverse('wc-package-detail',
                    args=['pkgfoo']))

        self.assertContains(response, "Ubuntu 11.10")
        self.assertNotContains(response, "Ubuntu Oneiric")
        self.assertContains(response, "Ubuntu 12.04")
        self.assertNotContains(response, "Ubuntu Precise")
        self.assertContains(response, "Ubuntu Quantal")

    def test_includes_right_license_type(self):
        self.factory.make_application(license="Foo GNL V1",
                                      package_name='pkgfoo')

        response = self.client.get(
            reverse('wc-package-detail', args=['pkgfoo']))

        self.assertContains(response, '<td>Foo GNL V1</td>')

    def test_no_license_assumes_OSS(self):
        self.factory.make_application(license="",
                                      package_name='pkgfoo')

        response = self.client.get(
            reverse('wc-package-detail', args=['pkgfoo']))

        self.assertContains(response, '<td>Open Source</td>')

    def test_response_includes_rating_summary(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        self.factory.make_application(
            package_name='foo', distroseries=distro,
            ratings_average=Decimal('3.5'), ratings_total=23)

        url = reverse('wc-package-detail', args=['lucid', 'foo'])
        response = self.client.get(url)

        self.assertContains(response, 'images/star-large-1.png', 3)
        self.assertContains(response, '23 Ratings', 1)

    def test_response_contains_link_to_email_reminder(self):
        response, app = self.get_app_and_response(name="salamander")

        self.assertContains(response, 'Email a link to download')

    def test_email_for_download_contents(self):
        response, app = self.get_app_and_response(name="salamander")
        data = {'email': 'joe@intestinal-flora.net'}
        url = self.get_app_details_url(app)

        self.client.post(url, data=data, follow=True)

        self.assertEqual(1, len(mail.outbox))
        email = mail.outbox[0]
        self.assertIn(app.name, email.body)
        self.assertIn(url, email.body)

    def test_email_for_download_success_message(self):
        response, app = self.get_app_and_response(name="salamander")
        data = {'email': 'joe@intestinal-flora.net'}
        url = self.get_app_details_url(app)

        response = self.client.post(url, data=data, follow=True)

        self.assertContains(response, 'Success. Your download link '
                            'has been sent.')

    def test_email_error_blank(self):
        response, app = self.get_app_and_response(name="salamander")
        data = {'email': ''}
        url = self.get_app_details_url(app)

        response = self.client.post(url, data=data, follow=True)

        self.assertNotContains(response,
                               'Success. Your download link has been sent.')
        self.assertContains(response, 'This field is required.')

    def test_email_error_bogus(self):
        response, app = self.get_app_and_response(name="salamander")
        data = {'email': 'bogus'}
        url = self.get_app_details_url(app)

        response = self.client.post(url, data=data, follow=True)

        self.assertNotContains(response,
                               'Success. Your download link has been sent.')
        self.assertContains(response, 'Enter a valid email address.')

    def test_version_and_label_displayed(self):
        app = self.factory.make_application(
            version='1.2.3', distroseries=self.factory.make_distroseries())

        response = self.client.get(self.get_app_details_url(app))

        self.assertContains(response, '<th>Version:</th>')
        self.assertContains(response, '<td>1.2.3</td>')

    def test_version_label_not_displayed(self):
        app = self.factory.make_application(
            version='', distroseries=self.factory.make_distroseries())

        response = self.client.get(self.get_app_details_url(app))

        self.assertNotContains(response, '<th>Version</th>')

    def test_debtags_displayed(self):
        app = self.factory.make_application(debtags='["joystick"]')

        response = self.client.get(self.get_app_details_url(app))

        self.assertContains(response, 'Hardware requirements')
        self.assertContains(response, 'joystick')

    def test_debtags_not_displayed(self):
        app = self.factory.make_application(debtags='')

        response = self.client.get(self.get_app_details_url(app))

        self.assertNotContains(response, 'Hardware requirements')

    def test_videos_displayed(self):
        app = self.factory.make_application()
        app.applicationmedia_set.create(
            media_type="video",
            url="http://example.com/video_iframe.html")

        response = self.client.get(self.get_app_details_url(app))

        self.assertContains(
            response,
            '<iframe src="http://example.com/video_iframe.html"')


class ApplicationDetailNoSeriesTestCase(TestCaseWithFactory):
    def test_renders_latest(self):
        # If a distroseries is not included in the url, we always render the
        # latest available app
        natty = self.factory.make_distroseries(code_name='natty',
                                               version='11.04')
        lucid = self.factory.make_distroseries(code_name='lucid',
                                               version='10.04')
        for dseries in [natty, lucid]:
            self.factory.make_application(package_name='pkgfoo',
                                          distroseries=dseries)

        url = reverse('wc-package-detail', args=['pkgfoo'])
        response = self.client.get(url)

        self.assertEqual(200, response.status_code)
        used = response.context[0]['application']
        self.assertEqual(natty, used.distroseries)


class ApplicationListTestCase(TestCaseWithFactory):
    def test_no_args_redirects_to_index(self):
        response = self.client.get(reverse('wc-package-list'))

        self.assertRedirects(response, reverse('wc-index'))

    def test_includes_apps_for_pkg_names(self):
        apps = [
            self.factory.make_application(),
            self.factory.make_application(),
            self.factory.make_application(),
        ]

        pkgs = "&".join('pkg_name=%s' % app.package_name for app in apps)
        response = self.client.get(reverse('wc-package-list') + '?' + pkgs)

        self.assertEqual(200, response.status_code)
        self.assertEqual(apps, response.context['applications'])

    def test_includes_depts_in_context(self):
        depts = Department.objects.filter(parent=None).order_by('name')

        pkgs = 'pkg_name=firefox'
        response = self.client.get(reverse('wc-package-list') + '?' + pkgs)

        self.assertEqual(200, response.status_code)
        self.assertEqual(list(depts), list(response.context['depts']))


class SearchTestCase(TestCaseWithFactory):
    def setUp(self):
        super(SearchTestCase, self).setUp()
        cache.clear()

    def test_no_search_retrieves_no_apps(self):
        # Ensure there's some data in the DB
        distro = self.factory.make_distroseries(code_name='lucid')
        self.factory.make_application(distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])
        for data in [{'q': ''}, {}]:

            response = self.client.get(url, data=data)

            self.assertEqual(response.context['page'].object_list, [])
            self.assertContains(response, "You didn't search for anything")

    def test_search_searches_package_name(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        app1 = self.factory.make_application(package_name='foo',
                                             distroseries=distro)
        app2 = self.factory.make_application(package_name='bar',
                                             distroseries=distro)
        app3 = self.factory.make_application(package_name='foobar',
                                             distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])
        expected = {'foo': set([app1, app3]), 'bar': set([app2, app3])}
        for q in expected:
            response = self.client.get(url, data={'q': q})
            self.assertEqual(
                expected[q], set(response.context['page'].object_list))

    def test_search_searches_name(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        app1 = self.factory.make_application(name='foo',
                                             distroseries=distro)
        app2 = self.factory.make_application(name='bar',
                                             distroseries=distro)
        app3 = self.factory.make_application(name='foobar',
                                             distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])
        expected = {'foo': set([app1, app3]), 'bar': set([app2, app3])}
        for q in expected:
            response = self.client.get(url, data={'q': q})
            self.assertEqual(
                expected[q], set(response.context['page'].object_list))

    def test_search_includes_link_to_app(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        app = self.factory.make_application(name='foo',
                                            distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])

        response = self.client.get(url, data={'q': 'foo'})

        url = reverse('wc-package-detail',
                      args=[app.distroseries.code_name, app.package_name])
        self.assertContains(response, url, count=2)

    def test_search_includes_link_to_app_without_distro(self):
        app = self.factory.make_application(name='foo', is_latest=True)
        url = reverse('wc-search')

        response = self.client.get(url, data={'q': 'foo'})

        url = reverse('wc-package-detail', args=[app.package_name])
        self.assertContains(response, url, count=2)

    def test_search_no_apps_found_says_so(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        url = reverse('wc-search', args=[distro.code_name])

        response = self.client.get(url, data={'q': 'sarasa'})

        self.assertContains(response, "No applications found")

    def test_response_shows_query_if_no_apps_found(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        query = 'someimprobablequery'
        url = reverse('wc-search', args=[distro.code_name])

        response = self.client.get(url, data={'q': query})

        self.assertContains(response, query)

    def test_response_shows_query_if_some_apps_found(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        query = 'someimprobablequery'
        self.factory.make_application(package_name=query, distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])

        response = self.client.get(url, data={'q': query})

        self.assertContains(response, query)

    def test_response_includes_rating_summary(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        query = 'someimprobablequery'
        self.factory.make_application(
            package_name=query, distroseries=distro,
            ratings_average=Decimal('3.5'), ratings_total=23)

        url = reverse('wc-search', args=[distro.code_name])

        response = self.client.get(url, data={'q': query})

        self.assertContains(response, 'images/star-small-1.png', 3)
        self.assertContains(response, '23 Ratings', 1)

    def test_response_shows_number_of_apps_found(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        self.factory.make_application(
            package_name='foo', distroseries=distro)
        self.factory.make_application(
            package_name='foobar', distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])

        response = self.client.get(url, data={'q': 'foo'})

        self.assertContains(response, "1 to 2 of 2")

    def test_response_shows_paginated_results(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        app1 = self.factory.make_application(
            name='foo', distroseries=distro)
        app2 = self.factory.make_application(
            name='fobar', distroseries=distro)
        app3 = self.factory.make_application(
            name='foobar', distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])

        with patch_settings(PAGE_BATCH_SIZE=2):
            response = self.client.get(url, data={'q': 'fo'})

        page = response.context['page']
        self.assertEqual(2, page.paginator.num_pages)
        self.assertEqual(3, page.paginator.count)
        self.assertEqual(2, page.next_page_number())

        # And we have six references to the next page, three at the top
        # and three at the bottom, as you have the linked "2" number, plus the
        # arrow and text "Next" link.
        self.assertContains(response, '<a href="?q=fo&page=2"', 6)

    def test_response_sorts_by_wilson_score(self):
        foobaa = self.factory.make_application(
            name='foobaz', wilson_score=0.3, is_latest=True)
        foobaz = self.factory.make_application(
            name='foobaz', wilson_score=0.53, is_latest=True)
        foobar = self.factory.make_application(
            name='foobar', wilson_score=0.05, is_latest=True)

        url = reverse('wc-search')
        response = self.client.get(url, data={'q': 'fo'})
        actual = list(response.context['page'].object_list)
        self.assertEqual([foobaz, foobaa, foobar], actual)  # order matters

    def test_invalid_page_doesnt_error(self):
        distro = self.factory.make_distroseries(code_name='lucid')
        self.factory.make_application(name='foo', distroseries=distro)
        url = reverse('wc-search', args=[distro.code_name])

        response = self.client.get(url, data={'q': 'fo', 'page': 'aef8'})
        page = response.context['page']
        self.assertEqual(1, page.number)

        response = self.client.get(url, data={'q': 'fo', 'page': '0'})
        page = response.context['page']
        self.assertEqual(1, page.number)

        response = self.client.get(url, data={'q': 'fo', 'page': '999'})
        page = response.context['page']
        self.assertEqual(1, page.number)

    def test_invalid_distroseries_returns_404(self):
        response = self.client.get(reverse('wc-search', args=['zirconic']))

        self.assertEqual(404, response.status_code)

    def test_no_distroseries_does_not_redirect(self):
        response = self.client.get(reverse('wc-search'))

        self.assertEqual(200, response.status_code)


class IndexTestCase(TestCaseWithFactory):
    def setUp(self):
        super(IndexTestCase, self).setUp()
        cache.clear()

    def test_index_contains_links_to_departments(self):
        dept = self.factory.make_department('foo')

        response = self.client.get(reverse('wc-index'))

        url = reverse('wc-department', kwargs={'dept_slug_or_id': dept.slug})
        self.assertContains(response, url)

    def test_exhibits_widget_doesnt_display_if_no_exhibits_published(self):
        self.factory.make_exhibit(published=False)
        self.factory.make_exhibit(published=True, display=False)

        response = self.client.get(reverse('wc-index'))

        self.assertNotContains(response, '<div class="exhibits-widget">')

    def test_exhibits_display_trumps_published(self):
        """If an exhibit has display=True it's shown even if published=False"""
        self.factory.make_exhibit(published=False, display=True)

        response = self.client.get(reverse('wc-index'))

        self.assertContains(response, '<div class="exhibits-widget">')

    def test_exhibits_falls_back_to_published(self):
        """If an exhibit has display=None, do what published says"""
        self.factory.make_exhibit(published=False, display=None)
        self.factory.make_exhibit(published=True, display=None)

        response = self.client.get(reverse('wc-index'))

        self.assertContains(response, '<li class="slide', count=1)

    def test_exhibits_orders_banners_by_weight(self):
        third = self.factory.make_exhibit(display=True, weight=3)
        second = self.factory.make_exhibit(display=True, weight=2)
        first = self.factory.make_exhibit(display=True, weight=1)

        response = self.client.get(reverse('wc-index'))

        first_pos = response.content.index(first.destination_url())
        second_pos = response.content.index(second.destination_url())
        third_pos = response.content.index(third.destination_url())

        self.assertTrue(third_pos > second_pos > first_pos)

    def test_featured_apps(self):
        app = self.factory.make_application(package_name='foobar')

        with patch_settings(FEATURED_APPS=['foobar', 'baz']):
            response = self.client.get(reverse('wc-index'))

        self.assertEqual([app], response.context[0]['featured_apps'])

    def test_featured_apps_doesnt_show_blank_slides(self):
        # Make 4 apps to fill exactly one slide
        apps = [self.factory.make_application() for x in range(4)]

        with patch_settings(FEATURED_APPS=[app.package_name for app in apps]):
            response = self.client.get(reverse('wc-index'))

        self.assertContains(response, 'class="carousel-controls"', 1)

    def test_link_to_dev_site(self):
        response = self.client.get(reverse('wc-index'))

        self.assertContains(
            response,
            '<a href="http://developer.ubuntu.com">'
            'Add your own application</a>')

    def test_links_to_other_items(self):
        links = [
            "https://www.facebook.com/ubuntulinux",
            "http://twitter.com/#!/ubuntu",
            "http://planet.ubuntu.com/",
            "https://help.ubuntu.com/community/Pay",
            "http://developer.ubuntu.com",
            "https://launchpad.net/ubuntu-webcatalog",
            "http://www.ubuntu.com",
            "http://www.canonical.com",
        ]
        response = self.client.get(reverse('wc-index'))
        for link in links:
            self.assertContains(response, link)

    def make_high_medium_and_low_apps(self, score_high=5.0, score_mid=4.0,
                                      score_low=1.0):
        high = self.factory.make_application(
            package_name='high', wilson_score=score_high, ratings_average=5,
            ratings_total=500)
        mid = self.factory.make_application(
            package_name='mid', wilson_score=score_mid, ratings_average=4,
            ratings_total=400)
        low = self.factory.make_application(
            package_name='low', wilson_score=score_low, ratings_average=1,
            ratings_total=100)
        return (high, mid, low)

    def test_top_rated_apps_in_response(self):
        high, mid, low = self.make_high_medium_and_low_apps()

        with patch_settings(NUMBER_TOP_RATED_APPS=1):
            response = self.client.get(reverse('wc-index'))

        self.assertEqual(1, len(response.context[0]['top_rated_apps']))
        self.assertEqual(high, response.context[0]['top_rated_apps'][0])

    def test_top_rated_apps_display_correctly(self):
        high, mid, low = self.make_high_medium_and_low_apps()

        with patch_settings(NUMBER_TOP_RATED_APPS=2):
            response = self.client.get(reverse('wc-index'))

        self.assertContains(response, 'class="carousel-controls"', 1)
        self.assertContains(response, high.package_name, 2)
        self.assertContains(response, mid.package_name, 2)
        self.assertContains(response, low.package_name, 0)

    def test_top_rated_includes_rating_summary(self):
        high, mid, low = self.make_high_medium_and_low_apps()

        with patch_settings(NUMBER_TOP_RATED_APPS=1):
            response = self.client.get(reverse('wc-index'))

        self.assertContains(response, 'images/star-small-1.png', 5)
        self.assertContains(response, '500 Ratings', 1)

    def test_no_wilson_score_is_never_picked_for_top_rated(self):
        app = self.factory.make_application(wilson_score=35)
        app_not_included = self.factory.make_application()

        with patch_settings(NUMBER_TOP_RATED_APPS=10):
            response = self.client.get(reverse('wc-index'))

        expected = reverse('wc-package-detail', args=[app.package_name])
        self.assertContains(response, expected, 2)
        self.assertNotContains(response, app_not_included.package_name)

    def test_top_rated_apps_does_not_include_duplicate_packages(self):
        high, mid, low = self.make_high_medium_and_low_apps(5.0, 4.0, 1.0)
        high2, mid2, low2 = self.make_high_medium_and_low_apps(4.8, 3.8, 0.8)
        high3, mid3, low3 = self.make_high_medium_and_low_apps(4.6, 3.6, 0.6)
        high4, mid4, low4 = self.make_high_medium_and_low_apps(4.4, 3.4, 0.4)

        with patch_settings(NUMBER_TOP_RATED_APPS=4):
            response = self.client.get(reverse('wc-index'))

        apps = response.context['top_rated_apps']
        # should have only first high, first mid and first low
        # and in that order
        self.assertEqual(len(apps), 3)
        self.assertEqual(apps[0], high)
        self.assertEqual(apps[1], mid)
        self.assertEqual(apps[2], low)

    def test_includes_cache_headers(self):
        with patch_settings(CACHE_MIDDLEWARE_SECONDS=600):
            response = self.client.get(reverse('wc-index'))

        self.assertEqual('max-age=600', response['cache-control'])


class TermsOfServiceTestCase(TestCase):

    def test_tos_is_accessible(self):
        response = self.client.get(reverse('wc-tos'))
        self.assertTemplateUsed(response, 'webcatalog/tos.html')
        self.assertTemplateUsed(response, 'webcatalog/tos_snippet.html')

    def test_tos_plain_is_accessible(self):
        response = self.client.get(reverse('wc-tos-plain'))
        self.assertTemplateUsed(response, 'webcatalog/tos_plain.html')
        self.assertTemplateUsed(response, 'webcatalog/tos_snippet.html')


class OverviewTestCase(TestCaseWithFactory):
    def setUp(self):
        super(OverviewTestCase, self).setUp()
        cache.clear()

    def test_department_contains_links_to_subdepartments(self):
        dept = self.factory.make_department('foo')
        subdept = self.factory.make_department('bar', parent=dept)
        self.factory.make_distroseries(code_name='lucid')

        response = self.client.get(reverse('wc-department', kwargs={
            'distro': 'lucid', 'dept_slug_or_id': dept.id}))

        url = reverse('wc-department',
                      kwargs={'dept_slug_or_id': subdept.slug})
        self.assertContains(response, url)

    def test_department_contains_links_to_subdepartments_no_distro(self):
        dept = self.factory.make_department('foo')
        subdept = self.factory.make_department('bar', parent=dept)

        url = reverse('wc-department',
                      kwargs={'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        url = reverse('wc-department',
                      kwargs={'dept_slug_or_id': subdept.slug})
        self.assertContains(response, url)

    def test_department_contains_links_to_apps(self):
        dept = self.factory.make_department('bar')
        app = self.factory.make_application(package_name='foo',
                                            departments=[dept])

        url = reverse('wc-department',
                      kwargs={'distro': app.distroseries.code_name,
                              'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        url = reverse('wc-package-detail',
                      args=[app.distroseries.code_name, app.package_name])
        self.assertContains(response, url, count=2)

    def test_department_contains_links_to_apps_with_no_distro(self):
        dept = self.factory.make_department('bar')
        app = self.factory.make_application(package_name='foo', is_latest=True,
                                            departments=[dept])

        url = reverse('wc-department', kwargs={'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        url = reverse('wc-package-detail', args=[app.package_name])
        self.assertContains(response, url, count=2)

    def test_department_with_no_subdepts_doesnt_contain_header(self):
        dept = self.factory.make_department('bar')
        distro = self.factory.make_distroseries(code_name='lucid')

        url = reverse('wc-department',
                      kwargs={'distro': 'lucid', 'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        self.assertNotContains(response, 'Subsections')

    def test_department_shows_paginated_results(self):
        dept = self.factory.make_department('bar')
        lucid = self.factory.make_distroseries(code_name='lucid')
        for count in range(3):
            app = self.factory.make_application(distroseries=lucid,
                                                departments=[dept])

        url = reverse('wc-department',
                      kwargs={'distro': 'lucid', 'dept_slug_or_id': dept.id})
        with patch_settings(PAGE_BATCH_SIZE=2):
            response = self.client.get(url)

        page = response.context['page']
        self.assertEqual(2, page.paginator.num_pages)
        self.assertEqual(3, page.paginator.count)
        self.assertEqual(2, page.next_page_number())

        # And we have six references to the next page, three at the top
        # and three at the bottom, as you have the linked "2" number, plus the
        # arrow and text "Next" link.
        self.assertContains(response, '<a href="?page=2"', 6)

    def test_invalid_page_doesnt_error(self):
        dept = self.factory.make_department('bar')
        distro = self.factory.make_distroseries(code_name='lucid')
        url = reverse('wc-department',
                      kwargs={'distro': 'lucid', 'dept_slug_or_id': dept.id})
        response = self.client.get(url, data={'page': 'aef8'})
        page = response.context['page']
        self.assertEqual(1, page.number)

        response = self.client.get(url, data={'page': '0'})
        page = response.context['page']
        self.assertEqual(1, page.number)

        response = self.client.get(url, data={'page': '999'})
        page = response.context['page']
        self.assertEqual(1, page.number)

    def test_department_includes_right_navigation(self):
        dept = self.factory.make_department('bar')
        default = self.factory.make_distroseries(
            code_name=settings.DEFAULT_DISTRO, version='11.04')
        lucid = self.factory.make_distroseries(
            code_name='lucid', version='10.04')
        maverick = self.factory.make_distroseries(
            code_name='maverick', version='10.10')

        url = reverse('wc-department',
                      kwargs={'distro': settings.DEFAULT_DISTRO,
                              'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        for ds in ['lucid', 'maverick']:
            url = reverse('wc-department',
                          kwargs={'distro': ds, 'dept_slug_or_id': dept.slug})
            self.assertContains(response, '<a href="{0}">Ubuntu'.format(url))

    def test_department_includes_rating_summary(self):
        dept = self.factory.make_department('bar')
        lucid = self.factory.make_distroseries(code_name='lucid')
        app = self.factory.make_application(
            distroseries=lucid, ratings_average=Decimal('3.5'),
            ratings_total=23, departments=[dept])

        url = reverse('wc-department',
                      kwargs={'distro': 'lucid', 'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        self.assertContains(response, 'images/star-small-1.png', 3)
        self.assertContains(response, '23 Ratings', 1)

    def test_invalid_distroseries_returns_404(self):
        dept = self.factory.make_department('bar')

        url = reverse('wc-department',
                      kwargs={'distro': 'amnesiac',
                              'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        self.assertEqual(404, response.status_code)

    def test_distroseries_are_ordered_by_reverse_code_name(self):
        dept = self.factory.make_department('bar')
        default = self.factory.make_distroseries(
            code_name=settings.DEFAULT_DISTRO, version='11.04')
        lucid = self.factory.make_distroseries(code_name='lucid',
                                               version='10.04')
        maverick = self.factory.make_distroseries(code_name='maverick',
                                                  version='10.10')

        url = reverse('wc-department',
                      kwargs={'distro': settings.DEFAULT_DISTRO,
                              'dept_slug_or_id': dept.id})
        response = self.client.get(url)

        url = reverse('wc-department',
                      kwargs={'distro': 'lucid', 'dept_slug_or_id': dept.slug})
        lucid_pos = response.content.find('<a href="{0}">Ubuntu'.format(url))
        url = reverse('wc-department',
                      kwargs={'distro': 'maverick',
                              'dept_slug_or_id': dept.slug})
        maver_pos = response.content.find('<a href="{0}">Ubuntu'.format(url))
        self.assertTrue(lucid_pos > maver_pos)

    def test_get_department_by_slug(self):
        dept = self.factory.make_department('bar')
        self.factory.make_distroseries(code_name='lucid')

        url = reverse('wc-department',
                      kwargs={'distro': 'lucid', 'dept_slug_or_id': dept.slug})
        response = self.client.get(url)

        self.assertEqual(200, response.status_code)

    def test_no_distroseries_does_not_redirect(self):
        dept = self.factory.make_department('bar')

        url = reverse('wc-department', kwargs={'dept_slug_or_id': dept.slug})
        response = self.client.get(url)

        self.assertEqual(200, response.status_code)

    def test_response_sorts_by_wilson_score(self):
        dept = self.factory.make_department('bar')
        foobaa = self.factory.make_application(
            name='foobaz', wilson_score=0.3, is_latest=True,
            departments=[dept])
        foobaz = self.factory.make_application(
            name='foobaz', wilson_score=0.53, is_latest=True,
            departments=[dept])
        foobar = self.factory.make_application(
            name='foobar', wilson_score=0.05, is_latest=True,
            departments=[dept])

        url = reverse('wc-department', kwargs={'dept_slug_or_id': dept.slug})
        response = self.client.get(url, data={'q': 'fo'})
        actual = list(response.context['page'].object_list)
        self.assertEqual([foobaz, foobaa, foobar], actual)  # order matters


class ApplicationReviewsTestCase(TestCaseWithFactory):

    def setUp(self):
        super(ApplicationReviewsTestCase, self).setUp()
        get_reviews_fn = 'rnrclient.RatingsAndReviewsAPI.get_reviews'
        self.get_reviews_patcher = patch(get_reviews_fn)
        self.mock_get_reviews = self.get_reviews_patcher.start()
        eg_review = ReviewDetails.from_dict(
            dict(package_name='foo', summary='foo summary',
                 rating='3.5'))
        self.mock_get_reviews.return_value = [eg_review]

    def tearDown(self):
        if hasattr(self, 'get_reviews_patcher'):
            self.get_reviews_patcher.stop()
        super(ApplicationReviewsTestCase, self).tearDown()

    def test_only_valid_apps(self):
        response = self.client.get(reverse('wc-package-reviews',
                                           args=['jaunty', 'doesntexist']))

        self.assertEqual(404, response.status_code)

    def test_uncached_calls_to_rnr_api(self):
        app = self.factory.make_application()

        url = reverse('wc-package-reviews',
                      args=[app.distroseries.code_name, app.package_name])
        response = self.client.get(url)

        self.assertEqual(1, self.mock_get_reviews.call_count)

    def test_second_call_cached(self):
        app = self.factory.make_application()

        url = reverse('wc-package-reviews',
                      args=[app.distroseries.code_name, app.package_name])
        for count in range(2):
            response = self.client.get(url)

        self.assertEqual(1, self.mock_get_reviews.call_count)

    def make_review_details(self, package_name=None, summary=None,
                            rating='3.5'):
        if package_name is None:
            package_name = self.factory.get_unique_string(prefix='pkg-')
        if summary is None:
            summary = self.factory.make_unique_string(prefix='summary-')
        return ReviewDetails.from_dict(
            dict(package_name=package_name, summary=summary,
                 rating=rating))

    def test_renders_reviews(self):
        app = self.factory.make_application()
        self.mock_get_reviews.return_value = [
            self.make_review_details(summary='review_summary1'),
            self.make_review_details(summary='review_summary2'),
        ]

        url = reverse('wc-package-reviews',
                      args=[app.distroseries.code_name, app.package_name])
        response = self.client.get(url)

        self.assertTemplateUsed(
            response, 'webcatalog/application_review_list.html')
        self.assertTemplateUsed(
            response, 'webcatalog/application_review_list_snippet.html')
        self.assertContains(response, 'review_summary1')
        self.assertContains(response, 'review_summary2')

    def test_renders_reviews_only_for_ajax_request(self):
        app = self.factory.make_application()
        self.mock_get_reviews.return_value = [
            self.make_review_details(summary='review_summary1'),
            self.make_review_details(summary='review_summary2'),
        ]

        url = reverse('wc-package-reviews-ajax',
                      args=[app.distroseries.code_name, app.package_name])
        response = self.client.get(url)

        self.assertTemplateNotUsed(
            response, 'webcatalog/application_review_list.html')
        self.assertTemplateUsed(
            response, 'webcatalog/application_review_list_snippet.html')
        self.assertContains(response, 'review_summary1')
        self.assertContains(response, 'review_summary2')

    def test_doesnt_break_for_invalid_package_name(self):
        self.get_reviews_patcher.stop()
        del self.get_reviews_patcher
        app = self.factory.make_application(package_name='invalid:name')

        url = reverse('wc-package-reviews',
                      args=[app.distroseries.code_name, app.package_name])
        response = self.client.get(url)

        self.assertEqual(200, response.status_code)

    def test_handles_api_error(self):
        app = self.factory.make_application()
        self.mock_get_reviews.side_effect = APIError('500', 'error')

        url = reverse('wc-package-reviews',
                      args=[app.distroseries.code_name, app.package_name])
        response = self.client.get(url)

        self.assertEqual(200, response.status_code)


class ApplicationScreenshotsTestCase(TestCaseWithFactory):
    def setUp(self):
        super(ApplicationScreenshotsTestCase, self).setUp()
        self.app = self.factory.make_application(is_latest=True)
        self.expected = ['http://example.com/foo', 'http://example.com/bar']

    def test_only_valid_apps(self):
        response = self.client.get(reverse('wc-package-screenshots',
                                           args=['doesntexist']))

        self.assertEqual(404, response.status_code)

    @patch('webcatalog.utilities.urllib.urlopen')
    def test_uncached_calls_to_rnr_api(self, mock_urlopen):
        self.client.get(reverse('wc-package-screenshots',
                                args=[self.app.package_name]))

        self.assertEqual(1, mock_urlopen.call_count)

    @patch('webcatalog.utilities.urllib.urlopen')
    def test_second_call_cached(self, mock_urlopen):
        for count in range(2):
            self.client.get(reverse('wc-package-screenshots',
                                    args=[self.app.package_name]))

        self.assertEqual(1, mock_urlopen.call_count)

    @patch('webcatalog.utilities.WebServices.get_screenshots_for_package')
    def test_renders_json(self, mock_get_screenshots):
        mock_get_screenshots.return_value = self.expected

        response = self.client.get(reverse('wc-package-screenshots',
                                           args=[self.app.package_name]))

        self.assertTemplateNotUsed(
            response, 'webcatalog/application_review_list.html')
        self.assertEqual('application/json', response['content-type'])
        self.assertEqual(json.dumps(self.expected), response.content)

    @patch('webcatalog.utilities.WebServices.get_screenshots_for_package')
    def test_has_caching_headers(self, mock_get_screenshots):
        mock_get_screenshots.return_value = self.expected

        with patch_settings(CACHE_MIDDLEWARE_SECONDS=600):
            url = reverse('wc-package-screenshots',
                          args=[self.app.package_name])
            response = self.client.get(url,
                                       HTTP_X_REQUESTED_WITH='XMLHttpRequest')

        self.assertEqual('max-age=600', response['cache-control'])

    @patch('webcatalog.utilities.urllib.urlopen')
    def test_doesnt_break_with_multiple_apps(self, mock_urlopen):
        self.factory.make_application(package_name=self.app.package_name)

        response = self.client.get(reverse('wc-package-screenshots',
                                           args=[self.app.package_name]))

        self.assertEqual(200, response.status_code)

    @patch('webcatalog.utilities.urllib.urlopen')
    def test_not_dependent_on_is_latest(self, mock_urlopen):
        lucid = self.factory.make_distroseries(code_name='lucid')
        maverick = self.factory.make_distroseries(code_name='maverick')
        self.factory.make_application(
            package_name='foo', distroseries=lucid, is_latest=False)
        self.factory.make_application(
            package_name='foo', distroseries=maverick, is_latest=False)

        response = self.client.get(reverse('wc-package-screenshots',
                                           args=['foo']))

        self.assertEqual(200, response.status_code)


class ApplicationRecommendsTestCase(TestCaseWithFactory):

    def setUp(self):
        super(ApplicationRecommendsTestCase, self).setUp()
        get_recommends_fn = (
            'webcatalog.utilities.WebServices.get_recommends_for_package')
        self.get_recommends_patcher = patch(get_recommends_fn)
        self.mock_get_recommends = self.get_recommends_patcher.start()
        self.addCleanup(self.get_recommends_patcher.stop)
        self.eg_recommends = {
            u'rid': u'defe066c7ad7c43f71bac58c3f23cc62',
            u'data': [
                {u'rating': 4.0, u'package_name': u'nautilus-gksu'},
                {u'rating': 4.0, u'package_name': u'tribaltrouble2'},
                {u'rating': 4.0, u'package_name': u'acm'},
                {u'rating': 4.0, u'package_name': u'zgv'},
                {u'rating': 3.0, u'package_name': u'nautilus-wallpaper'},
            ],
            u'app': u'firefox',
        }
        self.mock_get_recommends.return_value = self.eg_recommends
        cache.clear()

    def test_only_valid_apps(self):
        response = self.client.get(reverse('wc-package-recommends',
                                           args=['doesntexist']))

        self.assertEqual(404, response.status_code)

    def test_renders_widget_only(self):
        self.factory.make_application(package_name='firefox')

        response = self.client.get(
            reverse('wc-package-recommends', args=['firefox']))

        self.assertTemplateUsed(
            response, 'webcatalog/recommended_apps_widget.html')

    def test_handles_multiple_versions_of_app(self):
        # Same app in two distroseries.
        self.factory.make_application(package_name='firefox')
        app2 = self.factory.make_application(package_name='firefox')

        response = self.client.get(
            reverse('wc-package-recommends', args=['firefox']))

        self.assertEqual(app2, response.context['application'])

    def test_no_recommendations_renders_no_portlet_header(self):
        self.factory.make_application(package_name='firefox')
        self.mock_get_recommends.return_value['data'] = []

        response = self.client.get(
            reverse('wc-package-recommends', args=['firefox']))

        self.assertEqual('', response.content.strip())


class ComboViewTestCase(TestCase):
    """Tests for ComboView."""

    def test_no_fnames_in_request(self):
        """If there are no filenames in the request, a 404 is returned."""
        response = self.client.get(reverse('wc-combo'))
        self.assertEqual(404, response.status_code)
        self.assertEqual('text/plain', response['Content-Type'])

    def test_content_type_for_js(self):
        """
        If the requested files are JS files, the response content-type will be
        set to 'text/javascript'.
        """
        response = self.client.get(reverse('wc-combo') + '?js/utils.js')
        self.assertEqual(200, response.status_code)
        self.assertEqual('text/javascript', response['Content-Type'])
        expected = '/* js/utils.js */\n/* [missing] */\n'
        self.assertEqual(expected, response.content)

    def test_content_type_for_css(self):
        """
        If the requested files are CSS files, the response content-type will be
        set to 'text/css'.
        """
        response = self.client.get(reverse('wc-combo') + '?js/foo.css')
        self.assertEqual(200, response.status_code)
        expected = '/* js/foo.css */\n/* [missing] */\n'
        self.assertEqual('text/css', response['Content-Type'])
        self.assertEqual(expected, response.content)


class TasksViewTestCase(TestCaseWithFactory):
    """Tests for the task list"""
    def test_not_allowed_if_not_logged_in(self):
        url = reverse('wc-tasks')

        response = self.client.get(url)

        self.assertEqual(302, response.status_code)
        expected = reverse('wc-forbidden') + '?next=' + url
        self.assertTrue(response['location'].endswith(expected))

    def test_scheduler_perm_is_required(self):
        self.make_user(logged_in=True)
        url = reverse('wc-tasks')

        response = self.client.get(url)

        self.assertEqual(302, response.status_code)
        expected = reverse('wc-forbidden') + '?next=' + url
        self.assertTrue(response['location'].endswith(expected))

    def test_includes_all_tasks(self):
        self.maxDiff = None
        self.make_user(permissions=['schedule_tasks'], logged_in=True)
        url = reverse('wc-tasks')

        response = self.client.get(url)

        tasks = response.context[0]['task_list']
        expected = [
            {'doc': 'Import all exhibits', 'name': 'import_exhibits'},
            {
                'doc': 'Remove stale OAuth nonces from the DB',
                'name': 'cleanup_nonces',
            },
            {
                'doc': 'Remove stale sessions from the DB',
                'name': 'cleanup_sessions',
            },
            {
                'doc': 'Sleep for 5 seconds, then raise an error',
                'name': 'fail',
            },
            {
                'doc': 'Import all data from app-install-data and apt-cache',
                'name': 'import_app_install_data',
            },
            {
                'doc': ('Import all data from the Software '
                        'Center Agent (MyApps)'),
                'name': 'import_sca',
            },
            {
                'doc': 'Import ratings data since the last import',
                'name': 'import_ratings_stats',
            },
            {
                'doc': 'Import all ratings data',
                'name': 'import_ratings_stats_all',
            },
            {
                'doc': "Update the 'is_latest' bit on all Applications",
                'name': 'check_all_latest',
            },
        ]
        self.assertEqual(sorted(expected), sorted(tasks))

    @patch('webcatalog.tasks.call_command')
    def test_post_schedules_task(self, mock_call_command):
        self.make_user(permissions=['schedule_tasks'], logged_in=True)
        url = reverse('wc-tasks')
        data = {'task': 'import_exhibits'}

        response = self.client.post(url, data=data, follow=False)

        messages_cookie = response.client.cookies['messages'].output()
        msg_pat = r'Task id <span>[-\da-f]+</span> submitted'
        match = re.search(msg_pat, messages_cookie)
        self.assertFalse(match is None)
        self.assertEqual(1, mock_call_command.call_count)


class TaskStatusViewTestCase(TestCaseWithFactory):
    """Tests for the task status getter"""
    def test_not_allowed_if_not_logged_in(self):
        url = reverse('wc-task-status', args=['cafe'])

        response = self.client.get(url)

        self.assertEqual(302, response.status_code)
        expected = reverse('wc-forbidden') + '?next=' + url
        self.assertTrue(response['location'].endswith(expected))

    def test_scheduler_perm_is_required(self):
        self.make_user(logged_in=True)
        url = reverse('wc-task-status', args=['cafe'])

        response = self.client.get(url)

        self.assertEqual(302, response.status_code)
        expected = reverse('wc-forbidden') + '?next=' + url
        self.assertTrue(response['location'].endswith(expected))

    def test_unknown_task_is_reported_pending(self):
        self.make_user(permissions=['schedule_tasks'], logged_in=True)
        url = reverse('wc-task-status', args=['cafe'])

        response = self.client.get(url)

        data = json.loads(response.content)
        expected = {"status": "PENDING", "traceback": None, "id": "cafe"}

        self.assertEqual(expected, data)
