#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utilities used by various modules of the Apps Directory."""


from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'full_claimed_id',
    'UserAgentString',
    'WebServices',
    'WebServiceError',
]

import json
import logging
import math
import re
import urllib
from urlparse import urljoin

from django.conf import settings
from django.core.cache import cache

from piston_mini_client.auth import (
    BasicAuthorizer,
    OAuthAuthorizer,
)
from piston_mini_client.failhandlers import APIError
from piston_mini_client.validators import ValidationException
from rnrclient import RatingsAndReviewsAPI
from sreclient import SoftwareCenterRecommenderAPI
from ssoclient import SingleSignOnAPI


class WebServiceError(Exception):
    """Raised when we cannot connect to a web service."""


class WebServices:

    @property
    def logger(self):
        return logging.getLogger('rnr.utilities')

    @property
    def identity_provider(self):
        auth = BasicAuthorizer(settings.SSO_API_AUTH_USERNAME,
                               settings.SSO_API_AUTH_PASSWORD)
        return SingleSignOnAPI(
            service_root=settings.SSO_API_SERVICE_ROOT, auth=auth)

    @property
    def rnr_api(self):
        return RatingsAndReviewsAPI(service_root=settings.RNR_SERVICE_ROOT)

    def get_reviews_for_package(self, package_name, distroseries, page=1):
        key = 'get_reviews_for_package-{0}-{1}-{2}'.format(
            package_name, distroseries, page)
        cached_reviews = cache.get(key)
        if cached_reviews is not None:
            return cached_reviews

        try:
            fresh_reviews = self.rnr_api.get_reviews(
                packagename=package_name, distroseries=distroseries, page=page)
        except (ValidationException, APIError):
            fresh_reviews = []
        cache.set(key, fresh_reviews, settings.REVIEWS_CACHE_TIMEOUT)
        return fresh_reviews

    def get_recommends_for_package(self, package_name):
        key = 'get_recommends_for_package-{0}'.format(package_name)
        cached_recommends = cache.get(key)
        if cached_recommends is not None:
            return cached_recommends

        try:
            fresh_recommends = self.recommender_api.recommend_app(
                pkgname=package_name)
        except APIError:
            fresh_recommends = {'package': package_name, 'data': []}
        cache.set(key, fresh_recommends)
        return fresh_recommends

    def get_screenshots_for_package(self, package_name):
        key = 'get_screenshots_for_package-{0}'.format(
            package_name)
        cached_screenshots = cache.get(key)
        if cached_screenshots is not None:
            return cached_screenshots
        url = urljoin(settings.SCREENSHOTS_BASE_URL,
                      '/json/package/{0}'.format(package_name))
        fresh_screenshots = []
        try:
            response = urllib.urlopen(url)
            if response.getcode() == 200:
                data = json.loads(response.read())
                fresh_screenshots = [entry['large_image_url']
                                     for entry in data.get('screenshots', [])]
        except (IOError, ValueError, TypeError):
            pass
        cache.set(key, fresh_screenshots, settings.REVIEWS_CACHE_TIMEOUT)
        return fresh_screenshots

    def _fake_validate_token(self, token, openid_identifier, signature):
        """ This is a version of validate_token that gets the
            token_secret, consumer_secret from the plaintext signature.

            It will break once we use a real signature for oauth, but
            its very useful for testing as it does not require the special
            priviledges required for the call to
            identity_provider.authentications.validate_token()

            The token can still be validated (and will be) with a
            api.accounts.me() call
        """
        (consumer_secret, token_secret) = urllib.unquote(signature).split("&")
        result = {}
        result["token"] = token
        result["token_secret"] = token_secret
        result["consumer_key"] = openid_identifier
        result["consumer_secret"] = consumer_secret
        result["name"] = "fake-token"
        return result

    def get_data_for_account(self, token, openid_identifier, signature=None):
        if settings.SSO_AUTH_MODE_NO_UBUNTU_SSO_PLAINTEXT_ONLY:
            # token extraction based on PLAINTEXT token, the function
            # "validate_token" is really a "get_secrets_for_token()" call
            result = self._fake_validate_token(
                token, openid_identifier, signature)
        else:
            result = self.identity_provider.validate_token(
                token=token, consumer_key=openid_identifier)

        if result:
            authorizer = OAuthAuthorizer(
                token_key=result['token'], token_secret=result['token_secret'],
                consumer_key=result['consumer_key'],
                consumer_secret=result['consumer_secret'])
            api = SingleSignOnAPI(service_root=settings.SSO_API_SERVICE_ROOT,
                                  auth=authorizer)
            me = api.me()
            data = {
                'displayname': me.displayname,
                'openid_identifier': me.openid_identifier,
                'preferred_email': me.preferred_email,
                'unverified_emails': me.unverified_emails,
                'username': me.username,
                'verified_emails': me.verified_emails,
            }
            result.update(data)
        return result

    @property
    def recommender_api(self):
        return SoftwareCenterRecommenderAPI(
            service_root=settings.REC_SERVICE_ROOT)


def full_claimed_id(consumer_key):
    return '%s/+id/%s' % (settings.OPENID_SSO_SERVER_URL.strip('/'),
                          consumer_key)


class UserAgentString(object):
    """A helper object for interrogating a user agent string for info."""
    def __init__(self, useragent_string):
        self.useragent_string = useragent_string

    @property
    def is_linux(self):
        return self.useragent_string.find('Linux') > -1

    @property
    def is_ubuntu(self):
        return self.useragent_string.find('buntu') > 0

    @property
    def distroseries(self):
        distros = settings.UBUNTU_SERIES_FOR_VERSIONS
        match = re.search(
            "buntu/(?P<version>\d\d\.\d\d)", self.useragent_string)
        if match:
            version = match.groupdict()['version']
            return distros.get(version, None)
        return None

    @property
    def linux_arch(self):
        match = re.search('Linux (?P<arch>\w+)', self.useragent_string)
        if match:
            return match.groupdict()['arch']
        return None

    def matches_appinstall_arch(self, appinstall_arch_tag):
        # An empty appinstall arch tag implies all arches.
        if appinstall_arch_tag in ('', 'any', 'all'):
            return True

        browser_arch = self.linux_arch

        if appinstall_arch_tag == 'i386':
            return browser_arch in ('i386', 'x86')

        if appinstall_arch_tag in ('ia64', 'amd64'):
            return browser_arch in ('x86_64', 'i686')

        if appinstall_arch_tag == browser_arch:
            return True

        return False


def create_png_from_file(target_path, source_file, width, height):
    import gtk
    from glib import GError
    try:
        pix = gtk.gdk.pixbuf_new_from_file_at_size(source_file, width, height)
        pix.save(target_path, "png")
        return True
    except GError:
        return False


def pnormaldist(qn):
    '''Inverse normal distribution, based on the Ruby statistics2.pnormaldist
    '''
    b = [1.570796288, 0.03706987906, -0.8364353589e-3,
         -0.2250947176e-3, 0.6841218299e-5, 0.5824238515e-5,
         -0.104527497e-5, 0.8360937017e-7, -0.3231081277e-8,
         0.3657763036e-10, 0.6936233982e-12]

    if qn < 0 or qn > 1:
        raise ValueError("qn must be between 0.0 and 1.0")
    if qn == 0.5:
        return 0.0

    w1 = qn
    if qn > 0.5:
        w1 = 1.0 - w1
    w3 = -math.log(4.0 * w1 * (1.0 - w1))
    w1 = b[0]
    for i in range(1, 11):
        w1 = w1 + (b[i] * math.pow(w3, i))

    if qn > 0.5:
        return math.sqrt(w1 * w3)
    else:
        return -math.sqrt(w1 * w3)


def wilson_score(pos, n, power=0.2):
    if n == 0:
        return 0
    z = pnormaldist(1 - power / 2)
    phat = 1.0 * pos / n
    return (phat + z * z / (2 * n) - z *
            math.sqrt((phat * (1 - phat) + z * z /
                       (4 * n)) / n)) / (1 + z * z / n)


def dampened_rating(ratings, power=0.1):
    '''Calculate the dampened rating for an app given its collective ratings'''
    if not len(ratings) == 5:
        raise AttributeError('ratings argument must be a list of 5 integers')

    tot_ratings = 0
    for i in range(0, 5):
        tot_ratings = ratings[i] + tot_ratings

    sum_scores = 0.0
    for i in range(0, 5):
        ws = wilson_score(ratings[i], tot_ratings, power)
        sum_scores = sum_scores + float((i + 1) - 3) * ws

    return sum_scores + 3
