#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Views for the Apps Directory app."""

from __future__ import absolute_import

import json
import operator
import os
from inspect import getmembers

from celery.task import Task
from convoy.combo import combine_files, parse_qs
from django.conf import settings
from django.contrib import messages
from django.core.paginator import Paginator
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import (
    HttpResponse,
    HttpResponseForbidden,
    HttpResponseRedirect,
)
from django.shortcuts import (
    get_object_or_404,
    render_to_response,
)
from django.template import RequestContext
from django.template.loader import (
    render_to_string,
    get_template,
)
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from django.views.decorators.http import require_GET
from djcelery.models import TaskState

import webcatalog.tasks
from webcatalog.forms import EmailDownloadLinkForm
from webcatalog.models import (
    Application,
    Department,
    DistroSeries,
    Exhibit,
)
from webcatalog.decorators import scheduler_required
from webcatalog.utilities import WebServices


__metaclass__ = type
__all__ = [
    'application_detail',
    'application_recommends',
    'application_reviews',
    'department_overview',
    'forbidden',
    'index',
    'schedule_task',
    'search',
]


def _get_page_num_from_request(request, paginator):
    page_num = request.GET.get('page', '1')
    if page_num.isdigit() and 1 <= int(page_num) <= paginator.count:
        return int(page_num)
    return 1


def search(request, distro=None):
    query = request.GET.get('q', '')
    if distro:
        # Attempt to retrieve the DistroSeries to ensure that it exists
        get_object_or_404(DistroSeries, code_name=distro)

    apps = []
    if 'q' in request.GET:
        terms = query.split()
        if terms:
            ors = [Q(package_name__icontains=term) for term in terms]
            ors += [Q(name__icontains=term) for term in terms]
            if distro:
                apps = Application.objects.filter(
                    distroseries__code_name=distro)
            else:
                apps = Application.objects.filter(is_latest=True)
            apps = apps.filter(reduce(operator.or_, ors))

    crumbs = [
        {'name': 'All Apps', 'url': reverse('wc-index')},
        {'name': 'Search results', 'url': request.build_absolute_uri()},
    ]

    paginator = Paginator(apps, settings.PAGE_BATCH_SIZE)
    page_num = _get_page_num_from_request(request, paginator)

    context = RequestContext(request, {
        'query': query,
        'page': paginator.page(page_num),
        'breadcrumbs': crumbs,
        'available_distroseries': DistroSeries.objects.order_by('-code_name'),
        'distroseries': distro,
    })
    return render_to_response('webcatalog/search_results.html',
                              context_instance=context)


def index(request):
    depts = Department.objects.filter(parent=None).order_by('name')
    qs = Q(display=True) | Q(display=None, published=True,)
    exhibits = list(Exhibit.objects.filter(qs).order_by('weight'))
    featured_apps = [Application.objects.find_best(package_name=app)
                     for app in settings.FEATURED_APPS]
    featured_apps = [x for x in featured_apps if x]
    apps = Application.objects.filter(wilson_score__gt=0)
    # filter out apps that are top rated more than once because
    # of different distroseries

    seen = set()
    top_rated = []
    for app in apps:
        if not app.package_name in seen:
            top_rated.append(app)
            seen.add(app.package_name)
        if len(top_rated) == settings.NUMBER_TOP_RATED_APPS:
            break

    context = RequestContext(request, {
        'depts': depts,
        'exhibits': exhibits,
        'featured_apps': featured_apps,
        'top_rated_apps': top_rated,
    })
    return render_to_response('webcatalog/index.html',
                              context_instance=context)


def department_overview(request, dept_slug_or_id=None, distro=None):
    if dept_slug_or_id.isdigit():
        dept = get_object_or_404(Department, pk=dept_slug_or_id)
    else:
        dept = get_object_or_404(Department, slug=dept_slug_or_id)
    subdepts = Department.objects.filter(parent=dept)
    subdepts = subdepts.order_by('name')

    if distro:
        # Attempt to retrieve the DistroSeries to ensure that it exists
        get_object_or_404(DistroSeries, code_name=distro)
        apps = Application.objects.filter(distroseries__code_name=distro)
    else:
        apps = Application.objects.filter(is_latest=True)
    apps = apps.filter(departments=dept)
    paginator = Paginator(apps, settings.PAGE_BATCH_SIZE)
    page_num = _get_page_num_from_request(request, paginator)

    cntxt = {
        'dept': dept,
        'subdepts': subdepts,
        'page': paginator.page(page_num),
        'breadcrumbs': dept.crumbs(),
        'available_distroseries': DistroSeries.objects.order_by('-code_name'),
        'distroseries': distro,
    }
    return render_to_response('webcatalog/department_overview.html',
                              context_instance=RequestContext(request, cntxt))


def application_list(request):
    pkg_names = request.GET.getlist('pkg_name')
    if not pkg_names:
        # We have no need of a general paged view of apps here.
        return HttpResponseRedirect(reverse('wc-index'))

    applications = []
    for pkg_name in pkg_names:
        latest = Application.objects.find_best(
            package_name=pkg_name)
        if latest:
            applications.append(latest)

    depts = Department.objects.filter(parent=None).order_by('name')

    context = RequestContext(request,
                             dict(applications=applications, depts=depts))
    return render_to_response('webcatalog/application_list.html',
                              context_instance=context)


def application_detail(request, package_name, distro=None):
    if distro is None:
        app = Application.objects.find_best_or_404(package_name=package_name)
    else:
        app = Application.objects.find_best_or_404(
            package_name=package_name,
            distroseries__code_name=distro)
    if request.POST:
        form = EmailDownloadLinkForm(request.POST)
        if form.is_valid():
            link = request.build_absolute_uri()
            form.send_email(app.name, link)
            messages.success(request, _('Success. Your download link has been'
                                        ' sent.'))
            return HttpResponseRedirect(link)
    else:
        form = EmailDownloadLinkForm()

    debtags = None if not app.debtags else json.loads(app.debtags)

    atts = {
        'application': app,
        'available_distroseries': app.available_distroseries(),
        'breadcrumbs': app.crumbs(),
        'requested_series_codename': distro,
        'absolute_url': request.build_absolute_uri(
            reverse('wc-package-detail', args=[package_name])),
        'email_form': form,
        'debtags': debtags,
    }

    return render_to_response(
        'webcatalog/application_detail.html', RequestContext(
            request, atts))


def application_reviews(request, package_name, distro, ajax=False, page=1):
    app = get_object_or_404(Application, package_name=package_name,
                            distroseries__code_name=distro)

    # XXX michaeln 2011-09-15 bug=851662 Better review language options.
    reviews = WebServices().get_reviews_for_package(
        package_name, distroseries=distro, page=page)

    context = dict(application=app, reviews=reviews)
    if ajax:
        template = 'webcatalog/application_review_list_snippet.html'
    else:
        context['distroseries'] = distro
        context['breadcrumbs'] = app.crumbs()
        template = 'webcatalog/application_review_list.html'

    return render_to_response(template, RequestContext(request, context))


def application_recommends(request, package_name):
    app = Application.objects.find_best_or_404(package_name=package_name)

    recommends = WebServices().get_recommends_for_package(package_name)

    num_recommends = 0
    recommended_apps = []
    for result in recommends['data']:
        recommended_app = Application.objects.find_best(
            package_name=result['package_name'])
        if recommended_app:
            recommended_apps.append(recommended_app)
            num_recommends += 1
            if num_recommends == settings.NUM_RECOMMENDED_APPS:
                break

    context = dict(application=app, recommended_apps=recommended_apps)
    template = "webcatalog/recommended_apps_widget.html"
    return render_to_response(template, RequestContext(request, context))


def application_screenshots(request, package_name):
    app = Application.objects.find_best_or_404(package_name=package_name)
    screenshots = WebServices().get_screenshots_for_package(package_name)
    return HttpResponse(json.dumps(screenshots), mimetype='application/json')


def combo_view(request):
    """Handle a request for combining a set of files."""
    fnames = parse_qs(request.META.get("QUERY_STRING", ""))
    content_type = "text/plain"

    if fnames:
        if fnames[0].endswith(".js"):
            content_type = "text/javascript"
        elif fnames[0].endswith(".css"):
            content_type = "text/css"
        content = combine_files(
            fnames, os.path.abspath(settings.CONVOY_ROOT),
            resource_prefix=settings.MEDIA_URL, rewrite_urls=True)
        # We're turning the generator returned by combine_files into a string
        # here since GZipMiddleware would consume it if not. See Bug #822888.
        return HttpResponse(
            content_type=content_type, status=200,
            content="".join(content))
    return HttpResponse(content_type=content_type, status=404)


@scheduler_required
@never_cache
def tasks(request):
    if request.method == 'POST':
        task_name = request.POST.get('task', '')
        task = getattr(webcatalog.tasks, task_name, None)
        if not task:
            messages.error(request, 'Invalid task name "%s"' % task_name)
            return HttpResponseRedirect(reverse('wc-tasks'))
        result = task.delay()
        messages.success(request,
                         'Task id <span>%s</span> submitted' % result.task_id)
        return HttpResponseRedirect(reverse('wc-tasks'))
    is_task = lambda x: isinstance(x, Task)
    task_list = [
        {'name': key, 'doc': val.__doc__}
        for key, val in getmembers(webcatalog.tasks, is_task)
    ]
    context = {
        'task_list': sorted(task_list),
    }
    return render_to_response('webcatalog/task_list.html',
                              RequestContext(request, context))


@scheduler_required
@require_GET
@never_cache
def task_status(request, task_id):
    try:
        task = TaskState.objects.get(task_id=task_id)
        data = {
            'id': task.task_id,
            'status': task.state,
            'traceback': task.traceback,
        }
    except TaskState.DoesNotExist:
        data = {'id': task_id, 'status': 'PENDING', 'traceback': None}
    return HttpResponse(json.dumps(data), mimetype='application/json')


def forbidden(request):
    return HttpResponseForbidden(
        render_to_string('forbidden.html', RequestContext(request)))


class ErrorPage(object):
    def __init__(self, status, errormsg=None):
        self.template_name = '%s.html' % status
        self.status_code = status
        self.errormsg = errormsg

    def __call__(self, request):
        accept = request.META.get('ACCEPT', '')
        if 'application/json' in accept and not 'text/html' in accept:
            return self.render_json_error(request)
        else:
            return self.render_html_error(request)

    def render_html_error(self, request):
        oops_id = self.get_oops_id(request)
        template = get_template(self.template_name)
        atts = {
            'oops_id': oops_id,
            'errormsg': self.errormsg,
        }
        context = RequestContext(request, atts)
        return HttpResponse(template.render(context), status=self.status_code)

    def render_json_error(self, request):
        oops_id = self.get_oops_id(request)
        oops_context = request.META.get('oops.context', {})
        exc_info = oops_context.get('exc_info', (None, None, None))
        exception_type = unicode(exc_info[0])
        response = json.dumps({
            'oops_id': oops_id,
            'error_type': exception_type,
            'error_msg': "An error occurred and has been logged.",
        })
        return HttpResponse(response, status=self.status_code,
                            content_type='application/json')

    def get_oops_id(self, request):
        oops_report = request.environ.get('oops.report', {})
        return oops_report.get('id', 'OOPS ID not available.')


server_error = ErrorPage(500)
page_not_found = ErrorPage(404)


class ArtificialOopsException(Exception):
    pass


def die(request):
    """Just trigger an oops report"""
    raise ArtificialOopsException()
