#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""wsgi stack for the Apps Directory."""

import logging.config
import oops_dictconfig
import oops_wsgi
import oops_wsgi.django
import os
import platform
import uuid

from django.conf import settings
from django.core.handlers import wsgi


class EagerOOPSWSGIHandler(wsgi.WSGIHandler):
    """Pre-generates an oops id on exception."""
    def handle_uncaught_exception(self, request, resolver, exc_info):
        # This will pass the exception information back to oops.  It
        # should not be necessary once
        # https://code.djangoproject.com/ticket/16674 has been merged.
        if 'oops.context' in request.environ:
            request.environ['oops.context']['exc_info'] = exc_info
        # Generate OOPS id early so that we can render the error page right
        # away and not depend on passing thread locals around.
        if 'oops.report' in request.environ:
            unique_id = uuid.uuid4().hex
            request.environ['oops.report']['id'] = "OOPS-%s" % unique_id

        return super(EagerOOPSWSGIHandler, self).handle_uncaught_exception(
            request, resolver, exc_info)

    def __call__(self, environ, start_response):
        def start_response_with_exc_info(status, headers, exc_info=None):
            """Custom start_response callback for wsgi."""
            # This will pass the exception information back to oops_wgi.  It
            # should not be necessary once
            # https://code.djangoproject.com/ticket/16674 has been merged.
            if exc_info is None:
                exc_info = environ['oops.context'].get('exc_info', None)
            return start_response(status, headers, exc_info)
        return super(EagerOOPSWSGIHandler, self).__call__(
            environ, start_response_with_exc_info)


def make_app():
    """Encapsulate our webcatalog handler in an OOPS app."""
    if settings.SHOULD_SERVE_HTTPS:
        os.environ['HTTPS'] = 'on'

    default_id = ''.join(x for x in platform.node() if x.isalpha())
    appserver_id = getattr(settings, 'APPSERVER_ID', default_id)
    logging_config = settings.WEBAPP_LOGGING_CONFIG
    if logging_config:
        logging.config.fileConfig(logging_config)

    non_oops_app = EagerOOPSWSGIHandler()

    config = oops_dictconfig.config_from_dict(settings.OOPSES)
    oops_wsgi.install_hooks(config)
    oops_app = oops_wsgi.make_app(non_oops_app, config,
                                  oops_on_status=['500'])
    return oops_app
